/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... CREATE.C        */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 30-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Create menu     */
/*  sub functions                                       */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
09-APR-90 MR Final cleanup for LAB5.

*\r*/

/******************************
*
* Create handler module. Create
* all sorts of nifty objects, allocate
* storage, and link them in with the
* master system structure.  Allow cancelation
* of creation of any object by simply pressing
* the right mouse button - no fuss, no muss.
*
*/

/* ---------- System include files -------------------- */


#include <stdio.h>
#include <graphics.h>
#include <alloc.h>
#include <math.h>

/* ---------- User include files ---------------------- */

#include "lab.h"
#include "mouse.h"
#include "matrix3d.h"
#include "menu.h"
#include "spclmenu.h"
#include "objdefs.h"
#include "objops.h"
#include "RADCAD.H"               /* Access to sysinfo structure */

/* ---------- Module globals -------------------------- */

#include "createv.h"            /* Create menus                 */

/* ---------- Module procedure prototpyes ------------- */

void createLine( SYSINFO_TYPE *si );

void createPolygon( SYSINFO_TYPE *si );

void createCircle( SYSINFO_TYPE *si );

void createBoxOrRect( SYSINFO_TYPE *si, int box_flag );

void createPaint( SYSINFO_TYPE *si );

void createPoints( SYSINFO_TYPE *si );




/****************************************
 * CreateHandler - Handle the display of
 * the create menu, as well as user
 * interaction for the creation of the
 * following object types:
 * CR_LINE
 * CR_POLY - Opened or auto-closed
 * CR_CIRCLE
 * CR_BOX
 * CR_RECTANGLE
 * CR_PAINT
 * CR_POINTS
 * CR_HELP
 * CR_DONE
*/

int        CreateHandler ( SYSINFO_TYPE *si )

{ /* CreateHandler begin */

int             status;
int             exitnow;

exitnow = FALSE;

do
  { /* Until exitnow == TRUE */
  status = VertMenu( MAIN_CREATE, createMenu, createSel, BK_SAVE );
  switch ( status )
    {

    case( CR_LINE ):
      createLine( si );
      break;

    case( CR_POLY ):
      createPolygon( si );
      break;

    case( CR_CIRCLE ):
      createCircle( si );
      break;

    case( CR_BOX ):
      createBoxOrRect( si, TRUE );
      break;

    case( CR_RECTANGLE ):
      createBoxOrRect( si, FALSE );
      break;

    case( CR_PAINT ):
      createPaint( si );
      break;

    case( CR_POINTS ):
      createPoints( si );
      break;

    case( CR_HELP ):
      VertMenu( MAIN_CREATE, createHelp, createHelpSel, BK_OVRWRITE );
      break;

    default:
      exitnow = TRUE;
      break;
    }
  }
while (exitnow == FALSE);

ClearStatusLine();

return( status );

} /* CreateHandler end */


/****************************************
 * Create line handler.
 *
 * Get first point.  Get second point
 * while rubber banding.  Draw each
 * line in a solid foreground color
 * if accepted by the user.  Allocate
 * the new pointer and associated object.
 * Continue until the user presses the
 * right mouse button to indicate that
 * he's done.
 */

void createLine( SYSINFO_TYPE *si )
{ /* createLine begin */

int             index;
int             exitnow;
int             button;
int             create_ok;
VIEW_COORD_TYPE sPt[2];

StatusLine("LEFT - END POINT, RIGHT - done", si->MenuColor);
do
  {
  /*----------
  * Get first point of a line
  */
  setcolor( WHITE );
  GetPoint( NULL,
            &sPt[0],
            &button,
            si->ShowMousePos );
  if (button == LEFT_BUTTON)
    {
    GetPoint( &sPt[0],
              &sPt[1],
              &button,
              si->ShowMousePos );
    if (button == LEFT_BUTTON)
      { /* Create line object */
      /*----------
      * First get an index for
      * adding the new object to
      * our list.
      */
      index = GetObjectIndex();
      if (index != GET_OBJECT_FAILED)
        {
        create_ok = CreateObject( &si->ObjectList[index],
                                  LINE_ID,
                                  LINE_POINTS,
                                  NO_SIDE_PTS,
                                  CREATE_OP );

       if (create_ok)
         {
         InizObject( si->ObjectList[index], LINE_POINTS, &sPt[0] );
         }
        }
      }
    else
      {
      exitnow = TRUE;
      }
    }
  else
    {
    exitnow = TRUE;
    }
  }
while( exitnow == FALSE );

} /* createLine end */


/****************************************
 * Create polygon handler.
 * First ask the user to open or close
 * the polygon at fini.  Then allow the
 * user to create all the points he
 * wishes until the right button is pressed.
 * If closed true, then add a point that's
 * a copy of the first point.  Then create
 * an object, blasting these points into
 * it.  The VIEW_COORD_POINTS we allocated
 * along the way, simply copy those into the
 * created structure, and free up our temporary
 * one.
*/

void createPolygon( SYSINFO_TYPE *si )
{ /* createPolygon begin */

int             status;
int             index;
int             exitnow;
int             button;
int             totalPts;
int             nomemory;
int             create_ok;
VIEW_COORD_TYPE sPt[2];
VIEW_COORD_TYPE *polyPts;


polyPts = NULL;
nomemory = FALSE;
status = VertMenu( MAIN_CREATE,
                   createPolyTy,
                   createPolyTySel, BK_SAVE );
if (status == CLOSE_THE_POLYGON)
  StatusLine("LEFT - SET POINT, RIGHT - done, close object",
              si->MenuColor);
else
  StatusLine("LEFT - SET POINT, RIGHT - done", si->MenuColor);
do
  {
  /*----------
  * Get first point of polygon
  */
  setcolor( si->ObjectColor );
  GetPoint( NULL, &sPt[0], &button, si->ShowMousePos );
  if (button == LEFT_BUTTON) /* Then start an object */
    { /* Build a connected VIEW_COORD_POINT list till done */
    polyPts = (VIEW_COORD_TYPE *)realloc( polyPts, sizeof(VIEW_COORD_TYPE));
    if (polyPts != NULL)
      {
      polyPts[0].x = sPt[0].x;
      polyPts[0].y = sPt[0].y;
      totalPts      = 1;
      exitnow = FALSE;
      }
    else
      {
      exitnow = TRUE;
      ReportNoMemory();
      nomemory = TRUE;
      }
    /*----------
    * Now get points, rubber banding along the
    * way, until the right button is pressed.
    */
    while (exitnow == FALSE)
      {
      GetPoint( (VIEW_COORD_TYPE *)&polyPts[totalPts-1],
                &sPt[0], &button, si->ShowMousePos );
      if (button == LEFT_BUTTON)
        {
        polyPts = (VIEW_COORD_TYPE *)realloc( polyPts,
                    (totalPts+1)*sizeof(VIEW_COORD_TYPE));
        if (polyPts != NULL)
          {
          polyPts[totalPts].x = sPt[0].x;
          polyPts[totalPts].y = sPt[0].y;
          totalPts++;
          }
        else
          {
          exitnow = TRUE;
          ReportNoMemory();
          nomemory = TRUE;
          }
        }
      else
        { /* button == RIGHT_BUTTON */
        exitnow = TRUE;
        }
      }
    }
  else
    exitnow = TRUE;
  }
while( exitnow == FALSE );
/* Close the object */
if ( (status == CLOSE_THE_POLYGON) &&
     (nomemory == FALSE) &&
     (totalPts >= 3) )
  {
  polyPts = (VIEW_COORD_TYPE *)realloc( polyPts,
            (totalPts+1)*sizeof(VIEW_COORD_TYPE));
  if (polyPts != NULL)
    {
    polyPts[totalPts].x = polyPts[0].x;
    polyPts[totalPts].y = polyPts[0].y;
    totalPts++;
    }
  else
    nomemory = TRUE;
  }
if (nomemory == FALSE)
  {
  if (totalPts < 3)
    {
    StatusLine("Not enough points for a polygon", si->MenuColor );
    }
  else
    {
    index = GetObjectIndex();
    if (index != GET_OBJECT_FAILED)
      {
      create_ok = CreateObject( &si->ObjectList[index],
                                POLYGON_ID,
                                totalPts,
                                NO_SIDE_PTS,
                                CREATE_OP );

      if (create_ok)
        {
        InizObject( si->ObjectList[index], totalPts, polyPts );
        }
      }
    }
  }

if (polyPts != NULL)
  free(polyPts);

} /* createPolygon end */



/****************************************
* CR_PAINT handler - create a painting
* allow the user to select paint thinner
* setting, to prevent gobbling too much
* memory.
*/

void createPaint( SYSINFO_TYPE *si )
{ /* createPaint begin */

int             index;
int             index1;
int             button;
int             totalPts;
int             create_ok;
int             thinner;        /* PAINT thinner & point control */
int             restart;
VIEW_COORD_TYPE sPt[2];
VIEW_COORD_TYPE *polyPts;

thinner = VertMenu( MAIN_CREATE,
                    createPaintThin,
                    createPaintThinSel, BK_SAVE );
switch( thinner )
  {
  case( NO_THIN ): thinner = 0; break;
  case( THIN_BY_5 ): thinner = 5; break;
  case( THIN_BY_10 ): thinner = 10; break;
  case( THIN_BY_20 ): thinner = 20; break;
  default: thinner = 0;
  }
StatusLine("LEFT-to toggle brush on/off. RIGHT- done.", si->MenuColor);
Lsetwritemode( COPY_PUT );
setcolor( si->ObjectColor );

polyPts = NULL;
do
  {
  do
    {
    /*----------
    * Wait for startup
    * button press.
    */
    GetPoint( NULL, &sPt[0], &button, si->ShowMousePos );
    if (button == LEFT_BUTTON)
      { /* Start painting */
      polyPts = (VIEW_COORD_TYPE *)realloc( polyPts, sizeof(VIEW_COORD_TYPE) );
      if (polyPts != NULL)
        {
        polyPts[0].x = sPt[0].x;
        polyPts[0].y = sPt[0].y;
        totalPts = 1;
        do
          {
          getmousecoords( &sPt[0].x, &sPt[0].y );
          if ( (button == FALSE) &&
               ( (sPt[0].x != polyPts[totalPts-1].x) ||
                 (sPt[0].y != polyPts[totalPts-1].y) ) )
            {
            polyPts = (VIEW_COORD_TYPE *)realloc( polyPts,
                      (totalPts+1) * sizeof(VIEW_COORD_TYPE) );
            if (polyPts != NULL)
              {
              if (si->ShowMousePos == TRUE)
                ReportMousePos( sPt[0].x, sPt[0].y );
              polyPts[totalPts].x = sPt[0].x;
              polyPts[totalPts].y = sPt[0].y;
              totalPts++;
              Mhidemouse();
              putpixel( sPt[0].x, sPt[0].y, si->ObjectColor );
              Mshowmouse();
              }
            else
              ReportNoMemory();
            }
          button = mousebuttonpressed(LEFT_BUTTON);
          restart = mousebuttonpressed(RIGHT_BUTTON);
          }
        while( (button == FALSE) && (polyPts != NULL) && (restart==FALSE) );
        /*----------
        * Painted, button has been released
        * time to create an object and loop.
        *
        * A PAINTed object is simply a collection
        * of POINTS, so it will have a POINT_ID
        */

        /*----------
        * If thinner != 0 then the paint thinner
        * must thin the number of points by the
        * factor specified.  Keeps the paint job
        * from gobbling up too much memory.
        */
        if (restart == FALSE)
          {
          if (thinner != 0)
            {
            Mhidemouse();
            for (index=0; index<totalPts; index++)
              putpixel( polyPts[index].x, polyPts[index].y, si->BkColor );
            Mshowmouse();
            index1 = 0;
            for (index = 0; index < totalPts; index+=thinner)
              {
              polyPts[index1] = polyPts[index];
              index1++;
              }
            polyPts[index1] = polyPts[totalPts-1];
            totalPts = index1+1;
            }
          index = GetObjectIndex();
          Lsetwritemode( COPY_PUT );
          if (index != GET_OBJECT_FAILED)
            {
            create_ok = CreateObject( &si->ObjectList[index],
                                      POLYGON_ID,
                                      totalPts,
                                      NO_SIDE_PTS,
                                      CREATE_OP );
            if (create_ok)
              InizObject( si->ObjectList[index], totalPts, polyPts );
            button = LEFT_BUTTON;
            }
          else
            {
            ReportNoMemory();
            button = RIGHT_BUTTON;
            }
          }
        else
          {
          if (totalPts > 0)
            { /* erase canceled paint area */
            Mhidemouse();
            for (index=0; index<totalPts; index++)
              putpixel( polyPts[index].x, polyPts[index].y, si->BkColor );
            totalPts = 0;
            Mshowmouse();
            }
          }
        }
      else
        {
        ReportNoMemory();
        button = RIGHT_BUTTON;
        }
      } /* end if start painting */
    }
  while( (restart == TRUE) && (button != RIGHT_BUTTON) );
  }
while( button != RIGHT_BUTTON );

if (polyPts != NULL)
  free(polyPts);

ClearStatusLine();

} /* createPaint end */



/****************************************
* CR_POINTS handler. Create multiple
* object points or a multi-point object
*
*/

void createPoints( SYSINFO_TYPE *si )

{ /* createPoints begin */

int             index;
int             exitnow;
int             button;
int             totalPts;
int             create_ok;
int             point_ctl;
VIEW_COORD_TYPE sPt[2];
VIEW_COORD_TYPE *polyPts;

setcolor( si->ObjectColor );

point_ctl = VertMenu( MAIN_CREATE,
                    createPointsHow,
                    createPointsHowSel, BK_SAVE );

StatusLine("LEFT-Add Point, RIGHT-Done", si->MenuColor );
exitnow  = FALSE;
polyPts  = NULL;
totalPts = 0;

do
  {
  GetPoint( NULL, &sPt[0], &button, si->ShowMousePos );
  if (button == LEFT_BUTTON)
    {
    if (point_ctl == OBJECT_PER_POINT)
      {
      index = GetObjectIndex();
      if (index != GET_OBJECT_FAILED)
        {
        create_ok = CreateObject( &si->ObjectList[index],
                                  POINT_ID,
                                  POINT_POINTS,
                                  NO_SIDE_PTS,
                                  CREATE_OP );
        if (create_ok)
          InizObject( si->ObjectList[index], POINT_POINTS, &sPt[0] );
        }
      else
        {
        exitnow = TRUE;
        }
      }
    else
      {
      totalPts++;
      polyPts = (VIEW_COORD_TYPE *)realloc( polyPts,
                          totalPts * sizeof(VIEW_COORD_TYPE));
      if (polyPts != NULL)
        {
        polyPts[totalPts-1].x = sPt[0].x;
        polyPts[totalPts-1].y = sPt[0].y;
        Mhidemouse();
        putpixel( sPt[0].x, sPt[0].y, si->ObjectColor );
        Mshowmouse();
        }
      else
        {
        totalPts = 0;
        exitnow = TRUE;
        }
      }
    }
  else
    exitnow = TRUE;
  }
while (exitnow == FALSE);
if ( (point_ctl == POINTS_PER_OBJECT) && (totalPts > 0) )
  {
  index = GetObjectIndex();
  if (index != GET_OBJECT_FAILED)
    {
    create_ok = CreateObject( &si->ObjectList[index],
                              POINT_ID,
                              totalPts,
                              NO_SIDE_PTS,
                              CREATE_OP );
    if (create_ok)
      InizObject( si->ObjectList[index], totalPts, polyPts );
    StatusLine("Multiple POINT object created", si->MenuColor);
    }
  }

if (polyPts != NULL)
  free(polyPts);

ClearStatusLine();

} /* createPoints end */



/****************************************
* createCircle handler
*
* Allow the user to first set a center
* for the circle, then set the radius,
* then set the final position (interactive
* translation of circle).   RIGHT_BUTTON
* cancels it at any point in the generation
* of the circle.
*/

#define SMALLEST_CIRCLE_SIZE 4

void createCircle( SYSINFO_TYPE *si )

{ /* createCircle begin */

int             button;
int             numpts;
int             create_ok;
int             index;
VIEW_COORD_TYPE center;
int             radius;
int             sel;
int             cont;
float           angle_step;
float           rad_angle_step;
float           degrees;
float           fradius;
OBJECT_TYPE     *obPtr;

sel = VertMenu( MAIN_CREATE,
                createCircleType,
                createCircleTypeSel, BK_SAVE );

cont = TRUE;
switch( sel )
  {
  case( CIRCSTEP_120 ):
    angle_step = 120.0;
    break;

  case( CIRCSTEP_72 ):
    angle_step = 72.0;
    break;

  case( CIRCSTEP_60 ):
    angle_step = 60.0;
    break;

  case( CIRCSTEP_45 ):
    angle_step = 45.0;
    break;

  case( CIRCSTEP_20 ):
    angle_step = 20.0;
    break;

  case( CIRCSTEP_10 ):
    angle_step = 10.0;
    break;

  case( CIRCSTEP_5 ):
    angle_step = 5.0;
    break;

  default:
    cont = FALSE;
    break;
  }

rad_angle_step = Radians(angle_step);

if (cont)
  {
  do
    {
    StatusLine("LFT-Center of Circle, RIT-Cancel", si->MenuColor );
    GetPoint( NULL, &center, &button, si->ShowMousePos );
    radius = SMALLEST_CIRCLE_SIZE;
    if (button != RIGHT_BUTTON)
      {
      AudioSelect();
      StatusLine("LFT-Set Circle Size, RIT-Cancel", si->MenuColor );
      button = SizeCircle( &center, &radius );
      if (button != RIGHT_BUTTON)
        {
        AudioSelect();
        StatusLine("LFT-Set Final Circle Pos, RIT-Cancel", si->MenuColor );
        button = DragCircle( &center, radius );
        if (button != RIGHT_BUTTON)
          {
          AudioSelect();
          StatusLine("Creating new Circle", si->MenuColor );
          index = GetObjectIndex();
          if (index != GET_OBJECT_FAILED)
            {
            numpts = (int)(360.0/angle_step)+1;
            create_ok = CreateObject( &si->ObjectList[index],
                                      POLYGON_ID,
                                      numpts,
                                      NO_SIDE_PTS,
                                      CREATE_OP );
            if (create_ok)
              {
              degrees    = rad_angle_step;
              fradius    = (float)radius;
              obPtr = si->ObjectList[index];
              obPtr->vCenter.x = (float)center.x;
              obPtr->vCenter.y = (float)center.y;
              obPtr->vCenter.z = 0.0;
              obPtr->vCenter.h = 1.0;
              for (index=0; index<numpts; index++)
                {
                obPtr->vPts[index].x =  obPtr->vCenter.x +
                                        fradius * cos(degrees);
                obPtr->vPts[index].y =  obPtr->vCenter.y +
                                        fradius * sin(degrees);
                MapToScreen( &obPtr->vPts[index], &obPtr->sPts[index] );
                degrees = degrees + rad_angle_step;
                }
              CenterMinMaxObject( obPtr );
              DrawObject( obPtr, si->ObjectColor );
              }
            }
          }
        }
      }
    }
  while (button != RIGHT_BUTTON);
  }

ClearStatusLine();

} /* createCircle end */



/****************************************
* createBoxOrRect handler
*
* Allow the user to first set a center
* for a box or rectangle, then set the
* size, and then set the final position
* (interactive translation of object).
* RIGHT_BUTTON cancels it at any point.
*
*
*/

#define QUADRALATERAL_POINTS 5 /* Close on itself */

void createBoxOrRect( SYSINFO_TYPE *si, int box_flag )

{ /* createBoxOrRect begin */

int             button;
int             index;
int             create_ok;
int             width;
int             height;
OBJECT_TYPE     *obPtr;
VIEW_COORD_TYPE top;
VIEW_COORD_TYPE bot;
VIEW_COORD_TYPE center;

do
  {
  if (box_flag)
    StatusLine("LFT-Center of box, RIT-Cancel", si->MenuColor);
  else
    StatusLine("LFT-Center of rectangle, RIT-Cancel", si->MenuColor);
  GetPoint( NULL, &center, &button, si->ShowMousePos );
  if (button != RIGHT_BUTTON)
    {
    StatusLine("LFT-Set size, RIT-Cancel", si->MenuColor);
    top.x = center.x-4;
    top.y = center.y-4;
    bot.x = center.x+4;
    bot.y = center.y+4;
    button = SizeBox( &top.x, &top.y, &bot.x, &bot.y,
                      box_flag, si->ObjectColor );
    if (button != RIGHT_BUTTON)
      {
      AudioSelect();
      StatusLine("LFT-Final location, RIT-Cancel", si->MenuColor);
      DragBox( top.x, top.y, bot.x, bot.y,
               &center,
               &button, si->ObjectColor );
      if (button != RIGHT_BUTTON)
        {
        if (box_flag)
          StatusLine("Creating new BOX", si->MenuColor);
        else
          StatusLine("Creating new RECTANGLE", si->MenuColor);
        index = GetObjectIndex();
        if (index != GET_OBJECT_FAILED)
          {
          create_ok = CreateObject( &si->ObjectList[index],
                                    POLYGON_ID,
                                    QUADRALATERAL_POINTS,
                                    NO_SIDE_PTS,
                                    CREATE_OP );
          if (create_ok)
            {
            width  = bot.x-top.x;
            height = bot.y-top.y;

            top.x = center.x - width/2;
            top.y = center.y - height/2;

            bot.x = center.x + width/2;
            bot.y = center.y + height/2;

            obPtr = si->ObjectList[index];
            obPtr->sPts[0].x = top.x;
            obPtr->sPts[0].y = top.y;
            obPtr->sPts[1].x = bot.x;
            obPtr->sPts[1].y = top.y;
            obPtr->sPts[2].x = bot.x;
            obPtr->sPts[2].y = bot.y;
            obPtr->sPts[3].x = top.x;
            obPtr->sPts[3].y = bot.y;
            obPtr->sPts[4].x = top.x;
            obPtr->sPts[4].y = top.y;
            for (index = 0; index < QUADRALATERAL_POINTS; index++)
              {
              obPtr->vPts[index].x = (float)obPtr->sPts[index].x;
              obPtr->vPts[index].y = (float)obPtr->sPts[index].y;
              }
            CenterMinMaxObject( obPtr );
            DrawObject( obPtr, si->ObjectColor );
            }
          }
        else
          button = RIGHT_BUTTON;
        }
      }
    }
  }
while (button != RIGHT_BUTTON);

ClearStatusLine();

} /* createBoxOrRect end */


