/********************************************************************/
/* Graphics package - 512x512 grid built in memory - dump           */
/* facility will create EPSON.OUT image which can be sent           */
/* directly to the printer.                                         */
/********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <io.h>
#include <bios.h>
#include "egraph.h"
#include "egraphp.h"

#define OPEN_ERROR -1
/********************************************************************/
/* Variable declarations: GLOBAL                                    */
/********************************************************************/
static uschar Grid[YMAX][XMAX];        /* Graphics grid */
static uschar Text[TY_MAX][TX_MAX];    /* Text overlay grid */
static uschar Header[H_LMAX][H_LWIDTH]; /* Header strings */
static int Hcount;                     /* Header line counter */
static uschar Colr;                    /* ON or OFF     */
static int   Xc, Yc;                   /* X,Y current positon */
static int   Tx, Ty;                   /* Text X, Y position  */
static int   Tctl;                     /* Text control */

/****************************************
* Local procedure declarations
*/

void DrawVert( short y);
void DrawHoriz( short x);
void eg_cpoint(short xp,short yp,short xc,short yc);
int  Fil_Len(unsigned char *s, int c_max);
int  Len(unsigned char *s);
void Place(unsigned char *s);
void find_place(unsigned char *s);
void change_dir(short *sdirx, short *sdiry);
short try_place(unsigned char *s, short x, short y, short dirx);
void EgSearch ( int mode );
void Ffputc( char byte, int stream, int printer_flag );
void Ffputs( const char *str, int stream, int printer_flag );

/********************************************************************/
/* EgDump() - Create EPSON.OUT bit image graphics dump file         */
/* of the current image in grid[y][x].                              */
/********************************************************************/

unsigned char EgDump( char *outfile )

{       /* EgDump function */

/**************
Local Variables
*/

int x,y,i,xa,l;
uschar nibs, c, mask, errf;
int stream;
char fname[120];
int printer_flag;

strcpy(fname, outfile);
fname[3] = 0;
strupr(fname);

/* Look for LPT or PRN */

printer_flag = FALSE;

if (strcmp(fname, "LPT") == 0)
  {
  printer_flag = TRUE;
  stream = outfile[3] - '1'; /* LPT1: = 0, LPT2: = 1, etc. */
  }

if ( (printer_flag == FALSE) &&
     (strcmp(fname, "PRN") == 0) )
  {
  printer_flag = TRUE;
  stream = 0; /* default printer, 0 */
  }

if (printer_flag == FALSE)
  stream = open(outfile,
                O_WRONLY|O_TRUNC|O_CREAT|O_BINARY,
                S_IREAD|S_IWRITE);


if (stream != OPEN_ERROR)
  {
  l=ioctl( stream, 0, &l, 1); /* read IOCTL settings */
  l=l | 0x0020; /* Add RAW data flag, no cookin' */
  ioctl( stream,1,&l,1);
  Ffputc(27, stream, printer_flag);
  Ffputc('@', stream, printer_flag);
  Ffputc(27, stream, printer_flag);
  Ffputc(71, stream, printer_flag);  /* double strike for text */
  Ffputc(15,stream, printer_flag);     /* Set compressed print mode            */

  for (i=0; i < H_LMAX; i++)
    Ffputs( (char *)Header[i], stream, printer_flag);

  Ffputc(27,stream, printer_flag);     /* Set vertical spacing                 */
  Ffputc('A',stream, printer_flag);
  Ffputc(8,stream, printer_flag);

  for (y=0; y < YMAX; y+=8)
    {
    Ffputs("      ",stream, printer_flag);
    Ffputc(27,stream, printer_flag);   /* Escape leadin code                   */
    Ffputc('*',stream, printer_flag);  /* Selectable dot density mode          */
    Ffputc(5,stream, printer_flag);    /* One-to-one, 72dpi mode               */
    Ffputc(0,stream, printer_flag);    /* 0 + 256*2 = 512 bytes to send        */
    Ffputc(2,stream, printer_flag);
    for (x=0; x < XMAX*8; x++)
      {  /* x for */
      nibs = 0;
      for (i=0; i < 8; i++)
        { /* i for */
        xa = ( x/8 );
        if (xa >= XMAX)
          xa = XMAX-1;
        c = Grid[y+i][xa];
        mask = 0x80 >> (x & 0x07);
        c &= mask;
        c = (c != 0) ? 0x01:0x00;
        nibs = (nibs << 1) | c;
        } /* i endfor */
      Ffputc(nibs,stream, printer_flag);
      } /* x endfor */
    Ffputc(13,stream, printer_flag);
    EgPix_to_Char(x, y);
    l = Fil_Len(&Text[Ty][0], TX_MAX);
    if (l > 0)
      { /* put_text if */
      Ffputs("      ",stream, printer_flag);
      for (i=0; i < l; i++)
        Ffputc(Text[Ty][i],stream, printer_flag);
      } /* put_text endif */
    Ffputc(13,stream, printer_flag);
    Ffputc(10,stream, printer_flag);
    } /* y endfor */
  errf = A_OK;
  Ffputc(12,stream, printer_flag);  /* Top of Form */
  Ffputc(27,stream, printer_flag);
  Ffputc('@',stream, printer_flag); /* Reset Epson */
  if (printer_flag == FALSE)
    close(stream);
  errf = (A_OK);
  }
else
  {
  errf = FILERR;
  }

return( errf );
}       /* EgDump endfunction */

/********************************************************************/
/* HpDump() - Create HP deskjet or laser jet output image           */
/* of the current image in grid[y][x].                              */
/********************************************************************/

unsigned char HpDump( char *outfile )

{       /* HpDump function */

/**************
Local Variables
*/

int x,y,i,l;
uschar c, errf;
int stream;
char fname[120];
int printer_flag;

strcpy(fname, outfile);
fname[3] = 0;
strupr(fname);

/* Look for LPT or PRN */

printer_flag = FALSE;

if (strcmp(fname, "LPT") == 0)
  {
  printer_flag = TRUE;
  stream = outfile[3] - '1'; /* LPT1: = 0, LPT2: = 1, etc. */
  }

if ( (printer_flag == FALSE) &&
     (strcmp(fname, "PRN") == 0) )
  {
  printer_flag = TRUE;
  stream = 0; /* default printer, 0 */
  }

if (printer_flag == FALSE)
  stream = open(outfile,
                O_WRONLY|O_TRUNC|O_CREAT|O_BINARY,
                S_IREAD|S_IWRITE);


if (stream != OPEN_ERROR)
  {
  l=ioctl( stream, 0, &l, 1); /* read IOCTL settings */
  l=l | 0x0020; /* Add RAW data flag, no cookin' */
  ioctl( stream,1,&l,1);
/****************************************
* Initially, BAG output of text.
* Simply blow out the bit image
* graphics for an HP deskjet or laserjet
* printer.
* ESC;"*t75R" - Set hp resolution to 75 dpi
* ESC;"*b70W" - 70 bytes of raster data follows
* ESC;"*rB"   - End of raster graphics.
*
*/

  for (i=0; i < H_LMAX; i++)
    {
    Ffputs( (char *)Header[i], stream, printer_flag);
    }

  Ffputc(27, stream, printer_flag);
  Ffputc('*', stream,printer_flag);
  Ffputc('t', stream,printer_flag);
/* for 100 dpi
  Ffputc('1', stream,printer_flag);
  Ffputc('0', stream,printer_flag);
  Ffputc('0', stream,printer_flag);
*/

/* for 75 dpi */
  Ffputc('7', stream,printer_flag);
  Ffputc('5', stream,printer_flag);

  Ffputc('R', stream,printer_flag);

  for (y=0; y < YMAX; y++)
    {
    Ffputc(27,stream,printer_flag);
    Ffputc('*', stream,printer_flag);
    Ffputc('b', stream,printer_flag);
    Ffputc('7', stream,printer_flag);
    Ffputc('0', stream,printer_flag);
    Ffputc('W', stream,printer_flag);
    /****************************************
    * Indent
    */
    Ffputc('\0', stream,printer_flag);
    Ffputc('\0', stream,printer_flag);
    Ffputc('\0', stream,printer_flag);
    Ffputc('\0', stream,printer_flag);
    Ffputc('\0', stream,printer_flag);
    Ffputc('\0', stream,printer_flag);
    for (x=0; x < XMAX; x++)
      {  /* x for */
      c = Grid[y][x];
      Ffputc(c, stream, printer_flag);
      } /* x endfor */
    } /* y endfor */
    Ffputc(27,stream,printer_flag);
    Ffputc('*', stream,printer_flag);
    Ffputc('r', stream,printer_flag);
    Ffputc('B', stream,printer_flag);
  errf = A_OK;
  Ffputc(12,stream, printer_flag);  /* Top of Form */
  if (printer_flag == FALSE)
    close(stream);
  errf = (A_OK);
  }
else
  {
  errf = FILERR;
  }

return( errf );
}       /* HpDump endfunction */


/********************************************************************/
/* EgCls() - Clear Screen and iniz Xc, Yc                           */
/********************************************************************/

void EgCls( void )

{       /* EgCls function */

for (Xc=0; Xc<XMAX; Xc++)
  for (Yc=0; Yc<YMAX; Yc++)
    Grid[Yc][Xc] = (uschar)0;

for (Xc = 0; Xc < H_LMAX; Xc++)
  strcpy( (char *)Header[Xc], "\r\n");

Hcount = 0;

for (Yc=0; Yc < TY_MAX; Yc++)
  for (Xc = 0; Xc < TX_MAX; Xc++)
    Text[Yc][Xc] = 0;


Xc = 0;
Yc = 0;

Tx = 0;
Ty = 0;


}       /* EgCls endfunction */

/********************************************************************/
/* EgColor(C) - Set color to ON or OFF for plotting                 */
/********************************************************************/

void EgColor( unsigned char c)

{       /* EgColor function */

if (c)
  Colr = ON;
else
  Colr = OFF;

}       /* EgColor endfunction */

/********************************************************************/
/* EgPosition(X, Y) - Clip and set Xc, Yc to the position requested */
/********************************************************************/

void EgPosition(short x, short y)

{       /* EgPosition function */

if (x < 0)
  x = 0;

if (y < 0)
  y = 0;

if (x >= XMAX*8)
  Xc = XMAX*8-1;
else
  Xc = x;

if (y >= YMAX)
  Yc = YMAX-1;
else
  Yc = y;

}       /* EgPosition endfunction */

/********************************************************************/
/* EgPlot(X,Y) - Set or reset the point at X,Y - based on the       */
/* current Color setting.  Upon exit Xc, Yc will be updated to      */
/* X,Y - clipped.                                                   */
/********************************************************************/

void EgPlot( short x, short y )

{       /* EgPlot Function */

/*****************
/* Local Variables
*/

usshort xbyte,ybyte,xshift;
uschar mask;

/************
Function Body
*/

EgPosition (x, y);

xbyte =(usshort)( Xc/8 );

ybyte = (usshort)Yc;

if (xbyte >= XMAX)
  xbyte = XMAX-1;

xshift =(usshort)(Xc & 0x07);     /* Shift counter        */

mask = 0x80;

if (xshift !=0)
  mask >>= xshift;

if (Colr == ON)
  Grid[ybyte][xbyte] |= mask;
else
  Grid[ybyte][xbyte] &= (~mask);

}       /* EgPlot endfunction */

/********************************************************************/
/* EgDrawto(X, Y) - draw a continuous line from Xc, Yc to           */
/* X, Y. Compute slope and offset (m and b in y=mx+b) and then      */
/* generate the points on the grid.                                 */
/********************************************************************/

void EgDrawto(short x, short y)

{       /* EgDrawto function */








/************
Function Body
*/

if (x == Xc)
  DrawVert(y);  /* Xnew = Xcurrent, then slope=infinity, vertical line*/
else  if (y == Yc)
  DrawHoriz(x); /* Ynew = Ycurrent. then slope=0, draw horizontal line*/
else
  {     /* Not VERT or HORIZ else */
  EgDraw(Xc, Yc, x, y);
  }    /* Not VERT or HORIZ endelse */

EgPosition(x, y); /* Set final x,y values properly */

}  /* EgDrawto endfunction */

/********************************************************************/
/* EgDraw a line from x1, y1 to x2, y2, all 16bit integer math      */
/* no floating point - no multiply or divides                       */
/********************************************************************/


void EgDraw(short x1, short y1, short x2, short y2)

{ /* EgDraw function */
short x, y, deltax, deltay, xstep, ystep, direction;

x = x1;
y = y1;
xstep = 1;
ystep = 1;
if (x1 > x2)
  xstep = -1;
if (y1 > y2)
  ystep = -1;
deltax = abs(x2 - x1);
deltay = abs(y2 - y1);
if (deltax == 0)
  direction = -1;
else
  direction = 0;

while ( (x != x2) || (y != y2) )
  {  /* draw while */
  EgPlot (x, y);
  if (direction < 0)
    { /* minus_dir if */
    y += ystep;
    direction += deltax;
    } /* minus_dir endif */
  else
    { /* plus_dir else */
    x += xstep;
    direction -= deltay;
    } /* plus_dir endelse */
  } /* draw endwhile */

} /* EgDraw endfunction */

/********************************************************************/
/* DrawVert( y ) - Draw a vertical line from Yc to y                */
/********************************************************************/

void DrawVert( short y)

{       /* DrawVert function */

/**************
Local Variables
*/

short ystart, yend;

/************
Function Body
*/

if (y <= Yc)
  {
  ystart = y;
  yend   = Yc;
  }
else
  {
  ystart = Yc;
  yend   = y;
  }

for (y=ystart; y<=yend; y++)
  EgPlot(Xc, y);

}       /* DrawVert endfunction */

/********************************************************************/
/* DrawHoriz(x) - Draw a horizontal line from Xc to x               */
/********************************************************************/

void DrawHoriz( short x)

{       /* DrawHoriz function */

/**************
Local Variables
*/

short xstart, xend;

if (x <= Xc)
  {
  xstart = x;
  xend   = Xc;
  }
else
  {
  xstart = Xc;
  xend   = x;
  }

for (x=xstart; x<=xend; x++)
  EgPlot(x,Yc);

}       /* EgPlotHoriz endfunction */


/********************************************************************/
/* EgFrame(x1, y1, x2, y2) - Draw a frame from top left x1, y1      */
/* through bottom right x2, y2                                      */
/********************************************************************/

void EgFrame(short x1, short y1, short x2, short y2)

{ /* EgFrame function */

EgPlot (x1, y1);
EgDrawto(x2,y1);
EgDrawto(x2,y2);
EgDrawto(x1,y2);
EgDrawto(x1,y1);

} /* EgFrame endfunction */

/********************************************************************/
/* EgCircle(x, y, r) - Draw a circle with a center at x,y of        */
/* radius r.                                                        */
/********************************************************************/

void EgCircle ( short x, short y, short r )

{ /* EgCircle function */

short point, xp, yp;

xp = 0;
yp = r;

point = 3 - (r << 1);

while (xp < yp)
  { /* circle while */
  eg_cpoint(xp, yp, x, y);
  if (point < 0)
    { /* -point if */
    point = point + ( 4 * xp) + 6;
    } /* -point endif */
  else
    { /* +point else */
    point = point + 4 * (xp - yp) + 10;
    yp--;
    } /* +point endelse */
  xp++;
  } /* circle endwhile */


} /* EgCircle endfunction */


/********************************************************************/
/* eg_cpoint(xp, yp, xc, yc) - Generate 8 symmetrical circle           */
/* points for xp, yp - with the circle centered at xc, yc           */
/********************************************************************/

void eg_cpoint(short xp,short yp,short xc,short yc)

{ /* eg_cpoint function */

EgPlot (  xc+xp, yc+yp);
EgPlot (  xc+yp, yc+xp);
EgPlot (  xc+yp, yc-xp);
EgPlot (  xc+xp, yc-yp);
EgPlot (  xc-xp, yc-yp);
EgPlot (  xc-yp, yc-xp);
EgPlot (  xc-yp, yc+xp);
EgPlot (  xc-xp, yc+yp);

} /* eg_cpoint endfunction */

/********************************************************************/
/* EgPix_to_Char(x, y) - Convert pixel X, Y coordinates to          */
/* text x, y coordinates - globals Tx & Ty.                         */
/********************************************************************/

void EgPix_to_Char(short x,short  y)

{ /* EgPix_to_Char function */

if (x< 0)
  x = 0;

if (y < 0)
  y = 0;

Tx = (61 * x) >> 8;     /* Tx = (61 * x)/256 */

Ty = y >> 3;            /* Ty = y / 8        */

if (Tx >= TX_MAX)
  Tx = TX_MAX-1;


if (Ty >= TY_MAX)
  Ty = TY_MAX-1;

} /* EgPix_to_Char endfunction */

/********************************************************************/
/*  EgGprint(x, y, s) - Print string S on the Text overlay at       */
/* x,y - pixel coordinates EXPECTED, will be translated to          */
/* valid string coordinates.  The global Tctl flag will be used     */
/* to determine if a simple text OVERLAY is to be performed, or     */
/* if a SEARCH for an open block the size of string S is to be      */
/* found in the "vacinity" of X, Y.                                 */
/********************************************************************/

void EgGprint (short x, short y, unsigned char *s)

{       /* EgGprint function */

short i;
uschar c;

EgPosition(x, y);
EgPix_to_Char(x, y);      /* Set Tx, Ty global values */

switch (Tctl)
  {     /* text_control switch */
  case (SEA):
  Place(s);     /* Place string s in Text array, search for open slot */
  break;

  default:      /* Overlay default */
  x = Tx;
  i = 0;
  while ( (x < TX_MAX) && ( (c=s[i++]) != 0) )
    Text[Ty][x++] = c;
  break;
  }     /* text_control endswitch */

} /* EgGprint endfunction */



/********************************************************************/
/* EgHprint( s ) - Place the string s in the next available entry   */
/* of the Header() string array. If none is available then do       */
/* nothing and return an error flag.                                */
/********************************************************************/

unsigned char EgHprint( char *s )

{ /* EgHprint function */

uschar ferr;

ferr = FILERR;

if (Hcount < H_LMAX)
  {
  Hcount = Hcount + 1;
  strcpy( (char *)Header[Hcount], "      " );
  strncat( (char *)Header[Hcount], s, H_LWIDTH-2 );
  strcat( (char *)Header[Hcount], "\r\n" );
  ferr = A_OK;
  }

return( ferr );

} /* EgHprint end */



/********************************************************************/
/* Fil_Len (s, c_max) - find the last null in the string s          */
/* (less than or equal to c_max-1). Return this as LEN. Then        */
/* convert all nulls in string before LEN to space characters       */
/********************************************************************/

int Fil_Len(unsigned char *s, int c_max)

{ /* Fil_Len function */

/**************
Local Variables
*/

int i, ln;

ln = -1;

for (i=c_max-1; i >= 0; i--)
  { /* Reverse search for */
  if (s[i] != 0)
    { /* end_of_string if */
    ln = i+1;
    break;
    } /* end_of_string endif */
  } /* Reverse search endfor */

if (ln > 0)    /* Convert extraneous nulls to spaces */
  { /* n_space if */
  for (i=0; i < ln; i ++)
    if (s[i] == 0)
      s[i] = 32;
  } /* n_space endif */

return(ln);

} /* Fil_Len endfunction */

/********************************************************************/
/* Len(s) - find the length of a string and return it.              */
/********************************************************************/

int Len(unsigned char *s)

{ /* Len function */

int i;

i = 0;

while (s[i]) i++;

return(i);

} /* Len endfunction */

/********************************************************************/
/* Place(s) - Globals Tx and Ty are already set to the desired      */
/* text x,y position of the string s.  If it will not fit there     */
/* (string already in place) - then search until an open slot       */
/* is found, or the outer limits of the text work area are reached  */
/* in which case the string will not be placed.  If moved           */
/* then draw a line from Xc, Yc to the final Tx,Ty location.        */
/* EgGprint POSITIONS the graphics cursor at the original pos       */
/* requested for text.  If text is moved just PLOT Xc, Yc and       */
/* then DRAWTO to the graphic equivalents of Tx, Ty.                */
/********************************************************************/

void Place(unsigned char *s)

{ /* Place function */

/**************
Local Variables
*/

int i, l;
short place;

/*************
Function Body
*/

l = Len(s);
place = TRUE;

if ((Tx + l) > TX_MAX)
  Tx=TX_MAX-l;

for (i=0; i<l; i++)
  { /* empty_check for */
  if (Text[Ty][Tx+i])
    { /* !empty if */
    place = FALSE;
    break;
    } /* !empty endif */
  } /* empty_check endfor */

if (place == TRUE)
  { /* empty if */
  for (i=0; i < l; i++)
    Text[Ty][Tx+i] = s[i];
  } /* empty endif */
else
  { /* !empty else */
  find_place(s);        /* Find a slot for string s and place it */
  } /* !empty endelse */

} /* Place endfunction */

/********************************************************************/
/* find_place(s) - find a place where string s will fit             */
/* searching, to the window limits, diagonally from the current     */
/* Tx, Ty. When placed, draw a line from the original x,y requested */
/* for the text position to the actual placement. If it wont fit    */
/* then the string is just dropped.                                 */
/********************************************************************/

void find_place(unsigned char *s)

{ /* find_place function */

/**************
Local Variables
*/

short search, xlim, sdiry, sdirx, dx, dy, try, x, y, i;
int l;

l = Len(s);

search = TRUE;
xlim   = TX_MAX-l;
sdirx  =  1;
sdiry  = -1;            /* Search direction */
dx     = 0;             /* Search displacement */
dy     = 0;

while (search == TRUE )
  { /* search_ok while */
  dx++;
  dy++;
  if ( (dy > TY_MAX) || (dx > xlim) )
    break;
  for (i=0; i<4; i++) /* Try all 4 directions at current dx, dy */
    { /* 4dir for */
    try = TRUE;
    x = Tx + sdirx*dx;
    if (sdirx < 0)
      x -= l;   /* back off by len of string */
    if ( (x < 0) || (x > xlim) )    try  = FALSE;
    y = Ty + sdiry*dy;
    if ( (y < 0) || (y > TY_MAX) )  try  = FALSE;

    if (try == TRUE)
      search = try_place(s, x, y, sdirx);

    if (search == TRUE)
      change_dir(&sdirx, &sdiry);

    if ( search == FALSE )
      break;    /* exit loop if good placement */
    } /* 4dir endfor */
  } /* search_ok endwhile */

} /* find_place endfunction */

/********************************************************************/
/* change_dir(&sdirx, &sdiry) - search pattern  1, -1               */
/*                                              1,  1               */
/*                                             -1,  1               */
/*                                             -1, -1               */
/********************************************************************/

void change_dir(short *sdirx, short *sdiry)

{ /* change_dir function */

short x,y;

x = *sdirx;
y = *sdiry;

while (1==1)
  {
  if ( (x == 1) && (y == -1) )
    { /* up-right if */
    y=1;
    break;
    } /* up-right endif */
  if ( (x == 1) && (y == 1) )
    { /* down-right if */
    x=-1;
    break;
    } /* down-right endif */
  if ( (x == -1) && (y == 1) )
    { /* down-left if */
    y=-1;
    break;
    } /* down-left endif */
  if ( (x == -1) && (y == -1) )
    { /* up-left if */
    x=1;
    break;
    } /* up-left endif */
  }

*sdiry = y;
*sdirx = x;

} /* change_dir endfunction */

/********************************************************************/
/* try_place(s, x, y) - try placing string s at x,y. If it fits     */
/* set Tx, Ty to x,y - copy string into Text[Ty][Tx] array,         */
/* and then EgPlot(Xc,Yc) and EgDrawto(Tx*256/61-2, Ty*8-4).        */
/********************************************************************/

short try_place(unsigned char *s, short x, short y, short dirx)

{ /* try_place function */

int i,l;
short search;


l      = Len(s);
search = FALSE;

for (i=0; i < l; i++)
  if (Text[y][x+i])
    search = TRUE;      /* Continue searching */

if (search == FALSE)
  { /* fits if */
  for (i=0; i < l; i++)
    Text[y][x+i] = s[i];
  Ty = y;
  Tx = x;
  EgPlot(Xc, Yc);
  x = Tx;
  if (dirx < 0)
    x += l;
  x = ( (x * 256)/61 ) - 5*dirx;
  y = 8 * Ty + 4;
  EgDrawto(x,y);
  EgDrawto(Xc+4*dirx,y);
  } /* fits endif */

return(search);

} /* try_place endfunction */

/* EgSearch ( mode ) - Enable or disable text search mode */

void EgSearch ( int mode )

{ /* EgSearch procedure */

if (mode == SEA)
  Tctl = SEA;
else
  Tctl = OLY;

} /* EgSearch end */

/****************************************
* Max X and Y coordinates for epson
* graphics output.
*/

int EgMaxx( void )
{
/* Just a constant now. Allow for variable plot sizes later */

return( GXMAX );
}

int EgMaxy( void )
{
return( GYMAX );
}

/****************************************
* Ffputc - Output a character to the opened
* file, using unix style I/O
*/

void Ffputc( char byte, int stream, int printer_flag )
{

if (printer_flag == TRUE)
  biosprint(0, byte, stream);
else
  write( stream, &byte, 1 );

}

/****************************************
* Write a string using UNIX style I/O
*/

void Ffputs( const char *str, int stream, int printer_flag )
{

int i;

if (printer_flag == TRUE)
  {
  i = 0;
  while(str[i] != 0)
    {
    biosprint(0, str[i], stream);
    i++;
    }
  }
else
  {
  i = strlen(str);
  write( stream, str, i );
  }
}

/****************************************
* Convert screen X coordinate to epson
* printer coordinate
*/

int EgScPtToPrX( int screenMaxX, int screenX )
{
int egmx;

egmx = EgMaxx();

return ( (int)( ((long)screenX * (long)egmx)/(long)screenMaxX) );

}

/****************************************
* Convert screen Y coordinate to epson
* printer coordinate
*/

int EgScPtToPrX( int screenMaxX, int screenX );

int EgScPtToPrY( int screenMaxY, int screenY )
{
int egmy;

egmy = EgMaxy();

return ( (int)( ((long)screenY * (long)egmy)/(long)screenMaxY) );
}

