/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... ERASE.C         */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 30-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Erase functions */
/*  module for LAB5                                     */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
30-MAR-90 MR Ripped out of LAB5.C, getting too large and
             complex.  Restructuring to make more modular
             and easier to debug!
09-APR-90 MR Add Erase and Cleanup in a single function.
09-APR-90 MR Final cleanup for LAB5.

*\r*/
/******************************
* Hierarchy chart for: ERASE.C
*
* Erase menu handler. Allow the
* user to delete objects selectively.
* Allow the user to delete all objects.
* Allow the user to clean up all UNDO
* memory storage, thus freeing memory
* for more new objects. Every object,
* as it is edited after creation, has
* an UNDO pointer or backup copy of its
* structure before the last operation. This
* gobbles memory, but makes it very easy to
* undo any operation.  And, when constructed
* correctly, the UNDOs can be used for animation.
* It might be possible to develop lists of UNDOs
* and perform multiple frame animation on the
* objects.  Using EGA page swapping to improve
* animation.
*/


/* ---------- System include files -------------------- */

#include <stdio.h>
#include <graphics.h>
#include <alloc.h>

/* ---------- User include files ---------------------- */

#include "lab.h"
#include "mouse.h"
#include "menu.h"
#include "objdefs.h"
#include "objops.h"
#include "RADCAD.H"               /* Access to sysinfo structure */
#include "spclmenu.h"

/* ---------- Module globals -------------------------- */

#include "erasev.h"

/* ---------- Module procedure prototpyes ------------- */

void eraseAll( SYSINFO_TYPE *si );
void eraseObjects( SYSINFO_TYPE *si );
void eraseAllCleanup( SYSINFO_TYPE *si );




/****************************************
 * EraseHandler - Present the user with
 * the following ERASE options:
 *
 * ERASE_ALL
 * ERASE_OBJECTS
 * ERASE_HELP
 * ERASE_DONE
*/

int        EraseHandler  ( SYSINFO_TYPE *si )

{ /* EraseHandler begin */

int status;
int exitnow;

exitnow = FALSE;

do
  {
  status = VertMenu( MAIN_ERASE, eraseMenu, eraseMenuSel, BK_SAVE );
  switch( status )
    {
    case( ERASE_ALL ):
      if (si->TotalActiveObjects != 0)
        eraseAll( si );
      else
        noObjects("to erase", si->ErrorColor);
      exitnow = TRUE;
      break;

    case( ERASE_ALL_CLEAN ):
      if (si->TotalObjects != 0)
        eraseAllCleanup( si );
      else
        noObjects("to erase/cleanup", si->ErrorColor );
      exitnow = TRUE;
      break;

    case( ERASE_OBJECTS ):
      if (si->TotalActiveObjects != 0)
        eraseObjects( si );
      else
        noObjects("to erase", si->ErrorColor );
      exitnow = TRUE;
      break;

    case( ERASE_HELP ):
      VertMenu( MAIN_ERASE, eraseHelp, eraseHelpSel, BK_OVRWRITE );
      break;

    default:
      exitnow = TRUE;
      break;
    }
  }
while( exitnow == FALSE );

return(status);

} /* EraseHandler end */


/****************************************
* ERASE_ALL handler - allow the user
* to change his mind before blasting
* all objects into oblivion.
*
*/

void eraseAll( SYSINFO_TYPE *si )
{ /* eraseAll begin */

int status;
int index;

SetErrorColors( );
status = VertMenu( MAIN_ERASE, eraseVerify, eraseVerifySel, BK_SAVE );
SetStdMenuColors( );

if (status == ERASE_VERIFY_YES)
  {
  if (si->TotalActiveObjects > 0)
    {
    for (index=0; index<si->TotalObjects; index++)
      {
      if (si->ObjectList[index] != NULL)
        {
        if (si->ObjectList[index]->Operation != DELETED_OP)
          {
          DeleteObject( si->ObjectList[index] );
          }
        }
      }
    si->TotalActiveObjects = 0;
    }
  }

} /* eraseAll end */


/****************************************
 * Erase one or more objects. Loop until
 * the user presses the RIGHT_BUTTON to
 * cancel the object selection process.
 */

void eraseObjects( SYSINFO_TYPE *si )

{ /* eraseObjects begin */

int status;
int button;
VIEW_COORD_TYPE ignore;

do
  { /* do until OBJECT_NOT_SELECTED */
  if (si->TotalActiveObjects == 0)
    {
    StatusLine("NO OBJECTS to select", WHITE);
    status = OBJECT_NOT_SELECTED;
    }
  else
    {
    status = SelectObject("to ERASE" );
    if (status != OBJECT_NOT_SELECTED)
      {
      FrameObject( si->ObjectList[status], TRUE );
      StatusLine("LEFT-Delete, RIGHT-Abort", WHITE);
      GetPoint( NULL, &ignore, &button, si->ShowMousePos );
      FrameObject( si->ObjectList[status], TRUE );
      if (button == LEFT_BUTTON)
        {
        DeleteObject( si->ObjectList[status] );
        si->TotalActiveObjects--;
        }
      setcolor( si->ObjectColor );
      }
    } /* end else select object */
  } /* end until OBJECT_NOT_SELECTED */
while( status != OBJECT_NOT_SELECTED );
RedrawDisplay( si->ObjectColor );

} /* eraseObjects end */



/****************************************
* Erase all objects and destroy any
* associated Undo or Unerase storage.
*
*/

void eraseAllCleanup( SYSINFO_TYPE *si )

{ /* eraseAllCleanup begin */

int     status;


SetErrorColors( );
status = VertMenu( MAIN_ERASE,
                   eraseCleanVerify,
                   eraseCleanVerifySel,
                   BK_SAVE );
SetStdMenuColors( );

if (status == ERASE_CLEAN_VERIFY)
  {
  DestroyAllObjects();
  StatusLine("All Objects erased and memory freed", si->ErrorColor );
  }

} /* eraseAllCleanup end */

