/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... FILE.C          */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 30-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... File handler    */
/*  functions for lab5.                                 */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
30-MAR-90 MR Moved from LAB5.C for modularity.
07-APR-90 MR Add genuine file SAVE and LOAD capabilities.
09-APR-90 MR Add directory list capability.
09-APR-90 MR Final LAB5 cleanup.
14-FEB-92 MR Add HP printer support.


*\r*/

/* ---------- System include files -------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <graphics.h>

/* ---------- User include files ---------------------- */

#include "lab.h"
#include "mouse.h"
#include "matrix3d.h"
#include "menu.h"
#include "spclmenu.h"
#include "objdefs.h"
#include "objops.h"
#include "RADCAD.H"               /* Access to sysinfo structure */
#include "files.h"
#include "egraph.h"
#include "egraphp.h"

/* ---------- Module constants ------------------------ */

#define FILENAMELEN 20

/* ---------- Module globals -------------------------- */

#include "filev.h"

/* ---------- Module procedure prototpyes ------------- */

void fileSave( SYSINFO_TYPE *si );

void WriteObjectFile( FILE *fPtr, SYSINFO_TYPE *si );

int writeObject( FILE *fPtr, OBJECT_TYPE *obPtr, int undo_follows );

void fileLoad( SYSINFO_TYPE *si );

void ReadObjectFile ( FILE *fPtr, SYSINFO_TYPE *si );

void MapAndScale( OBJECT_TYPE *obPtr,
                  float scalex,
                  float scaley,
                  int rescale);

char *GetFilename( const char *path,
		   const char *prompt,
		   SYSINFO_TYPE *si );


void filePrint( SYSINFO_TYPE *si );



/****************************************
 * FileHandler - Provide the user with the
 * following file options:
 *
 * FILE_SAVE
 * FILE_LOAD
 * FILE_HELP
 * FILE_DONE
 */

int        FileHandler   (SYSINFO_TYPE *si )

{ /* FileHandler begin */

int status;
int exitnow;

do
  {
  status = VertMenu( MAIN_FILE,
                     fileMenu,
                     fileMenuSel, BK_SAVE );
  switch( status )
    {
    case( FILE_SAVE ):
      fileSave( si );
      break;

    case( FILE_LOAD ):
      fileLoad( si );
      break;


    case( FILE_PRINT ):
      filePrint( si );
      break;

    case( FILE_HELP ):
      VertMenu( MAIN_FILE,
                helpFileMenu,
                helpFileMenuSel, BK_OVRWRITE );
      break;

    default:
      exitnow = TRUE;
      break;
    }
  }
while(exitnow == FALSE);

ClearStatusLine();

return( status );

} /* FileHandler end */



/****************************************
* fileSave( OBJECT_TYPE *si ) -
*
* Save the file. First ask the user for
* a filename.  Then open the file and
* save the graphics information.
*/

void fileSave( SYSINFO_TYPE *si )

{ /* fileSave begin */

char            filename[DEFAULT_STR_SIZE];
char            msg[DEFAULT_STR_SIZE];
char            *cPtr;
FILE            *fPtr;

if (si->TotalActiveObjects == 0)
  StatusLine("No objects to save", si->ErrorColor);
else
  {
  cPtr = GetFilename( "*.CAD", "Save filename?", si );
  if (cPtr != NULL)
    {
    strcpy(filename, cPtr);
    strupr(filename);
    cPtr = strchr(filename, '.');
    if (cPtr == NULL)
      strcat(filename, ".CAD");
    strcpy(msg,"Writing CAD file ");
    strcat(msg, filename);
    fPtr = fopen( filename, "wb" );

    if (fPtr != NULL)
      {
      StatusLine( msg, si->MenuColor );
      WriteObjectFile( fPtr, si );
      AudioSelect();
      }
    else
      {
      strcpy(msg, "File error, can't create: ");
      strcat(msg,filename);
      StatusLine(msg, si->ErrorColor);
      AudioError();
      }
    }
  }

} /* fileSave end */



/****************************************
* WriteObjectFile
*
* File already open, ready for output.
* Write all objects and their associated
* undo elements.
*
*/

void WriteObjectFile( FILE *fPtr, SYSINFO_TYPE *si )


{ /* WriteObjectFile begin */

int             index;
int             io_error;
OBJECT_TYPE     *obPtr;

/****************************************

The data structure for the OBJECT_TYPES is to be written.  The
file format is as follows:

int     MAXX    - Max X coord for screen resolution picture was edited in
int     MAXY    - Max Y coord for screen resolution picture was edited in
int     TotalObjects    - Total objects in file (not counting UNDO objects)

OBJECT_TYPE structure as follows:

int             Ident
int             Operation
int             NumPts
int             PtsPerSide
VIEW_COORD_TYPE sMin
VIEW_COORD_TYPE sMax
VECTOR3D_TYPE   vCenter
VIEW_COORD_TYPE sAxis[2]
VECTOR3D_TYPE   vAxis[2]
int             Undo            - If UNDO == 0, then the next record is
                                  a new object.  If UNDO == 1, then
                                  the next record is the Undo object
                                  for the current record.  It is not
                                  counted as one of the TotalObjects.
                                  It is created as a new object and
                                  then attached to the previous object's
                                  Undo pointer.

VECTOR3D_TYPE   vPts * NumPts   - Don't write sPts to file, wasteful of
                                  disk space.  Call MapToScreen as
                                  vPts are LOADed back in, to build sPts.

*/

index = getmaxx();
fwrite( &index, sizeof(int), 1, fPtr );
index = getmaxy();
fwrite( &index, sizeof(int), 1, fPtr );
fwrite( &si->TotalActiveObjects, 1, sizeof(int), fPtr );

index = 0;
io_error = FALSE;

do
  {
  if (si->ObjectList[index] != NULL)
    {
    obPtr = si->ObjectList[index];
    if (obPtr->Operation != DELETED_OP)
      {
      if (obPtr->Undo == NULL)
        io_error = writeObject( fPtr, obPtr, FALSE );
      else
        {
        io_error = writeObject( fPtr, obPtr, TRUE );
        if (io_error == FALSE)
          io_error = writeObject( fPtr, obPtr->Undo, FALSE);
        }
      }
    }
  index++;
  }
while ( (io_error == FALSE) && (index < si->TotalObjects) );

if (io_error == TRUE)
  {
  StatusLine("File write error", si->ErrorColor);
  AudioError();
  }

fclose( fPtr );

} /* WriteObjectFile end */



/****************************************
* Write an object to the opened file.
*
* The flag undo_follows is either True
* or False.
*
* Write the structure in the order shown
* below.  Return TRUE if total record write
* was successful.  Return FALSE if it failed.
*
*/

int writeObject( FILE *fPtr, OBJECT_TYPE *obPtr, int undo_follows )

{ /* writeObject begin */

int     item_count;
int     io_error;

/*
int             Ident
int             Operation
int             NumPts
int             PtsPerSide
VIEW_COORD_TYPE sMin
VIEW_COORD_TYPE sMax
VECTOR3D_TYPE   vCenter
VIEW_COORD_TYPE sAxis[2]
VECTOR3D_TYPE   vAxis[2]
int             Undo            - If UNDO == 0, then the next record is
VECTOR3D_TYPE   vPts * NumPts
*/

io_error = TRUE;
item_count = fwrite( &obPtr->Ident, sizeof(int), 1, fPtr );
if (item_count == 1)
  {
  item_count = fwrite( &obPtr->Operation, sizeof(int), 1, fPtr );
  if (item_count == 1)
    {
    item_count = fwrite( &obPtr->NumPts, sizeof(int), 1, fPtr );
    if (item_count == 1)
      {
      item_count = fwrite( &obPtr->PtsPerSide, sizeof(int), 1, fPtr );
      if (item_count == 1)
        {
        item_count = fwrite( &obPtr->sMin, sizeof(VIEW_COORD_TYPE),
                              1, fPtr);
        if (item_count == 1)
          {
          item_count = fwrite( &obPtr->sMax, sizeof(VIEW_COORD_TYPE),
                                1, fPtr);
          if (item_count == 1)
            {
            item_count = fwrite( &obPtr->vCenter, sizeof(VECTOR3D_TYPE),
                                 1, fPtr);
            if (item_count == 1)
              {
              item_count = fwrite( &obPtr->sAxis[0],
                                   sizeof(VIEW_COORD_TYPE),
                                   2, fPtr);
              if (item_count == 2)
                {
                item_count = fwrite( &obPtr->vAxis[0],
                                     sizeof(VECTOR3D_TYPE),
                                     2, fPtr);
                if (item_count == 2)
                  {
                  item_count = fwrite( &undo_follows,
                                       sizeof(int),
                                       1, fPtr);
                  if (item_count == 1)
                    {
                    item_count = fwrite( &obPtr->vPts[0],
                                         sizeof(VECTOR3D_TYPE),
                                         obPtr->NumPts, fPtr );
                    if (item_count == obPtr->NumPts)
                      io_error = FALSE;
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }

return(io_error);

} /* writeObject end */


/****************************************
* fileLoad - Load a CAD file!
*/

void fileLoad( SYSINFO_TYPE *si )

{ /* fileLoad begin */

FILE            *fPtr;
char            *cPtr;
int             sel;
int             cont;
char            filename[DEFAULT_STR_SIZE];
char            msg[DEFAULT_STR_SIZE];



if (si->TotalActiveObjects != 0)
  {
  sel = VertMenu( MAIN_FILE,
                  fileLoadMenu,
                  fileLoadMenuSel,
                  BK_SAVE );

  switch( sel )
    {
    case( LOAD_OVERWRITE ):
      DestroyAllObjects();
      cont = TRUE;
      break;

    case( LOAD_MERGE):
      cont = TRUE;
      break;

    default:
      cont = FALSE;
      break;
    }
  }
else
  {
  DestroyAllObjects();
  cont = TRUE;
  }


if (cont == TRUE)
  {
  cPtr = GetFilename( "*.CAD", "Load Filename ?", si);
  if (cPtr != NULL)
    {
    strcpy(filename, cPtr );
    strupr(filename);
    cPtr = strchr(filename, '.');
    if (cPtr == NULL)
      strcat(filename, ".CAD");
    strcpy(msg,"Reading CAD file ");
    strcat(msg, filename);
    fPtr = fopen( filename, "rb" );

    if (fPtr != NULL)
      {
      StatusLine( msg, si->MenuColor );
      ReadObjectFile( fPtr, si );
      AudioSelect();
      }
    else
      {
      strcpy(msg, "File error, can't read: ");
      strcat(msg,filename);
      StatusLine(msg, si->ErrorColor);
      AudioError();
      }
    }
  }

} /* fileLoad end */



/****************************************
* ReadObjectFile - Read new object file
* into memory.
*/

void ReadObjectFile ( FILE *fPtr, SYSINFO_TYPE *si )

{ /* ReadObjectFile begin */

int             index;
int             ident;
int             operation;
int             numpts;
int             ptsperside;
int             undo_follows;
int             exitnow;
int             file_maxx;
int             file_maxy;
int             file_totalrecords;
int             record_count;
int             size;
int             rescale;
float           scalex;
float           scaley;
int             create_ok;
OBJECT_TYPE     *obPtr;


fread( &file_maxx, sizeof(int), 1, fPtr );
fread( &file_maxy, sizeof(int), 1, fPtr );
fread( &file_totalrecords, sizeof(int), 1, fPtr );

rescale = FALSE;
scalex  = 1.0;
scaley  = 1.0;

if ( (file_maxx != (getmaxx()) ) ||
     (file_maxy != (getmaxy()) ) )
  {
  rescale = TRUE;
  scalex = ( (float)getmaxx() ) / (float)file_maxx;
  scaley = ( (float)getmaxy() ) / (float)file_maxy;
  }

exitnow      = FALSE;
record_count = 0;
do
  {
  fread( &ident, sizeof(int), 1, fPtr );
  fread( &operation, sizeof(int), 1, fPtr );
  fread( &numpts, sizeof(int), 1, fPtr );
  fread( &ptsperside, sizeof(int), 1, fPtr );
  index = GetObjectIndex();
  if (index != GET_OBJECT_FAILED)
    {
    create_ok = CreateObject(&si->ObjectList[index],
                             ident,
                             numpts,
                             ptsperside,
                             operation );
    if (create_ok)
      {
      obPtr = si->ObjectList[index];
      fread( &obPtr->sMin, sizeof(VIEW_COORD_TYPE),
             1, fPtr );
      fread( &obPtr->sMax, sizeof(VIEW_COORD_TYPE),
             1, fPtr );
      fread( &obPtr->vCenter, sizeof(VECTOR3D_TYPE),
             1, fPtr );
      fread( &obPtr->sAxis[0], sizeof(VIEW_COORD_TYPE),
             2, fPtr );
      fread( &obPtr->vAxis[0], sizeof(VECTOR3D_TYPE),
             2, fPtr );
      fread( &undo_follows, sizeof(int), 1, fPtr );
      size = fread( &obPtr->vPts[0], sizeof(VECTOR3D_TYPE),
                    numpts, fPtr );
      if (size == numpts)
        {
        MapAndScale( obPtr, scalex, scaley, rescale );
        DrawObject( obPtr, si->ObjectColor );
        record_count++;
        if (undo_follows)
          {
          fread( &ident, sizeof(int), 1, fPtr );
          fread( &operation, sizeof(int), 1, fPtr );
          fread( &numpts, sizeof(int), 1, fPtr );
          fread( &ptsperside, sizeof(int), 1, fPtr );
          create_ok = CreateObject(&obPtr->Undo,
                                   ident,
                                   numpts,
                                   ptsperside,
                                   operation );
          if (create_ok)
            {
            obPtr = obPtr->Undo;
            fread( &obPtr->sMin, sizeof(VIEW_COORD_TYPE),
                   1, fPtr );
            fread( &obPtr->sMax, sizeof(VIEW_COORD_TYPE),
                   1, fPtr );
            fread( &obPtr->vCenter, sizeof(VECTOR3D_TYPE),
                   1, fPtr );
            fread( &obPtr->sAxis[0], sizeof(VIEW_COORD_TYPE),
                   2, fPtr );
            fread( &obPtr->vAxis[0], sizeof(VECTOR3D_TYPE),
                   2, fPtr );
            fread( &undo_follows, sizeof(int), 1, fPtr );
            size = fread( &obPtr->vPts[0], sizeof(VECTOR3D_TYPE),
                          numpts, fPtr );
            if (size == numpts)
              {
              MapAndScale( obPtr, scalex, scaley, rescale );
              }
            else
              {
              StatusLine("Read error", si->ErrorColor );
              exitnow = TRUE;
              }
            } /* end if create_ok for Undo */
          else
            exitnow = TRUE; /* CreateObject for Undo failed */
          } /* end if undo_follows */
        }
      else
        {
        StatusLine("Unexpected input error", si->ErrorColor );
        fclose(fPtr);
        AudioError();
        exitnow = TRUE;
        }
      }
    else
      exitnow = TRUE; /* CreateObject failed */
    }
  else
    exitnow = TRUE; /* GetObjectIndex failed */
  }
while ( (exitnow == FALSE) && (record_count < file_totalrecords) );

fclose(fPtr);

AudioSelect();

} /* ReadObjectFile end */



/****************************************
* MapAndScale - scale the points to new
* display resolution of rescale == TRUE.
* Then map to screen.  Don't redraw since
* this module doesn't know the difference
* from a current and undo pointer.
*/

void MapAndScale( OBJECT_TYPE *obPtr,
                  float scalex,
                  float scaley,
                  int rescale)

{ /* MapAndScale begin */

int             index;


for (index = 0; index < obPtr->NumPts; index++)
  {
  if (rescale)
    {
    obPtr->vPts[index].x = obPtr->vPts[index].x * scalex;
    obPtr->vPts[index].y = obPtr->vPts[index].y * scaley;
    }
  MapToScreen( &obPtr->vPts[index], &obPtr->sPts[index] );
  }

if (rescale)
  {
  obPtr->sMin.x = (int)(scalex * (float)obPtr->sMin.x);
  obPtr->sMin.y = (int)(scaley * (float)obPtr->sMin.y);
  obPtr->sMax.x = (int)(scalex * (float)obPtr->sMin.x);
  obPtr->sMax.y = (int)(scaley * (float)obPtr->sMin.y);
  for (index=0; index<2; index++)
    {
    obPtr->vAxis[index].x = scalex * obPtr->vAxis[index].x;
    obPtr->vAxis[index].y = scaley * obPtr->vAxis[index].y;
    MapToScreen( &obPtr->vAxis[index], &obPtr->sAxis[index] );
    }
  CenterMinMaxObject( obPtr );
  }

} /* MapAndScale end */



/****************************************
* char *GetFilename( const char *path );
*
* Perform a directory search using the
* given path. Present a dialog
* allowing the user to select one of
* the inputs.  The dialog will be
* as follows:
*
* 0  Select File
* 1
* 2  prev page
* 3  FILE1.EXT
* 4  FILE2.EXT
* 5  FILE3.EXT
* 6  FILE4.EXT
* 7  FILE5.EXT
* 8  FILE6.EXT
* 9  FILE7.EXT
* 10 FILE8.EXT
* 11 FILE9.EXT
* 12 FILE10.EXT
* 13 next page
* 14
* 15 input new filename
*
* If the user selects previous page,
* the previous group of files is displayed
* if any.  If next page is selected, the
* next group of files is displayed,
* if available.  If input new filename is
* selected, then we will use StatusLineInput()
* to get a new filename from the keyboard.
*
* Return a character pointer to the filename
* the user selected.  Return a NULL if the
* dialog is cancelled or the user didn't input
* anything on the status line input.
*
*/

static char fname[80];

#define MAXFILES 16
#define PREV_PAGE_OFFSET        2
#define NEXT_PAGE_OFFSET        13
#define INPUT_NEW_OFFSET        15
#define FIRST_FILE_OFFSET       3
#define LAST_FILE_OFFSET        12


char *GetFilename( const char *path, const char *prompt, SYSINFO_TYPE *si )

{ /* GetFilename begin */

char    filesel[MAXFILES+1];
char   *filelist[MAXFILES+1];
char   **filegroup;
char   *selection;
char    nofile[2];
int     numfiles, firstfile, finx;
int     index;
int     status;
int     filecnt;
int     exitnow;

strcpy(nofile, " ");

for (index=0; index< MAXFILES; index++)
  {
  filesel[index] = 'N';
  filelist[index] = nofile;
  }

filesel[MAXFILES]       = 0;
filelist[MAXFILES]      = NULL;
selection               = NULL;

filegroup = DirectoryList( path, &numfiles );

if (numfiles == 0)
  {
  StatusLineGet("New filename: ", fname, si->MenuColor, si->MenuBkColor, 20 );
  if ( (strlen(fname)) != 0)
    selection = fname;
  else
    {
    selection = NULL;
    StatusLine("Cancel filename input", si->MenuColor);
    }
  }
else
  {
  if (numfiles > 10)
    filesel[NEXT_PAGE_OFFSET] = 'S';
  filesel[INPUT_NEW_OFFSET] = 'S';
  filelist[0] = (char *)&prompt[0];
  filelist[1] = nofile;      /* filename.ext */
  filelist[PREV_PAGE_OFFSET] = "   [prev  page]";
  filelist[NEXT_PAGE_OFFSET] = "   [next  page]";
  filelist[14] = nofile;
  filelist[INPUT_NEW_OFFSET] = " [enter new name] ";
  exitnow = FALSE;
  filecnt = 10;
  if (numfiles < 10)
    filecnt = numfiles;
  firstfile = 0;
  finx = FIRST_FILE_OFFSET;
  for (index=firstfile; index<filecnt; index++)
    {
    filelist[finx] = filegroup[index];
    filesel[finx] = 'S';
    finx++;
    }

  do
    {
    status = VertMenu( DIALOG_BOX_INDEX,
                       filelist,
                       filesel,
                       BK_SAVE );
    if (status == INPUT_NEW_OFFSET)
      {
      StatusLineGet("New filename: ", fname,
                    si->MenuColor, si->MenuBkColor,20 );
      exitnow = TRUE;
      if ( (strlen(fname)) != 0)
        selection = fname;
      else
        {
        selection = NULL;
        StatusLine("Cancel filename input", si->MenuColor);
        }
      }
    else if (status == NEXT_PAGE_OFFSET)
      {
      firstfile = filecnt;
      filecnt = filecnt + 10;
      if (filecnt > numfiles)
        {
        filecnt = numfiles;
        filesel[PREV_PAGE_OFFSET] = 'S';
        filesel[NEXT_PAGE_OFFSET] = 'N';
        }
      else
        {
        filesel[PREV_PAGE_OFFSET] = 'S';
        filesel[NEXT_PAGE_OFFSET] = 'S';
        }

      for (index=FIRST_FILE_OFFSET; index<FIRST_FILE_OFFSET+10; index++)
        {
        filelist[index] = nofile;
        filesel[index]  = 'N';
        }
      finx = FIRST_FILE_OFFSET;
      for (index = firstfile; index < filecnt; index++)
        {
        filelist[finx] = filegroup[index];
        filesel[finx] = 'S';
        finx++;
        }
      }
    else if (status == PREV_PAGE_OFFSET)
      {
      firstfile = firstfile - 10;
      filecnt = filecnt - 10;
      if (firstfile <= 0)
        {
        firstfile = 0;
        filesel[PREV_PAGE_OFFSET] = 'N';
        filecnt = numfiles;
        if (filecnt > 10)
          filecnt = 10;
        }
      if (filecnt < numfiles)
        filesel[NEXT_PAGE_OFFSET] = 'S';
      else
        filesel[NEXT_PAGE_OFFSET] = 'N';
      for (index=FIRST_FILE_OFFSET; index<FIRST_FILE_OFFSET+10; index++)
        {
        filelist[index] = nofile;
        filesel[index]  = 'N';
        }
      finx = FIRST_FILE_OFFSET;
      for (index = firstfile; index < filecnt; index++)
        {
        filelist[finx] = filegroup[index];
        filesel[finx] = 'S';
        finx++;
        }
      }
    else if ( (status >= FIRST_FILE_OFFSET) && (status <= LAST_FILE_OFFSET))
      {
      strcpy(fname, filelist[status] );
      selection = fname;
      exitnow = TRUE;
      }
    else
      {
      StatusLine("Cancel filename input", si->MenuColor);
      selection = NULL;
      exitnow = TRUE;
      }
    }
  while (exitnow == FALSE);
  }

FreeDirectoryList();

return(selection);

} /* GetFilename end */

/****************************************
* filePrint - Print the picture to the
* Epson or HP printer.
*/


void filePrint( SYSINFO_TYPE *si )

{ /* filePrint begin */

int     index;
int     status;
int     doneHeader;
int     cancelPlot;
int     filePlot;
char   *cPtr;
char    msg[80];
char    header[80];
int     printerSelect;

StatusLine("Output Image to EPSON or compatible printer", si->MenuColor);

printerSelect = VertMenu( MAIN_FILE,
                          printTypeMenu,
                          printTypeSel,
                          BK_SAVE );

status = VertMenu( MAIN_FILE,
                   printMenu,
                   printMenuSel, BK_SAVE );

cancelPlot = FALSE;
filePlot   = FALSE;

switch( status )
  {
  case ( PRINT_DISK ):
    StatusLine("File to send Epson plot to ?", si->MenuColor );
    cPtr = GetFilename( "*.BIN", "Plot filename?", si );
    if (cPtr == NULL)
      cancelPlot = TRUE;
    else
      filePlot = TRUE;
    break;

  case ( PRINT_LPT1 ):
    cPtr = "LPT1";
    break;
  case ( PRINT_LPT2 ):
    cPtr = "LPT2";
    break;
  case ( PRINT_LPT3 ):
    cPtr = "LPT3";
    break;
  case ( PRINT_CANCEL ):
    cancelPlot = TRUE;
    break;
  default:
    cancelPlot = TRUE;
  }

if (cancelPlot == FALSE)
  {
  EgCls();

  StatusLine( "Filling plotter image", si->MenuColor );

  if (si->TotalActiveObjects != 0)
    {
    for (index=0; index<si->TotalObjects; index++)
      {
      if (si->ObjectList[index] != NULL)
        if (si->ObjectList[index]->Operation != DELETED_OP)
          PlotObject( si->ObjectList[index], 1 );
      }
    }

  EgHprint( "RadCad, Revision 2.0, by Matthew J. W. Ratcliff\n");

  /******************************
  * Allow the user to input a
  * title, multiple header lines,
  * up to 12 maximum. The Epson
  * graphics printer driver
  * can handle up to 14. The first
  * two are reserved for the RadCad
  * header line and one blank line.
  */

  index = 2;
  doneHeader = FALSE;
  MessageBox
    ("Now type one or\nmore lines for Plot title.\nEnter only=DONE\n");
  do
    {
    sprintf(msg,"Plot title line %d (RTN=done) ? ", index-1 );
    StatusLineGet(msg, header, si->MenuColor, si->MenuBkColor, 30 );
    if (strlen(header) == 0)
      doneHeader = TRUE;
    else
      EgHprint( header );
    index++;
    }
  while ( (doneHeader == FALSE) && (index < H_LMAX) );

  EgFrame(0, 0, EgMaxx(), EgMaxy() );

  sprintf(msg, "Dumping image to %s", cPtr );
  StatusLine( msg, si->MenuColor );
  if (printerSelect == PRINTER_EPSON)
    {
    EgDump( cPtr );
    }
  else
    {
    HpDump( cPtr );
    }


  if (filePlot == TRUE)
    {
    sprintf( msg, "COPY %s PRN: /B\nfor hard copy", cPtr );
    MessageBox( msg );
    }
  }

} /* filePrint end */


