/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... FILES.C         */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 08-APR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Generate        */
/*  directory listing.                                  */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
09-APR-90 MR Final cleanup for LAB5

*\r*/
/******************************
* Hierarchy chart for: FILES.C
*
* Use findfirst() and findnext()
* to build a directory listing
* according to a user's specification.
* Dynamically allocate storage for
* the name strings and return a character
* pointer array to the caller, making it
* easy for him to use the filenames
* to present a directory listing in
* any manner desired.
*/

/* ---------- System include files -------------------- */

#include <dir.h>
#include <dos.h>
#include <alloc.h>
#include <string.h>

/* ---------- User include files ---------------------- */

#include "lab.h"
#include "files.h"

/* ---------- Module globals -------------------------- */

static char **fileList = NULL;
static int  files = 0;



/****************************************
* Return a pointer to an array of
* string pointers, the last being a
* null pointer.
*
* Algorithm:  Use Turbo C's 'findfirst' and
* 'findnext' functions to get a directory
* listing.  Dynamically allocate storage
* for the strings, saving their
* pointers in fileList.  Return the char **
* fileList to the caller, who may read the
* strings out at his leisure.  A filecount is
* also passed back through an integer pointer.
*
* A call to FreeDirectoryList will free the
* string storage previously allocated for the
* directory listing.
*
*/


char **DirectoryList( const char *path, int *filecount )

{ /* DirectoryList begin */

struct  ffblk ffblk;
int     done;

if (files > 0)
  FreeDirectoryList();

*filecount = 0;

done = findfirst( path, &ffblk, 0);

files = 0;
while (!done)
  {
  files++;
  fileList = (char **)realloc( fileList,
                               files * sizeof(char *) );
  if (fileList != NULL)
    {
    fileList[files-1] = (char *)malloc( 15 * sizeof(char) );
    if (fileList[files-1] != NULL)
      {
      strncpy( fileList[files-1], ffblk.ff_name, 14 );
      }
    else
      done = TRUE;
    }
  else
    done = TRUE;
  if (!done)
    done = findnext( &ffblk );
  }

*filecount = files;

return( fileList );

} /* DirectoryList end */


/****************************************
* FreeDirectoryList - If a previous call
* has been made to DirectoryList, then
* character storage has been allocated
* but not freed.  Free it up now.
*
*/


void FreeDirectoryList( void )

{ /* FreeDirectoryList end */

int index;

if (files > 0)
  {
  for (index = 0; index < files; index++)
    {
    if (fileList[index] != NULL)
      {
      free(fileList[index]);
      }
    }
  }

files = 0;

} /* FreeDirectoryList end */

