/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... LAB.C           */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 13-FEB-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Perform general */
/*  LAB project functions.                              */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
01-APR-90 MR Drop hercules support. Mouse drivers don't work
             with it, and it's just wasting memory!
07-APR-90 MR iabs() - Return the absolute value of an integer.
09-APR-90 MR Final cleanup for RADCAD.

*\r*/

/******************************
* Hierarchy chart for: LAB.C
*
* This package provides access
* to all the functions common
* to all my lab projects for
* the semester, including video
* detection.
*
* Note that VIDEOID.ASM is an
* adaptation of a utility from Microsoft's
* book on video adapters. Their's was
* written as a MEMORY MODEL DEPENDENT
* procedure - not my idea of useful. I
* rewrote it as a FUNCTION, thus it can
* be easily called from any memory model,
* even mixed memory model programs, with
* no difficulties. The information sent
* back tells you of the capabilities of the
* PRIMARY and SECONDARY (if any) graphics
* adapters in the system this program
* is running on.  It has NOT been extended
* to handle SUPER VGA. There are lots of
* those sorts of drivers available now, off
* CompuServe and the like.  But I didn't have
* time to integrate them into RadCad.
*
*/

/* ---------- System include files -------------------- */

#include <graphics.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/* ---------- User include files ---------------------- */

#include "lab.h"

/* ----------------------------------------------------
Problem: Present the user with a menu of all the
graphics modes available on his system. With all
the different graphics adapters available, this
can be a difficult task.  I found through
extensive testing that Borland's BGI drivers don't
reliably detect all modes properly. (For example,
initgraph() calls result in no errors when
selecting HERCMONO on a VGA system, when the VGA
system has NO Hercules capabilities.)  The problem
is compounded by the fact the a PC can have one or
two video adapters installed.  The user may wish
to use the alternate display for this program, if
we provide the capability.  Reliable detection of
primary and secondary video adapters, and
resolving all the video modes available is the
task of the function VideoID().  This came from
appendix C of the Microsoft Press book
"Programmer's Guide to PC and PS/2 Video
Displays".  I modified the code from what was
presented in the book, to work as a function
instead of a procedure - making one piece of code
compatible with ALL memory models available to the
Turbo C user.  See VIDEOID.H for the declaration
and VIDEOID.ASM for the actual code.
---------------------------------------------------- */

#include "videoid.h"



/*\p*********************************************/
/* Procedure:                                   */
/* SelectGraphicsMode                           */
/*                                              */
/* Inputs:                                      */
/* GRAPH_DESCR_TYPE *gdesPtr - Graphics mode    */
/* description structure to be filled.          */
/*                                              */
/* Output:                                      */
/* enum STATUS_TYPE - SUCCEEDED, in graphics    */
/* mode now, ready for plotting                 */
/* FAILED, user aborted, exit program           */
/*                                              */
/* Globals:                                     */
/* none                                         */
/*\p*********************************************/

enum STATUS_TYPE SelectGraphicsMode( GRAPH_DESCR_TYPE *gdesPtr,
                                     const char *prompt )

{ /* SelectGraphicsMode begin */

unsigned long video_modes;
VID_TYPE *vPtr;
int g_mode;
int g_driver;
int status;
int mode_selected;
int ch;
char unusedString[80];
int i;
int cga_available;
int ega_available;
int mcga_available;
int vga_available;
int graphics_available;

/*------------------------------------------------------*/
/* Detect one or two video display adapters             */
/* to determine what options to present to the user     */
/* for the graphical display.                           */
/*------------------------------------------------------*/

video_modes = VideoID();
vPtr        = (VID_TYPE *)&video_modes;

/*------------------------------------------------------*/
/* gdesPtr is expected by the caller to be filled       */
/* don't exit without initializing it to something!     */
/*------------------------------------------------------*/

gdesPtr->Max.x          = 1;
gdesPtr->Max.y          = 1;
gdesPtr->MaxColor       = 1;
gdesPtr->Driver         = DETECT;
gdesPtr->Mode           = DETECT;

/*------------------------------------------------------*/
/* Are all the expected bgi drivers available?          */
/*------------------------------------------------------*/

if ( (registerfarbgidriver(EGAVGA_driver_far) < 0) ||
     (registerfarbgidriver(CGA_driver_far)    < 0) )
  { /* if no bgi */
  printf("BGI driver driver startup error\n");
  printf("Press [ENTER] to continue ?");
  gets(unusedString);
  status = FAILED;
  } /* end if no bgi */
else
  { /* else got bgi */
  /* Initialize graphics flags    */

  status                  = SUCCEEDED;
  cga_available           = FALSE;
  ega_available           = FALSE;
  vga_available           = FALSE;
  graphics_available      = FALSE;

  for (i=0; i<2; i++)     /* Check both the active and    */
                          /* secondary video subsystems   */
    { /* for both video devices */
    switch ( vPtr->VideoSubsystem )
      { /* switch */

      /*------------------------------------*/
      /* CGA graphics available?            */
      /*------------------------------------*/
      case ( VID_CGA ):
        cga_available      = TRUE;
        graphics_available = TRUE;
        break;

      /*------------------------------------*/
      /* EGA graphics available?            */
      /*------------------------------------*/
      case ( VID_EGA ):
        ega_available      = TRUE;
        cga_available      = TRUE; /* EGA is downward compatible */
        graphics_available = TRUE;
        break;

      /*------------------------------------*/
      /* MCGA graphics available?           */
      /*------------------------------------*/
      case ( VID_MCGA ):
        mcga_available     = TRUE; /* 640x480 - 2 color         */
        cga_available      = TRUE; /* downward compatible       */
                                   /* MCGA does not have an EGA */
                                   /* compatible mode, however  */
        graphics_available = TRUE;
        break;

      /*------------------------------------*/
      /* VGA graphics available?            */
      /*------------------------------------*/
      case ( VID_VGA ):
        vga_available       = TRUE;
        ega_available       = TRUE; /* Downward compatible */
        cga_available       = TRUE; /* Downward compatible */
        graphics_available  = TRUE;
        break;

      } /* end switch */
    vPtr++;
    } /* end for */

  if (graphics_available == FALSE)
    {
    status = FAILED;
    printf("\nNo graphics capabilities detected for this system!");
    printf("Press [ENTER] to continue ? ");
    }
  else
    { /* else graphics_available */
    status = SUCCEEDED;
    /*------------------------------------*/
    /* Force normal text mode and ask     */
    /* user what graphics mode to use     */
    /*------------------------------------*/
    closegraph();
    do
      {
      clrscr();
      printf("\n%s\n\n", prompt);
      printf("\n*** Select Graphics Mode ***\n");
      printf("\nThe following modes are available in your computer:\n\n");

      if (vga_available == TRUE)
        {
        /* VGAHI */
        puts("(V)GA graphics mode 640x480, 16 color");
        }
      if (mcga_available == TRUE)
        {
        /* MCGAHII */
        puts("(M)CGA graphics mode 640x480, 2 color");
        }
      if (ega_available == TRUE)
        {
        /* EGAHI */
        puts("(E)GA graphics mode 640x350, 16 color");
        }
      if (cga_available == TRUE)
        {
        /* CGAHI*/
        puts("(C)GA graphics mode 640x200, 2 color");
        }
      printf("\nPress appropriate letter ? ");
      do
        {
        ch = getch();
        if ( (ch >= 'a') && (ch <= 'z') )
          ch = ch - 'a'+'A';
        }
      while ( (ch != 'V') && (ch != 'M') && (ch != 'E') &&
              (ch != 'C') );
      printf("%c\n\n", ch );
      mode_selected = FALSE;
      switch( ch )
        { /* switch on ch */
        case( 'V' ):
          if (vga_available == FALSE)
            puts("VGA not available");
          else
            {
            puts("Use VGA graphics mode");
            g_driver = VGA;
            g_mode   = VGAHI;
            mode_selected = TRUE;
            }
          break;
        case( 'M' ):
          if (mcga_available == FALSE)
            puts("MCGA not available");
          else
            {
            puts("Use MCGA graphics mode");
            g_driver = MCGA;
            g_mode   = MCGAHI;
            mode_selected = TRUE;
            }
          break;
        case( 'E' ):
          if (ega_available == FALSE)
            puts("EGA not available");
          else
            {
            puts("Use EGA graphics mode");
            g_driver = EGA;
            g_mode   = EGAHI;
            mode_selected = TRUE;
            }
          break;
        case( 'C' ):
          if (cga_available == FALSE)
            puts("CGA not available");
          else
            {
            g_mode = CGAHI;
            g_driver = CGA;
            mode_selected = TRUE;
            }
          break;
        default:
          puts("Unexpected input");
        } /* end switch on ch */
      puts("Press [ENTER] to continue");
      gets(unusedString);
      }
     while (mode_selected == FALSE);

    /*--------------------------*/
    /* We detected video card.  */
    /* User selected video mode.*/
    /* Now test if BGI can      */
    /* initialize it.           */
    /*--------------------------*/

    graphics_available = testGraphics( &g_driver, &g_mode );
    if (graphics_available == TRUE)
      { /* if graphics_available */
      gdesPtr->Max.x    = getmaxx();
      gdesPtr->Max.y    = getmaxy();
      gdesPtr->MaxColor = getmaxcolor();
      gdesPtr->Driver   = g_driver;
      gdesPtr->Mode     = g_mode;
      status            = SUCCEEDED;
      } /* end if graphics_available */
    else
      { /* else couldn't open graphics */
      status = FAILED;
      closegraph(); /* just to be sure */
      } /* end else couldn't open graphics */
    } /* end else graphics_available */

  } /* end else registerbgi successful */

return( status );

} /* SelectGraphicsMode end */



/*\p*********************************************/
/* Procedure:                                   */
/*  testGraphics                                */
/*                                              */
/*  Test for the availablity of the requested   */
/*  graphics driver and mode. If available,     */
/*  initialize it and return TRUE, else         */
/*  return FALSE.                               */
/*                                              */
/* Inputs:                                      */
/* int *driver - graphics driver request        */
/* int *mode   - graphics mode request          */
/*                                              */
/* Output:                                      */
/* TRUE/FALSE - available, not-available        */
/*                                              */
/* Globals:                                     */
/* none                                         */
/*\p*********************************************/

int testGraphics( int *driver, int *mode )

{ /* testGraphics begin */
int available;
int g_error;

initgraph(driver, mode, "");
g_error = graphresult();

if (g_error < 0)
  available = FALSE;
else
  available = TRUE;

return( available );
} /* testGraphics end */


/*\p*********************************************/
/* Procedure:                                   */
/* PresentTitle                                 */
/*                                              */
/* Inputs:                                      */
/* char *problemName - Name of this program     */
/* char *description[] - Program description    */
/*                                              */
/* Output:                                      */
/* none                                         */
/*                                              */
/* Globals:                                     */
/* none                                         */
/*\p*********************************************/
void PresentTitle( char *problemName, char *description[] )
{ /* PresentTitle begin */

int index;
char nullstr[80];

clrscr();

puts("The Rad CAD Project, by Matthew J. W. Ratcliff");

printf("Program: %s\n", problemName );

index = 0;

while (description[index] != NULL)
  {
  puts(description[index]);
  index++;
  }

printf("Press [ENTER] to continue ?");
gets(nullstr);

} /* PresentTitle end */

/*
void Lsetwritemode( int mode );

Lab set write mode - We cannot "getwritemode" from
Turbo C. It allows you to set it, but not GET it.
Therefore, we must write our own interface, which
saves a local copy of the write mode.

All calls in subsequent files must use Lsetwritemode
and Lgetwritemode.  If anyone bypasses these and goes
directly to setwritemode(), we lose our control of
the display.
*/

static int currentmode = COPY_PUT;

void Lsetwritemode( int mode )

{ /* Lsetwritemode begin */

currentmode = mode;

setwritemode( mode );
} /* Lsetwritemode end */

/* int Lgetwritemode( void );
 */

int Lgetwritemode( void )
{ /* Lgetwritemode begin */

return( currentmode );

} /* Lgetwritemode end */



/****************************************
* int iabs( int a );
*
* Return the absolute value of the integer a
*
*/

int iabs( int a )

{ /* iabs begin */

if (a < 0)
  return( -a );
else
  return( a );

} /* iabs end */


