/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... LAB.H           */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 31-JAN-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Standard        */
/*  header file used in all Lab projects by MJWR        */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
31-JAN-90 MR Create file of standard definitions, useful in all
              CSc343 lab assignments
09-APR-90 MR Final cleanup for LAB5.

*\r*/

/****************************************
* Define LAB, indicating this file has
* been included by a predicessor
*/

#define LAB

/*------------------------------------------------------*/
/* Status definition - codes to return from certain     */
/* routines which need only indicate if they were       */
/* successful in their required task or not.            */
/*------------------------------------------------------*/

typedef enum STATUS_TYPE { SUCCEEDED, FAILED };

/*------------------------------------------------------*/
/* The constants TRUE and FALSE are self explanitory.   */
/* In Ada and some other languages they are actually    */
/* defined as part of the language.  Not so with C.     */
/* Often one or more include files will define these    */
/* constants.  Use the #ifndef (if not defined), #endif */
/* construct to define them conditionally.              */
/*------------------------------------------------------*/

#ifndef TRUE
#define TRUE    1
#endif

#ifndef FALSE
#define FALSE   0
#endif

/*------------------------------------------------------*/
/* Console I/O related constants                        */
/*------------------------------------------------------*/


#define DEFAULT_STR_SIZE 80     /* Default string size */

#define BEEP            (char)7

#define ESC             27
#define CR              13
#define LF              10
#define BS              8
#define SPACE           32

/*------------------------------------------------------*/
/* Menu interaction and other interactive keys          */
/*------------------------------------------------------*/

#define UP_ARROW        72
#define DN_ARROW        80
#define LEFT_ARROW      75
#define RIGHT_ARROW     77
#define HOME            71
#define END             79
#define PAGE_UP         73
#define PAGE_DN         81
#define COMMAND_KEY     0

/*------------------------------------------------------*/
/* Define a convenient structure for the description    */
/* of a single point, called a VIEW_COORD_TYPE          */
/* Our view display device always uses integer values   */
/* for coordinates.                                     */
/*------------------------------------------------------*/

typedef struct VIEW_COORD_TYPE
        {
        int     x;
        int     y;
        } VIEW_COORD_TYPE;

/*----------------------------------------------
Define a structure which describes the current
graphics mode in use.  Turbo C provides function
calls to provide the max X, max Y, and max Color
values.  However, repeating these function calls
are wasteful of cpu time and unnecessary. The
GRAPH_DESCR_TYPE structure is filled by the
SelectGraphicsMode() procedure.  Use it for all
scaling and color selections.
-------------------------------------------------*/

typedef struct GRAPH_DESCR_TYPE
        {
        VIEW_COORD_TYPE Max;            /* Max.x and Max.y              */
        int             MaxColor;       /* Max colors in this mode      */
        int             Driver;         /* Graphics driver in use       */
        int             Mode;           /* Graphics mode selected       */
        } GRAPH_DESCR_TYPE;
/*------------------------------------------------------*/
/* Define a convenient structure for the description    */
/* of a single point, called a WORLD_COORD_TYPE.  This  */
/* will be used to represent points in terms of the     */
/* "real world".                                        */
/*------------------------------------------------------*/

typedef struct WORLD_COORD_TYPE
        {
        float   wx;
        float   wy;
        } WORLD_COORD_TYPE;


/*------------------------------------------------------*/
/* These functions will be used in all my LAB           */
/* assignments.                                         */
/*------------------------------------------------------*/


/*------------------------------------------------------*/
/* SelectGraphicsMode - Determine all the graphics      */
/* modes available on the machine this program is       */
/* executed on.  Present these to the user and          */
/* allow him to select one.  Fill the GRAPH_DESCR_TYPE  */
/* structure with the necessary elements for scaling    */
/* and color mapping.  Return the enumerated STATUS_TYPE*/
/* as SUCCEEDED or FAILED.  If failed, graphics cannot  */
/* be run, so exit gracefully from the program.  Else   */
/* set the graphics mode before return, and continue    */
/* with the interactive graphics exercise.              */
/*------------------------------------------------------*/
/* 07-FEB-90 - Add prompt character string to select	*/
/* graphics mode, makes it more informative.		*/
/*------------------------------------------------------*/

enum STATUS_TYPE SelectGraphicsMode( GRAPH_DESCR_TYPE *gdesPtr,
                                     const char *prompt );


/*------------------------------------------------------*/
/* Support routine for SelectGraphicsMode to test       */
/* for the user's selected graphics mode, and           */
/* initialize it if possible. Return TRUE if the        */
/* requested driver/mode are available, FALSE if not.   */
/*------------------------------------------------------*/

int testGraphics( int *driver, int *mode );

/*------------------------------------------------------*/
/* Each lab assignment must present the name of the     */
/* program, author, and an assignment description.      */
/* The problemName will simply be the name of this      */
/* program.  The description will be an array of        */
/* character pointers, the last line being a null.      */
/* This allows me to define a nice description of the   */
/* program at the top of each assignment, and simply    */
/* point to that in the PresentTitle call. No status    */
/* is returned, since it is assumed that any PC can     */
/* handle the printf() function with no errors.         */
/*------------------------------------------------------*/

void PresentTitle( char *problemName, char *description[] );


/*
 * LAB set and get write mode functions. Turbo C allows
 * you to set the write mode, but not get it.  We need
 * to be able to get it, to save it, before outputting
 * certain messages.  During message output, we don't
 * necessarily know if the application is using XOR
 * or COPY write modes.  So Lsetwritemode() saves the
 * current value, and Lgetwritemode() returns the last
 * mode set, to provide this functionality.
 */

void Lsetwritemode( int mode );
int Lgetwritemode( void );

/****************************************
* int iabs( int a );
*
* Return the absolute value of the integer a
*
*/

int iabs( int a );
