/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... MATRIX3D.C      */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 13-FEB-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Code file for   */
/*  general 3d  matrix operations & structures.         */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
09-APR-90 MR Final cleanup for LAB5.

*\r*/
/******************************
*
* By using homogeneous coordinates
* to represent points for the
* objects in this system (that is (x,y,z,h),
* where h is a scaling value), we can use
* matrix MULTIPLICATION to perform
* ALL transformations including:
* Scale, Translate, and Rotate.
* It is also possible to perform a
* TRANSLATE, SCALE, ROTATE, TRANSLATE,
* with a single matrix multiply, for example.
* We build a composite transformation matrix
* by constructing individual transformation
* matrices, then multiplying them to build
* a composite matrix that does all the operations
* in a single step. Then we have a single vector-matrix
* multiply per data point instead of many. Optimizations
* have been made where possible. But this
* package is general enough to enable you to perform
* just about any geometric transformation on
* graphics data of the format specified in MATRIX3D.H.

*/


/* ---------- System include files -------------------- */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <mem.h>

/* ---------- User include files ---------------------- */

#include "matrix3d.h"

/* ---------- Module globals -------------------------- */

VECTOR3D_ARRAY_TYPE *wvPtr;     /* Working vector 3d pointer    */
                                /* for 1d array access of a     */
                                /* vector.                      */
VECTOR3D_ARRAY_TYPE *wv2Ptr;    /* Working vector 3d pointer    */
                                /* for 1d array access of a     */
                                /* vector.                      */
VECTOR3D_ARRAY_TYPE *wrPtr;     /* Working vector 3d pointer    */
                                /* for 1d array access of a     */
                                /* vector. Working result pointer*/

/* ---------- Module procedure prototpyes ------------- */

#define MK3D(A) wvPtr = (VECTOR3D_ARRAY_TYPE *)(A)
#define MK3D2(A) wv2Ptr = (VECTOR3D_ARRAY_TYPE *)(A)
#define MK3DR(B) wrPtr= (VECTOR3D_ARRAY_TYPE *)(B)



/****************************************
 * Procedure:
 * Radians - Convert an angle expressed in
 * degrees to radians.
 *
 * Inputs:
 * float deg_angle - Angle in degrees to
 * convert
 *
 * Output:
 * float - Angle in radians
 *
 * Globals:
 * none
 */

float Radians( float deg_angle )

{ /* Radians begin */

float rad_angle;

rad_angle = PI * deg_angle / PI_DEGREES;

return( rad_angle );

} /* Radians end */


/****************************************
 * Procedure:
 * Degrees - Convert an angle expressed in
 * radians to degrees.
 *
 * Inputs:
 * float rad_angle - Angle measure in radians
 *
 * Output:
 * float - Angle measure in degrees
 *
 * Globals:
 * none
 */

float Degrees( float rad_angle )

{ /* Degrees begin */

float deg_angle;

deg_angle = PI_DEGREES * rad_angle / PI;

return( deg_angle );

} /* Degrees end */

/****************************************
 * Zero the matrix.  This is done with
 * the "memset" function in C, simply
 * filling that block of memory with all
 * zeros, forcing all floats in the array
 * to zero. FASTEST way to fill a matrix
 * with zeros.
 */

void Zero3d ( MATRIX3D_TYPE *mat )

{ /* Zero3d begin */

memset( (void *)mat, 0, sizeof(MATRIX3D_TYPE) );

} /* Zero3d end */

/****************************************
 * This function will print a 4x4 matrix
 * to the screen.  It generally isn't
 * used for the operations we need in
 * interactive 3d graphics, but is
 * useful in debugging
 */

void MatPrint3d( MATRIX3D_TYPE *m )

{ /* MatPrint3d begin */

int row, col;

printf("[ ");
for (row=0; row<4; row++)
  {
  if (row != 0)
    printf("  ");
  printf("[ ");
  for(col=0; col<4; col++)
    {
    printf(" %f", m->Mat[row][col]);
    if (col<3)
      printf(",");
    else
      printf(" ]");
    }
  if (row < 3)
    printf("\n");
  }

printf(" ]\n");

} /* MatPrint3d end */

/****************************************
 * This function will print a 1x4 vector
 * on the screen.  It generally isn't
 * used for the operations we need in
 * interactive 3d graphics, but is useful
 * in debugging.
 */

void VectPrint3d( VECTOR3D_TYPE *v )

{ /* VectPrint3d begin */

int i;

MK3D(v);   /* Macro - make wvPtr a an array access copy of v */

printf("[");
for (i=0; i<4; i++)
  {
  printf(" %f", wvPtr->Vect[i] );
  if (i < 3)
    printf(",");
  }

printf(" ]\n");

} /* VectPrint3d end */


/****************************************
 * This function will generate an identity
 * matrix for the caller.
 */

void Identity3d( MATRIX3D_TYPE *m )
{ /* Identity3d begin */

Zero3d( m ); /* Zero it out first */

m->Mat[0][0] = m->Mat[1][1] = m->Mat[2][2] = m->Mat[3][3] = (float)1.0;

} /* Identity3d end */

/****************************************
 * This function will transpose a 4x4
 * matrix, useful for inverting certain
 * rotation matrices.
 */

void Transpose3d( MATRIX3D_TYPE *m )
{ /* Transpose3d begin */

int row, col;
MATRIX3D_TYPE t;

for (col=0; col<4; col++)
  for (row=0; row<4; row++)
    t.Mat[row][col] = m->Mat[col][row];

Dup3d( m, &t );

} /* Transpose3d end */

/****************************************
 * Duplicate a 4x4 matrix.  Use to keep
 * a backup copy of a matrix for some
 * graphical operations which we might
 * want to undo.
 */

void Dup3d( MATRIX3D_TYPE *dest, MATRIX3D_TYPE *src )
{ /* Dup3d begin */

memcpy( dest, src, sizeof( MATRIX3D_TYPE ) );

} /* Dup3d end */

/****************************************
 * Set one of the elements of the 4x4
 * matrix.  User specifies column, row,
 * and new value for that element.
 */

void SetElement3d( MATRIX3D_TYPE *m,  int row, int col, float element )

{ /* SetElement3d begin */

m->Mat[row][col] = element;

} /* SetElement3d end */

/****************************************
 * Set an entire row of a matrix by
 * copying the vector into the specified
 * row.
 */

void SetRow3d( MATRIX3D_TYPE *m, VECTOR3D_TYPE *v, int row )

{ /* SetRow3d begin */

int col;

MK3D(v);   /* Macro - make wvPtr a an array access copy of v */

for (col=0; col<4; col++)
  m->Mat[row][col] = wvPtr->Vect[col];

} /* SetRow3d end */



/****************************************
 * Set an entire column of a matrix by
 * copying the vector into the
 * specified column.
 */

void SetCol3d( MATRIX3D_TYPE *m, VECTOR3D_TYPE *v, int col )

{ /* SetCol3d begin */

int row;

MK3D(v);   /* Macro - make wvPtr a an array access copy of v */

for (row=0; row<4; row++)
  m->Mat[row][col] = wvPtr->Vect[row];

} /* SetCol3d end */



/****************************************
 * VectMatMult3d - Multiply the specified
 * matrix by the specified vector, and
 * place results in user supplied output
 * vector.
 */

void VectMatMult3d( VECTOR3D_TYPE *v, MATRIX3D_TYPE *m,
                    VECTOR3D_TYPE *res )

{ /* VectMatMult3d begin */

int row, col;
float entry;
float m1, v1;


/****************************************
* UNION's would be more efficient, but
* this works just fine and is easier
* for me to follow.
*/

MK3D(v);     /* MACRO - make wvPtr an array access to v      */
MK3DR(res);  /* MACRO - make wrPtr an array access to res    */

/****************************************
 *
 * |v0,v1,v2,v3| * | m00 m01 m02 m03 |
 *                 | m10 m11 m12 m13 |
 *                 | m20 m21 m22 m23 |
 *                 | m30 m31 m32 m33 |
 *
 * = | (v0*m00 + v1*m01 + v2*m02 + v3*m03),
 *     (v0*m10 + v1*m11 + v2*m12 + v3*m13),
 *     (v0*m20 + v1*m21 + v2*m22 + v3*m23),
 *     (v0*m30 + v1*m31 + v2*m32 + v3*m33),
 */

/****************************************
 * Optimization:  As each partial sum is
 * computed test to see if both elements
 * of the multiplication are nonzero. If
 * either is zero, don't multiply and add.
 * If both are nonzero, and one of the
 * entries is 1.0, then don't multiply,
 * simply copy the other element into the
 * entry to be added.  If both are nonzero
 * and not equal to 1.0, then perform
 * the relatively LONG floating point
 * multiplication.  The floating point
 * comparisons are time consuming too,
 * however many of our 3d matrices are
 * rather sparse with lots of zeros and
 * 1's, so this will pay off by comparison
 * to a general purpose vector/matrix
 * multiplication.
 */


for(col=0; col<4; col++)
  {
  entry = 0.0;
  for (row=0; row<4; row++)
    {
    m1 = m->Mat[row][col];
    v1 = wvPtr->Vect[row];
    if ( (v1 != 0.0) && (m1 != 0.0) )
      {
      if (m1 == 1.0)
        entry = entry + v1;
      else if (v1 == 1.0)
        entry = entry + m1;
      else
        entry = entry + m1*v1;
      }
    }
  wrPtr->Vect[col] = entry;
  }

} /* VectMatMult3d end */


/****************************************
 * RotMatRad3dZ( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the Z axis of r radians.
 */

void RotMatRad3dZ( MATRIX3D_TYPE *m, float r )
{ /* RotMatRad3dZ begin */

/****************************************
 * Rz(r) = |  cos(r)  sin(r)  0  0 |
 *         | -sin(r)  cos(r)  0  0 |
 *         |    0       0     1  0 |
 *         |    0       0     0  1 |
 *
 * Convert m to a zero matrix for starters,
 * then set the proper elements as shown
 * above.
 */

float c, s;

Identity3d( m );

c = cos(r);
s = sin(r);

/* Column 0 */

m->Mat[0][0] =  c;
m->Mat[1][0] = -s;

/* Column 1 */

m->Mat[0][1] =  s;
m->Mat[1][1] =  c;

} /* RotMatRad3dZ end */



/****************************************
 * RotMatRad3dX( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the X axis of r radians.
 */



void RotMatRad3dX( MATRIX3D_TYPE *m, float r )

{ /* RotMatRad3dX begin */

/****************************************
 * Rx(r) = | 1    0      0     0 |
 *         | 0  cos(r) sin(0)  0 |
 *         | 0 -sin(r) cos(0)  0 |
 *         | 0    0      0     1 |
 *
 * Convert m to a zero matrix for starters,
 * then set the proper elements as shown
 * above.
 */

float c, s;

Identity3d( m );

c = cos(r);
s = sin(r);

/* Column 1 */

m->Mat[1][1] =  c;
m->Mat[2][1] = -s;

/* Column 2 */

m->Mat[1][2] =  s;
m->Mat[2][2] =  c;

} /* RotMatRad3dX end */


/****************************************
 * RotMatRad3dY( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the Y axis of r radians.
 */


void RotMatRad3dY( MATRIX3D_TYPE *m, float r )

{ /* RotMatRad3dY begin */

/****************************************
 * Ry(r) = |  cos(r)  0  -sin(r)  0 |
 *         |    0     1     0     0 |
 *         |  sin(r)  0   cos(r)  0 |
 *         |    0     0     0     1 |
 *
 * Convert m to a zero matrix for starters,
 * then set the proper elements as shown
 * above.
 */

float c, s;

Identity3d( m );

c = cos(r);
s = sin(r);

/* Column 0 */

m->Mat[0][0] =  c;
m->Mat[2][0] =  s;

/* Column 2 */

m->Mat[0][2] = -s;
m->Mat[2][2] =  c;

} /* RotMatRad3dY end */


/****************************************
 * RotMatDeg3dZ( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the 'N' axis of d degrees.
 */


void RotMatDeg3dZ( MATRIX3D_TYPE *m, float d )
{ /* RotMatDeg3dZ begin */

/****************************************
 * The algorithm for generating the
 * rotation matrices for degrees is identical
 * to that for radians.  In fact, we first
 * make a local float which becomes
 * the radians equivalent to the angle
 * in degrees.  We then make a subsequent
 * call to the radian rotation procedure.
 * This allows the user to work in degrees
 * all the time, without mucking about with
 * degrees/radians conversions...however working
 * in degrees is a bit slower because of the
 * additional conversion overhead.
 */

float r;

r = Radians( d );

RotMatRad3dZ( m, r );

} /* RotMatDeg3dZ end */



/****************************************
 * RotMatDeg3dX( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the X axis of d degrees.
 */



void RotMatDeg3dX( MATRIX3D_TYPE *m, float d )

{ /* RotMatDeg3dX begin */

/****************************************
 * The algorithm for generating the
 * rotation matrices for degrees is identical
 * to that for radians.  In fact, we first
 * make a local float which becomes
 * the radians equivalent to the angle
 * in degrees.  We then make a subsequent
 * call to the radian rotation procedure.
 * This allows the user to work in degrees
 * all the time, without mucking about with
 * degrees/radians conversions...however working
 * in degrees is a bit slower because of the
 * additional conversion overhead.
 */

float r;

r = Radians( d );

RotMatRad3dX( m, r );

} /* RotMatDeg3dX end */


/****************************************
 * RotMatDeg3dY( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the Y axis of d degrees.
 */



void RotMatDeg3dY( MATRIX3D_TYPE *m, float d )

{ /* RotMatDeg3dY begin */

/****************************************
 * The algorithm for generating the
 * rotation matrices for degrees is identical
 * to that for radians.  In fact, we first
 * make a local float which becomes
 * the radians equivalent to the angle
 * in degrees.  We then make a subsequent
 * call to the radian rotation procedure.
 * This allows the user to work in degrees
 * all the time, without mucking about with
 * degrees/radians conversions...however working
 * in degrees is a bit slower because of the
 * additional conversion overhead.
 */

float r;

r = Radians( d );

RotMatRad3dY( m, r );

} /* RotMatDeg3dY end */



/****************************************
 * Mult3d( MATRIX3D_TYPE *s1,
 *         MATRIX3D_TYPE *s2,
 *         MATRIX3D_TYPE *res );
 *
 * Matrix multiplication:
 *
 * res = s1 * s2;
 *
 * Remember, matrix multiplication is NOT
 * commutative, so the caller must be careful
 * to specify the sources, s1, and s2, in
 * the desired sequence of multiplication!
 *
 */

void  Mult3d( MATRIX3D_TYPE *s1,
              MATRIX3D_TYPE *s2,
              MATRIX3D_TYPE *res )

{ /* Mult3d begin */

int dc, dr, r;
float sm1, sm2; /* Make local copies before operating   */
                /* Eliminates indexing arithmetic       */
                /* for multiple tests and operations    */

MATRIX3D_TYPE result;

/****************************************
 * Optimized matrix multiply.
 * If an entry is zero, don't bother
 * with wasting a multiply and add.
 * If either entry is 1.0, don't bother
 * with multiplying, just add the other
 * entry.  If both are nonzero and not
 * equal to 1.0, then perform the multiply
 * and add.  In the most general sense
 * this will slow things down.  However,
 * for 3d operations and homogeneous
 * coordinates, we'll find LOTS of 1's and
 * 0's in our transformation matrices.
 * This is matrix multiply is optimized
 * for these operations.
 *
 * The output of this matrix multiply was
 * verified with MathCad 2.0 and an
 * HP28s calculator on 24-MAR-90.
 *
 */


float entry;

for (dr=0; dr<4; dr++)
  {
  for (dc=0; dc<4; dc++)
    {
    entry = 0.0;
    for (r=0; r<4; r++)
      {
      sm1 = s1->Mat[dr][r];
      sm2 = s2->Mat[r][dc];
      if ( (sm1 != 0.0) && (sm2 != 0.0) )
        {
        if ( sm1 == 1.0 )
          entry = entry + sm2;
        else if (sm2 == 1.0)
          entry = entry + sm1;
        else
          entry = entry + sm1*sm2;
        }
      }
    result.Mat[dr][dc] = entry;
    }
  }

/****************************************
* Note that we perform the multiplication
* into a local MATRIX3D_TYPE, then copy
* it into the result pointer passed in.
*
* This allows the caller to perform
* multiple Mult3d operations, to build
* a composite transform, for example:
*
* Mult3d( matA, matB, matA );
* Mult3d( matA, matC, matA );
*
* Then:  matA = matA * matB * matC
*
* If we multiplied directly into the user's
* 'res' pointer, the above wouldn't be possible.
*
*/

memcpy( res, &result, sizeof(MATRIX3D_TYPE) );

} /* Mult3d end */



/****************************************
 * RotMatRad3dXYZ( MATRIX3D_TYPE *m,
 *                 float rx,
 *                 float ry,
 *                 float rz );
 *
 * Generate a matrix of rotatin about
 * the x, y, and z axes.  The angles
 * are specified in radians.
 *
 */


void RotMatRad3dXYZ( MATRIX3D_TYPE *m,
                     float rx,
                     float ry,
                     float rz )
{ /* RotMatRad3dXYZ begin */

MATRIX3D_TYPE rrx, rry, rrz, rtemp;

/****************************************
 * Generate each of the 3 rotation
 * matrices for each axis.
 */

RotMatRad3dX( &rrx, rx );
RotMatRad3dY( &rry, ry );
RotMatRad3dZ( &rrz, rz );

/****************************************
 * No generate the composite matrix
 * containing each of the 3 rotation
 * matrices. The end result is:
 *
 * m = rrx * rry * rrz;
 */

Mult3d( &rrx, &rry, &rtemp );
Mult3d( &rtemp, &rrz, m );

} /* RotMatRad3dXYZ end */



/****************************************
 * RotMatDeg3dXYZ( MATRIX3D_TYPE *m,
 *                 float dx,
 *                 float dy,
 *                 float dz );
 *
 * Generate a matrix of rotatin about
 * the x, y, and z axes.  The angles
 * are specified in degrees.
 *
 */

void RotMatDeg3dXYZ( MATRIX3D_TYPE *m,
                     float dx,
                     float dy,
                     float dz )

{ /* RotMatDeg3dXYZ begin */

MATRIX3D_TYPE rrx, rry, rrz, rtemp;

/****************************************
 * Generate each of the 3 rotation
 * matrices for each axis.
 */

RotMatDeg3dX( &rrx, dx );
RotMatDeg3dY( &rry, dy );
RotMatDeg3dZ( &rrz, dz );

/****************************************
 * No generate the composite matrix
 * containing each of the 3 rotation
 * matrices. The end result is:
 *
 * m = rrx * rry * rrz;
 */

Mult3d( &rrx, &rry, &rtemp );
Mult3d( &rtemp, &rrz, m );

} /* RotMatDeg3dXYZ end */



/****************************************
 * TranslateMat3d( MATRIX3D_TYPE *m,
 *                 float dx,
 *                 float dy,
 *                 float dz );
 *
 * Generate a translation matrix for
 * delta x, y, and z values given
 */

void TranslateMat3d( MATRIX3D_TYPE *m,
                     float dx,
                     float dy,
                     float dz )

{ /* TranslateMat3d begin */

/****************************************
 * T(dx, dy, dz ) = | 1  0  0  0 |
 *                  | 0  1  0  0 |
 *                  | 0  0  1  0 |
 *                  | dx dy dz 1 |
 *
 * Generate an identity matrix, then
 * initialize columns 0, 1, & 2 of
 * row 3, as shown.
 */



Identity3d( m );

/* Row 3 */

m->Mat[3][0] = dx;
m->Mat[3][1] = dy;
m->Mat[3][2] = dz;

} /* TranslateMat3d end */



/****************************************
 * ScaleUniformMat3d( MATRIX3D_TYPE *m,
 *                    float s );
 *
 * Generate a uniform scaling matrix
 * for the scaling parameter s.
 *
 */

void ScaleUniformMat3d( MATRIX3D_TYPE *m,
                        float s )

{ /* ScaleUniformMat3d begin */

/****************************************
 * s(Sx, Sy, Sz) where Sx=Sy=Sz
 *
 * s(s,s,s) = | s 0 0 0 |
 *            | 0 s 0 0 |
 *            | 0 0 s 0 |
 *            | 0 0 0 1 |
 */


Zero3d( m );

/****************************************
 * Set 3 elements along the diagonal
 * equal to scale factor s.  Use the
 * multiple equate construct.  Some
 * C compilers can perform some additional
 * optimization on such forms, as opposed
 * to multiple, separate equates
 */

m->Mat[0][0] = m->Mat[1][1] = m->Mat[2][2] = s;

m->Mat[3][3] = 1.0;

} /* ScaleUniformMat3d end */



/****************************************
 * ScaleDifferentialMat3d( MATRIX3D_TYPE *m,
 *                         float sx,
 *                         float sy,
 *                         float sz );
 *
 * Generate a differential scaling
 * matrix along all 3 axes.
 */

void ScaleDifferentialMat3d( MATRIX3D_TYPE *m,
                             float sx,
                             float sy,
                             float sz )

{ /* ScaleDifferentialMat3d begin */

/****************************************
 * Scale(sx,sy,sz) = | sx  0  0  0 |
 *                   | 0   sy 0  0 |
 *                   | 0   0  sz 0 |
 *                   | 0   0  0  1 |
 *
 * Zero the matrix, set the scale
 * factors along the diagonal and
 * set the homogeneous element to 1.
 *
 */

Zero3d( m );

m->Mat[0][0] = sx;
m->Mat[1][1] = sy;
m->Mat[2][2] = sz;
m->Mat[3][3] = 1.0;

} /* ScaleDifferentialMat3d end */

