/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... MATRIX3D.H      */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 13-FEB-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Header file for */
/*  general 3d  matrix operations & structures.         */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
09-APR-90 MR Remove some unused vector operations.
09-APR-90 MR Final cleanup for LAB5.

*\r*/

#define MATRIX3D        /* Define MATRIX3D, to indicate */
                        /* it has been included.        */

/****************************************
 * Type definition for the basic 3D
 * matrix structure.  A pointer to this
 * data type will be required for all
 * matrix operations that follow.
 * This package is specific to 3D
 * operations, suitable for homogeneous
 * coordinate transformations.  The
 * routines will be optimized as much
 * as practical, without sacrificing the
 * generality or understandibility of
 * operations involved.
 */

typedef struct MATRIX3D_TYPE
  {
  float Mat[4][4];
  } MATRIX3D_TYPE;

/****************************************
 * Type definition for the basic 3d
 * homogeneous coordinate type.
 */

typedef struct VECTOR3D_TYPE
  {
  float x;
  float y;
  float z;
  float h;
  } VECTOR3D_TYPE;

/****************************************
 * Sometimes it's more convenient to
 * access the vector as a one dimensional
 * array.
 */

typedef struct VECTOR3D_ARRAY_TYPE
  {
  float Vect[4];
  } VECTOR3D_ARRAY_TYPE;

/*----------
* Index definitions
* for acessing the
* elements of the
* homogeneous coordinate
*/

#define VX 0
#define VY 1
#define VZ 2
#define VH 3

/*--------------------------------------*/
/* Triginometric constants:             */
/*--------------------------------------*/

#define PI                      (float)3.14159265359
#define PI_DEGREES              (float)180.0

/* One degree, in radians*/

#define ONE_DEGREE_RADIANS      (float)0.017453292


/****************************************
 * Convert an angle in radians to degrees
 */

float Degrees( float rad_angle );

/****************************************
 * Convert an angle in degrees to radians
 */

float Radians( float deg_angle );

/****************************************
 * Zero the matrix.  This is done with
 * the "memset" function in C, simply
 * filling that block of memory with all
 * zeros, forcing all floats in the array
 * to zero. FASTEST way to fill a matrix
 * with zeros.
 */

void Zero3d ( MATRIX3D_TYPE *m );

/****************************************
 * This function will print a 4x4 matrix
 * to the screen.  It generally isn't
 * used for the operations we need in
 * interactive 3d graphics, but is
 * useful in debugging
 */

void MatPrint3d( MATRIX3D_TYPE *m );

/****************************************
 * This function will print a 1x4 vector
 * on the screen.  It generally isn't
 * used for the operations we need in
 * interactive 3d graphics, but is useful
 * in debugging.
 */

void VectPrint3d( VECTOR3D_TYPE *v );

/****************************************
 * This function will generate an identity
 * matrix for the caller.
 */

void Identity3d( MATRIX3D_TYPE *m );

/****************************************
 * This function will transpose a 4x4
 * matrix, useful for inverting certain
 * rotation matrices.
 */

void Transpose3d( MATRIX3D_TYPE *m );

/****************************************
 * Duplicate a 4x4 matrix.  Use to keep
 * a backup copy of a matrix for some
 * graphical operations which we might
 * want to undo.
 */

void Dup3d( MATRIX3D_TYPE *dest, MATRIX3D_TYPE *src );

/****************************************
 * Set one of the elements of the 4x4
 * matrix.  User specifies column, row,
 * and new value for that element.
 */

void SetElement3d( MATRIX3D_TYPE *m,  int row, int col, float element );

/****************************************
 * Set an entire row of a matrix by
 * copying the vector into the specified
 * row.
 */

void SetRow3d( MATRIX3D_TYPE *m, VECTOR3D_TYPE *v, int row );

/****************************************
 * Set an entire column of a matrix by
 * copying the vector into the
 * specified column.
 */

void SetCol3d( MATRIX3D_TYPE *m, VECTOR3D_TYPE *v, int col );

/****************************************
 * VectMatMult3d - Multiply the specified
 * matrix by the specified vector, and
 * place results in user supplied output
 * vector.
 */

void VectMatMult3d( VECTOR3D_TYPE *v, MATRIX3D_TYPE *m,
                    VECTOR3D_TYPE *res );

/****************************************
 * RotMatRad3d'N'( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the 'N' axis of r radians.
 */

void RotMatRad3dZ( MATRIX3D_TYPE *m, float r );

void RotMatRad3dX( MATRIX3D_TYPE *m, float r );

void RotMatRad3dY( MATRIX3D_TYPE *m, float r );


/****************************************
 * RotMatDeg3d'N'( MATRIX3D_TYPE *m, float r )
 *
 * Generate a Matrix of rotation about
 * the 'N' axis of d degrees.
 */


void RotMatDeg3dZ( MATRIX3D_TYPE *m, float d );

void RotMatDeg3dX( MATRIX3D_TYPE *m, float d );

void RotMatDeg3dY( MATRIX3D_TYPE *m, float d );


/****************************************
 * RotMatRad3dXYZ( MATRIX3D_TYPE *m,
 *                 float rx,
 *                 float ry,
 *                 float rz );
 *
 * Generate a matrix of rotatin about
 * the x, y, and z axes.  The angles
 * are specified in radians.
 *
 */


void RotMatRad3dXYZ( MATRIX3D_TYPE *m,
                     float rx,
                     float ry,
                     float rz );

/****************************************
 * RotMatDeg3dXYZ( MATRIX3D_TYPE *m,
 *                 float dx,
 *                 float dy,
 *                 float dz );
 *
 * Generate a matrix of rotatin about
 * the x, y, and z axes.  The angles
 * are specified in degrees.
 *
 */


void RotMatDeg3dXYZ( MATRIX3D_TYPE *m,
                     float dx,
                     float dy,
                     float dz );

/****************************************
 * Mult3d( MATRIX3D_TYPE *s1,
 *         MATRIX3D_TYPE *s2,
 *         MATRIX3D_TYPE *res );
 *
 * Matrix multiplication:
 *
 * res = s1 * s2;
 *
 * Remember, matrix multiplication is NOT
 * commutative, so the caller must be careful
 * to specify the sources, s1, and s2, in
 * the desired sequence of multiplication!
 *
 */

void  Mult3d( MATRIX3D_TYPE *s1,
              MATRIX3D_TYPE *s2,
              MATRIX3D_TYPE *res );

/****************************************
 * TranslateMat3d( MATRIX3D_TYPE *m,
 *                 float dx,
 *                 float dy,
 *                 float dz );
 *
 * Generate a translation matrix for
 * delta x, y, and z values given
 */

void TranslateMat3d( MATRIX3D_TYPE *m,
                     float dx,
                     float dy,
                     float dz );

/****************************************
 * ScaleUniformMat3d( MATRIX3D_TYPE *m,
 *                    float s );
 *
 * Generate a uniform scaling matrix
 * for the scaling parameter s.
 *
 */

void ScaleUniformMat3d( MATRIX3D_TYPE *m,
                        float s );

/****************************************
 * ScaleDifferentialMat3d( MATRIX3D_TYPE *m,
 *                         float sx,
 *                         float sy,
 *                         float sz );
 *
 * Generate a differential scaling
 * matrix along all 3 axes.
 */

void ScaleDifferentialMat3d( MATRIX3D_TYPE *m,
                             float sx,
                             float sy,
                             float sz );

