/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... MENU.C          */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 12-FEB-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Make menu       */
/*  selections.                                         */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
13-MAR-90 MR Add StatusLine routine (migrated from LAB2.C)
13-MAR-90 MR Modify MENU to handle mouse I/O.
16-MAR-90 MR Begin "drop down" menu extensions
09-APR-90 MR Final cleanup for LAB5.
22-OCT-90 MR Final cleanup for LAB5, CSc443!

*\r*/

/* ---------- System include files -------------------- */

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <graphics.h>
#include <alloc.h>
#include <dos.h>
/* ---------- User include files ---------------------- */

#include "MOUSE.H"
#include "LAB.H"
#include "MATRIX3D.H"
#include "MENU.H"

/* ---------- Module constants ------------------------ */

#define DEFAULT_CHAR_SIZE 1

#define BAR_HEIGHT 7

#define BAR_Y_OFFSET 4

#define HIGHLIGHT_BAR_Y 10

#define BOTTOM_MENU_BAR 12

#define TEXT_COLOR BLACK
#define BAR_COLOR  LIGHTBLUE

#define BEVEL_ADJUST 4

/*
 * Mouse position field width:
 * X:000 Y:000
 */

#define MOUSE_POS_FIELD_WIDTH 12

/* ---------- Module globals -------------------------- */

static GRAPH_DESCR_TYPE MenuInfo;

static VIEW_COORD_TYPE WorkAreaTop;
static VIEW_COORD_TYPE WorkAreaBot;

static VIEW_COORD_TYPE topLeft[MAX_MENU_ITEMS];
static VIEW_COORD_TYPE botRight[MAX_MENU_ITEMS];
static void *menuPtrs[MAX_MENU_ITEMS];

static int promptColor = LIGHTBLUE;
static int menuColor   = TEXT_COLOR;

static int topBorderColor = WHITE;
static int botBorderColor = DARKGRAY;
static int infoColor      = LIGHTGRAY;
static int barColor       = LIGHTBLUE;
static int bkColor        = BLACK;

static int normalMenu     = TRUE;

/* ---------- Module procedure prototpyes ------------- */

/* Local procedure to initialize some static
 * variables for the mouse display field position
 * Thus we don't have to recompute its location every
 * time, making it a bit faster on the display.
 */

void inizMousePos( void );



/*\p*********************************************/
/* Procedure:                                   */
/* int GrafMenuSelect ( char *items[],          */
/*                      int eraseitem );        */
/* Inputs:                                      */
/* char *items[] - Menu selections to choose    */
/* from.                                        */
/* int eraseitem - If this menu selection is    */
/* chosen, erase menu on exit - else leave      */
/* in place.                                    */
/* Output:                                      */
/* int - Item number. Return -1 if no selection */
/* (escape key cance/exit)                      */
/* Globals:                                     */
/*\p*********************************************/

int GrafMenuSelect( char *items[],
                    int eraseitem )

{ /* GrafMenuSelect begin */

int             i;
int             size;
int             status;
int             leftbutton;
int             selected;
int             selections;
int             exitnow;
int             savec;
int             mx;
int             mbot;
int             spacing;
VIEW_COORD_TYPE mouse;

/* Enable menu access */
SetViewMenuArea();
/* Hide the mouse pointer while drawing */
Mhidemouse();
MpurgeButtons();

savec = getcolor();

for (i=0; i<MAX_MENU_ITEMS; i++)
  menuPtrs[i] = (void *)NULL;

/* Enable 8x8 text font */

settextstyle( DEFAULT_FONT, HORIZ_DIR, DEFAULT_CHAR_SIZE );

settextjustify( LEFT_TEXT, TOP_TEXT );

moveto( 0, 0 );
mx = MenuInfo.Max.x;
mbot = MenuBarHeight();

infobox( 0, 0, mx, mbot,
         infoColor,
         topBorderColor,
         botBorderColor,
         BK_OVRWRITE );

i = 0;

spacing = textwidth("W");

outtext( " " );
setcolor( barColor );
line( getx()-spacing/2, BEVEL_ADJUST, getx()-spacing/2, mbot-BEVEL_ADJUST );

Mhidemouse();
while ( items[i] != NULL )
  {
  mx = getx();
  moveto( mx, BAR_Y_OFFSET );
  topLeft[i].x = mx-1;
  topLeft[i].y = gety()-1;
  botRight[i].x = topLeft[i].x + textwidth(items[i]) + 1;
  botRight[i].y = topLeft[i].y + textheight(items[i]) + 1;
  size = imagesize(topLeft[i].x, topLeft[i].y, botRight[i].x, botRight[i].y);
  menuPtrs[i] = (void *)malloc(size);
  setcolor( menuColor );
  moveto( topLeft[i].x+1, topLeft[i].y+1 );
  outtext( items[i] );
  getimage( topLeft[i].x, topLeft[i].y,
            botRight[i].x, botRight[i].y,
            menuPtrs[i] );
  outtext( " " );
  setcolor( barColor );
  line( getx()-spacing/2, BEVEL_ADJUST, getx()-spacing/2, mbot-BEVEL_ADJUST );
  i++;
  }

selections = i;
moveto( mx, BAR_Y_OFFSET );
topLeft[i].x = mx;
topLeft[i].y = gety();
botRight[i].x = topLeft[i].x + textwidth(items[i]);
botRight[i].y = topLeft[i].y + textheight(items[i]);

exitnow = FALSE;

Mshowmouse();
do
  { /* Outer loop, wait for valid selection */
  do
    {
    Mshowmouse();
    do
      { /* Wait for mouse on menu bar */
      getmousecoords( &mouse.x, &mouse.y );
      exitnow = mouseinbox( topLeft[0].x, topLeft[0].y,
                            botRight[selections].x,
                            botRight[selections].y,
                            mouse.x, mouse.y );
      if (exitnow == FALSE)
        MpurgeButtons();
      }
    while (exitnow == FALSE);
    /* Mouse now in menu bar rectangle
     * Now get coordinates and find selection
     * if any
     */
    getmousecoords( &mouse.x, &mouse.y );
    status = findselection( &mouse, topLeft, botRight, selections );
    if (status == NO_SELECTION)
      MpurgeButtons();
    }
  while (status == NO_SELECTION);
    /* Mouse pointer is over one of the selections
     * now. Highlight it and test for button press
     */
  Mhidemouse();
  putimage(topLeft[status].x, topLeft[status].y,
           menuPtrs[status], NOT_PUT );
  Mshowmouse();
  do
    { /* Now wait for left button, or mouse movement off the box */
    leftbutton = mousebuttonpressed(LEFT_BUTTON);
    getmousecoords( &mouse.x, &mouse.y );
    selected   = mouseinbox( topLeft[status].x,
                             topLeft[status].y,
                             botRight[status].x,
                             botRight[status].y,
                             mouse.x, mouse.y );
    }
  while ( (leftbutton == FALSE) &&
          (selected == TRUE) );
  if ( (leftbutton == TRUE) &&
       (selected == TRUE) )
    {
    exitnow = TRUE;
    }
  else
    {
    Mhidemouse();
    putimage( topLeft[status].x,
              topLeft[status].y,
              menuPtrs[status],
              COPY_PUT );
    Mshowmouse();
    exitnow = FALSE;
    MpurgeButtons();
    }
  }
while (exitnow == FALSE);


if ( (eraseitem == ERASE_ALWAYS) ||
     ( status == eraseitem) )
  {
  /* Erase menu */
  Mhidemouse();
  box( 0, 0, MenuInfo.Max.x, BOTTOM_MENU_BAR,
       bkColor, FILL_BOX );
  }

Mshowmouse();
setcolor( savec );


for(i=0; i<MAX_MENU_ITEMS; i++)
  {
  if (menuPtrs[i] != NULL)
    {
    free(menuPtrs[i]);
    menuPtrs[i] = (void *)NULL;
    }
  }

SetViewDrawingArea();
AudioSelect();
MpurgeButtons();
return( status );

} /* GrafMenuSelect end */




/*\p***********************************
 Procedure:
 void StatusLineY( char *msg,
                   int color,
                   int y );
 Display a message at the y pos on the
 screen in the selected color.

 Inputs:
 char *msg - String to display
 int color - Foreground color to use
 int y     - Start y position of message
 Output:
 pretty things

 Globals:

\p*************************************/

void StatusLineY( char *msg, int color, int y )

{ /* StatusLineY begin */
int i, moff;

SetViewMenuArea( );
/*---------------------------------
Display a status line at the bottom
of the screen for prompts and
other interactive feedback
---------------------------------*/

moff = Mouseoff();
i = getcolor();
setcolor( color );

Mhidemouse();

settextjustify( LEFT_TEXT, BOTTOM_TEXT );
outtextxy( 0, y, msg );

setcolor(i);

if (moff == FALSE)
  Mshowmouse();

SetViewDrawingArea( );

} /* StatusLineY end */


/*\p***********************************

findselection:

Search the rectangles, specified by the list
of topLeft and botRight coordinate pairs, to
see if the current mouse coordinates are
within any of them.  If so, return the index
value for that match.  If no matches are
found, then return NO_SELECTION.  Used to
decode the menu for mouse control.

\p************************************/

int findselection( VIEW_COORD_TYPE *mouse,
                   VIEW_COORD_TYPE *topLeft,
                   VIEW_COORD_TYPE *botRight,
                   int entries )
{ /* findselection begin */
int status;
int i;
int exitnow;

status = NO_SELECTION;

i = 0;
exitnow = FALSE;

do
  {
  if ( (mouse->x >= topLeft->x) && (mouse->x <= botRight->x) &&
       (mouse->y >= topLeft->y) && (mouse->y <= botRight->y) )
    {
    status = i;
    exitnow = TRUE;
    }
  else
    {
    topLeft++;
    botRight++;
    i++;
    }
  }
while ( (exitnow == FALSE) && (i < entries) );

return(status);

} /* findselection end */


/*\p***********************************

void mline( int x1, int y1, int x2, int y2 );

This function is for drawing a line, while
using the mouse. It simply calls the turbo C
graphics line function, in between hidemouse,
showmouse calls.  This is best used when
making frequent calls to draw lines between
mouse movements.  When drawing a great number
of graphics primitives, it is best to simply
hide the mouse and then make all the calls to
update the display, then show mouse.  For
user interaction, mline is best, however.

\p************************************/


void mline( int x1, int y1, int x2, int y2 )
{ /* mline begin */

int moff;

moff = Mouseoff();

Mhidemouse();

line( x1, y1, x2, y2 );

if (moff == FALSE)
  Mshowmouse();

} /* mline end */


/*\p***********************************

void mlineto( int x2, int y2 );

This function is for drawing a line relative
to CP, while using the mouse. It simply calls
the turbo C graphics lineto function, in
between hidemouse, showmouse calls.  This is
best used when making frequent calls to draw
lines between mouse movements.
\p************************************/


void mlineto( int x2, int y2 )
{ /* mline begin */

int moff;

moff = Mouseoff();

Mhidemouse();

lineto( x2, y2 );

if (moff == FALSE)
  Mshowmouse();

} /* mline end */

/*\p***********************************

void *infobox( x1, y1, x2, y2, color, tl_edge, rb_edge, bk_save );

Draw a filled, beveled box from x1, y2 to x2,
y2.  Draw the box's interior in the "color"
specified.  Draw a bevel along the left and
top in the tl_edge color. Draw another bevel
along the right and bottom edge in the color
rb_edge.

The infobox will be used for all menu bars
and message boxes.  First the infobox is
drawn, then the text or graphical information
is displayed on top of the box area, by some
other procedure.  (It is up to the caller of
infobox to perform clipping within that
box.)

Since the box might be drawn over top of
important information the user may request
infobox() to save the background.  This
routine will then allocate storage for the
background area, copy it (getimage), and
return a pointer to that data.  If an
allocation error occurs, then a NULL pointer
is returned.

If the user doesn't care about the
background, then he can tell infobox() to
simply overwrite it, ignoring anything
underneath.  Then a null pointer shall be
returned.

It is up to the caller requesting the
background to be stored to restore the
background and release the block of memory
allocated.  Many calls to infobox(),
storing background memory, but not restoring,
can cause out of memory errors.

Note that the user may overlap multiple
beveled boxes this way, and then restore in
any sequence he wishes, until he frees the
memory.

\p************************************/

void *infobox  ( int x1,
                 int y1,
                 int x2,
                 int y2,
                 int color,
                 int tl_edge,
                 int rb_edge,
                 int bk_save )

{ /* infobox begin */


int y;
int size;
void *bufPtr;
struct viewporttype view;

bufPtr = (void *)NULL;

if (bk_save == BK_SAVE)
  {
  size = imagesize(x1, y1, x2, y2);
  bufPtr = (void *)malloc(size);
  if (bufPtr != NULL)
    getimage(x1, y1, x2, y2, bufPtr);
  }

if (normalMenu == TRUE)
  {
  if ( ( (x2-x1) > 4 ) &&
       ( (y2-y1) > 4 ) )
    { /* if big enough, draw interior */
    setcolor(color);
    for (y = y1+2; y<=y2-2; y++)
      line(x1+2, y, x2-2, y);
    } /* if big enough, draw interior */
  }
else
  {
  getviewsettings( &view );
  setviewport( x1, y1, x2, y2, 0 );
  clearviewport();
  setviewport( view.left, view.top,
               view.right, view.bottom,
               view.clip );
  }

setcolor(tl_edge);
line(x1,y1,x1,y2);
line(x1,y1,x2,y1);
if (normalMenu == TRUE)
  {
  line(x1+1,y1+1,x1+1,y2-1);
  line(x1+1,y1+1, x2-1, y1+1);
  }

setcolor(rb_edge);
line(x2,y1,x2,y2);
line(x1,y2,x2,y2);
if (normalMenu == TRUE)
  {
  line(x2-1,y1+1,x2-1,y2-1);
  line(x1+1,y2-1,x2-1,y2-1);
  }

return(bufPtr);

} /* infobox end */



/****************************************
* Draw a box on the display from tx,ty
* to bx,by.  Use the color specified.
* Fill is a T/F flag.  If true then fill
* the box with the color specified.
*
* void box( int tx,
*	    int ty,
*	    int bx,
*	    int by,
*	    int color,
*	    int fill );
*/

void box( int tx,
          int ty,
          int bx,
          int by,
          int color,
          int fill )

{ /* box begin */

int y;
int moff;
int svc;

svc = getcolor();

setcolor( color );
moff = Mouseoff();

if (moff == FALSE)
  Mhidemouse();

if (fill == TRUE)
  {
  for (y=ty; y<=by; y++)
    line(tx,y,bx,y);
  }
else
  {
  moveto(tx,ty);
  lineto(bx,ty);
  lineto(bx,by);
  lineto(tx,by);
  lineto(tx,ty);
  }

if (moff == FALSE)
  Mshowmouse();

setcolor( svc );
} /* box end */

/* Draw a diamond
 * in the color specified
 * if fill == TRUE then fill the
 * diamond
 */

void diamond( int cx,
              int cy,
              int height,
              int width,
              int color,
              int fill )

{ /* diamond begin */

int moff;
int svc;

svc = getcolor();

setcolor( color );
moff = Mouseoff();

if (moff == FALSE)
  Mhidemouse();

/****************************************
* To fill the diamond, simply draw a cross
* hair in it, dont bother with filling
* it completely. Too much trouble!
*/

if (fill == FILL_DIAMOND)
  {
  moveto(cx,cy-height);
  lineto(cx+width,cy);
  lineto(cx,cy+height);
  lineto(cx-width,cy);
  lineto(cx,cy-height);
  lineto(cx,cy-height);
  moveto(cx-width,cy);
  lineto(cx+width,cy);
  }
else
  {
  moveto(cx,cy-height);
  lineto(cx+width,cy);
  lineto(cx,cy+height);
  lineto(cx-width,cy);
  lineto(cx,cy-height);
  }

if (moff == FALSE)
  Mshowmouse();

setcolor( svc );
} /* diamond end */



int mousehidden = FALSE;

/****************************************
*
* Mhidemouse();
* Call hide mouse, save the state of the mouse.
* Allows callers to determine if the mouse is hidden
* and selectively hide/unhide it
*
*/

void Mhidemouse( void )
{ /* Mhidemouse begin */

mousehidden = TRUE;
hidemouse();

} /* Mhidemouse end */



/****************************************
 * Show the mouse and set the mouse hidden
 * state flag to false
 */

void Mshowmouse( void )

{ /* Mshowmouse begin */

mousehidden = FALSE;

showmouse();
} /* Mshowmouse end */


/****************************************
 * Report the state of the current
 * mouse pointer.
 *
 * If mouse off, return true
 * else return false
 */

int Mouseoff( void )
{ /* Mouseon begin */

return( mousehidden );

} /* Mouseoff end */



/**************************************
 int VertMenu( int index,
               char *items[],
               char *selects );

 index - same as that returned by
 previous GrafMenuSelect call (the
 global topLeft[] and botRight[]
 arrays still valid from last call)
 *items[] - Null pointer terminated
 character array list of vert menu
 bar entries
 *select - Character array detailing
 those entries that are selectable,
 disabled, or not selectable at all

 Return a sequential index (int) into the
 items array that was selected by the
 user.  VM_NO_SELECT entries are ignored
 and not counted.  VM_DISABLED and VM_SELECT
 items are considered in the indexing
 algorithm.

 To be added:

 If "index" == -1, then it isn't a vertical
 menu at all, but rather a dialog box.
 In this instance, the "menu" is centered
 vertically and horizontally on the
 display.  The select and index rules
 apply.  At least one item must be selectable.

 If selects == NULL, then assume only the last
 entry is to be selectable and the rest are
 non-selectable items.

***************************************/

int VertMenu( int index,
              char *items[],
              char *selects,
              int save )

{ /* VertMenu begin */

int                     i;
int                     len;
int                     l;
int                     status;
int                     selected;
int                     leftbutton;
int                     entries;
int                     exitnow;
int                     size;
VIEW_COORD_TYPE         tl;
VIEW_COORD_TYPE         br;
VIEW_COORD_TYPE         tlv[MAX_MENU_ITEMS];
VIEW_COORD_TYPE         brv[MAX_MENU_ITEMS];
VIEW_COORD_TYPE         mouse;
void                    *vmBuf;

SetViewMenuArea( );
i = 0;
len = 0;

while(items[i] != NULL)
  {
  /* Find max string length in vertical menu */
  l = strlen(items[i]);
  if (l > len)
    len = l;
  i++;
  }
entries = i;

if (index == DIALOG_BOX_INDEX)
  {
  tl.x = MenuInfo.Max.x/2 - len*textwidth("W")/2;
  tl.y = MenuInfo.Max.y/2 - BEVEL_ADJUST - (entries*textheight("W"))/2;
  }
else
  {
  tl.x = topLeft[index].x;
  tl.y = botRight[index].y + BEVEL_ADJUST + 2;
  }

br.x = tl.x + 2*BEVEL_ADJUST + len * textwidth("W");
br.y = tl.y + 2*BEVEL_ADJUST + (entries * textheight("W")) + 1;

/* Force box to fit on screen */

if (br.x > MenuInfo.Max.x)
  {
  i = br.x - MenuInfo.Max.x;
  tl.x = tl.x - i;
  br.x = MenuInfo.Max.x;
  if (tl.x < 0)
    tl.x = 0;
  }

if (br.y > MenuInfo.Max.y)
  {
  i = MenuInfo.Max.y - br.y;
  br.y = MenuInfo.Max.y;
  tl.y = tl.y - i;
  if (tl.y < 0)
    tl.y = 0;
  }

Mhidemouse();
vmBuf = infobox( tl.x, tl.y, br.x, br.y,
                  infoColor,
                  topBorderColor,
                  botBorderColor,
                  save );

if ( (vmBuf == NULL) && (save == BK_SAVE) )
  {
  ClearStatusLine();
  StatusLine("Out of MEMORY for screen menus!", LIGHTRED);
  }

tlv[0].x = tl.x + BEVEL_ADJUST;
tlv[0].y = tl.y + BEVEL_ADJUST;
brv[0].x = br.x - BEVEL_ADJUST;
brv[0].y = tl.y + textheight("W") + BEVEL_ADJUST - 1;

setcolor( menuColor );
for (i=0; i<entries; i++)
  {
  if (selects == NULL)
    if (i < entries-1)
      setcolor(promptColor);
    else
      setcolor(menuColor);
  else if (selects[i] == VM_NO_SELECT)
    setcolor(promptColor);
  else
    setcolor(menuColor);
  outtextxy( tlv[i].x, tlv[i].y, items[i] );
  menuPtrs[i] = NULL;
  if (selects == NULL)
    {
    if (i == entries-1)
      {
      size = imagesize( tlv[i].x, tlv[i].y, brv[i].x, brv[i].y );
      menuPtrs[i] = (void *)malloc(size);
      if (menuPtrs[i] != NULL)
        {
        getimage( tlv[i].x, tlv[i].y, brv[i].x, brv[i].y, menuPtrs[i] );
        }
      }
    }
  else if (selects[i] == VM_SELECT)
    {
    size = imagesize( tlv[i].x, tlv[i].y, brv[i].x, brv[i].y );
    menuPtrs[i] = (void *)malloc(size);
    if (menuPtrs[i] != NULL)
      {
      getimage( tlv[i].x, tlv[i].y, brv[i].x, brv[i].y, menuPtrs[i] );
      }
    }
  tlv[i+1].x = tlv[i].x;
  brv[i+1].x = brv[i].x;
  tlv[i+1].y = brv[i].y + 1;
  brv[i+1].y = brv[i].y + textheight("W");
  }

/* Now enable mouse pointer and allow
 * user to select one of the menu items.
 * When done, return the selection
 * to the caller, clear the menu off the
 * screen.
 */

Mshowmouse();
MpurgeButtons();
status = 0;
do
  { /* Outer mouse handler loop, until valid selection */
  do
    { /* Until inside menu & selected */
    Mshowmouse();
    do
      { /* Until inside menu */
      getmousecoords( &mouse.x, &mouse.y );
      exitnow = mouseinbox( tlv[0].x, tlv[0].y,
                            brv[entries].x, brv[entries].y,
                            mouse.x, mouse.y );
      if (exitnow == FALSE)
        {
        exitnow = mousebuttonpressed( LEFT_BUTTON );
        if (exitnow == FALSE)
          exitnow = mousebuttonpressed( RIGHT_BUTTON );
        if (exitnow == TRUE)
          status = CANCEL_VERT_MENU;
        }
      }
    while (exitnow == FALSE);
    if (status != CANCEL_VERT_MENU)
      {
      /* Mouse pointer in Vertical menu
       * rectangle. Now get coordinates
       * and find selection, if any.
       */
      getmousecoords( &mouse.x, &mouse.y );
      status = findselection( &mouse, tlv, brv, entries );
      if (status == NO_SELECTION)
        MpurgeButtons();
      if (status != NO_SELECTION)
        {
        if (selects == NULL)
          if (status != (entries-1) )
            status = NO_SELECTION;
        if (selects != NULL)
          if (selects[status] != VM_SELECT)
            status = NO_SELECTION;
        }
      } /* end if !CANCEL_VERT_MENU */
    }
  while (status == NO_SELECTION);
  if (status != CANCEL_VERT_MENU)
    {
    Mhidemouse();
    putimage( tlv[status].x, tlv[status].y,
              menuPtrs[status], NOT_PUT );
    Mshowmouse();
    do
      {
      leftbutton = mousebuttonpressed( LEFT_BUTTON );
      getmousecoords( &mouse.x, &mouse.y );
      selected = mouseinbox( tlv[status].x, tlv[status].y,
                             brv[status].x, brv[status].y,
                             mouse.x, mouse.y );
      }
    while ( (leftbutton == FALSE) &&
            (selected   == TRUE) );
    if ( (selected == TRUE) &&
         (leftbutton == TRUE) )
      {
      exitnow = TRUE;
      }
    else
      {
      exitnow = FALSE;
      Mhidemouse();
      putimage( tlv[status].x, tlv[status].y,
                menuPtrs[status], COPY_PUT );
      Mshowmouse();
      MpurgeButtons();
      }
    }
  else
    {
    exitnow = TRUE;
    }
  }
while( exitnow == FALSE );

/* Restore background, free menuPtrs, and exit
 */

Mhidemouse();
if ( (save == BK_SAVE) && (vmBuf != NULL) )
  {
  putimage(tl.x, tl.y, vmBuf, COPY_PUT);
  free(vmBuf);
  }
else
  {
  setfillstyle( SOLID_FILL, bkColor );
  bar( tl.x, tl.y, br.x, br.y );
  }

Mshowmouse();

for (i=0; i<entries; i++)
  {
  if (menuPtrs[i] != NULL)
    {
    free(menuPtrs[i]);
    menuPtrs[i] = NULL;
    }
  }

SetViewDrawingArea( );
AudioSelect();

MpurgeButtons();
return(status);

} /* VertMenu end */


/****************************************
 * void MpurgeButtons( void )
 *
 * Purge any button presses pending
 * before continuing, to prevent
 * spurious button clicks from
 * confusing the system, since
 * mouse.c buffers the stupid things!
 */

void MpurgeButtons( void )
{ /* MpurgeButtons begin */

int exitnow;
int i;

do
  {
  exitnow = TRUE;
  do
    {
    i = mousebuttonpressed( LEFT_BUTTON );
    if (i == TRUE)
      exitnow = FALSE;
    }
  while ( i == TRUE );

  do
    {
    i = mousebuttonpressed( RIGHT_BUTTON );
    if (i == TRUE)
      exitnow = FALSE;
    }
  while ( i == TRUE );
  }
while (exitnow == FALSE);

} /* MpurgeButtons end */

static saveMenuColor = TEXT_COLOR;

/******************************
* Set fast or slow menu display
* speed.  Fast makes faster display
* updates, and better looking
* screen dumps to printer.
*/

void SetMenuSpeed( int normal )
{
normalMenu = normal;

if (normalMenu == FALSE)
  {
  saveMenuColor = menuColor;
  menuColor     = WHITE;
  }
else
  menuColor = saveMenuColor;

}
/****************************************
 * void SetMenuColors( int topedgecolor,
 *                     int botedgecolor,
 *                     int facecolor,
 *                     int textcolor,
 *                     int prmptcolor,
 *                     int bcolor );
 *
 * If any color is NO_COLOR_CHANGE (-1)
 * then it will not be changed.
 *
 * Set the module global variables to the
 * appropriate colors for the menus. This allows
 * us to control color selections based on the
 * graphics mode enabled.
 */

void SetMenuColors( int topedgecolor,
                    int botedgecolor,
                    int facecolor,
                    int textcolor,
                    int prmptcolor,
                    int bcolor,
                    int normal )
{ /* SetMenuColors begin */

/* Save current color set
 */

if (topedgecolor != NO_COLOR_CHANGE)
  topBorderColor = topedgecolor;

if (botedgecolor != NO_COLOR_CHANGE)
  botBorderColor = botedgecolor;

if (facecolor != NO_COLOR_CHANGE)
  infoColor = facecolor;

if (textcolor != NO_COLOR_CHANGE)
  menuColor = textcolor;

if (prmptcolor != NO_COLOR_CHANGE)
  promptColor = prmptcolor;

if (bcolor != NO_COLOR_CHANGE)
  barColor = bcolor;

normalMenu = normal;

if (normalMenu == FALSE)
  menuColor = WHITE;

} /* SetMenuColors end */


/****************************************
 * Place markers for the mouse coordinate
 * display field
 */

static int mstatx = -1;
static int mstaty = -1;


/*

Report the position passed in as X, Y coordinates on the status line.
Erase the previous message, if one was there.

*/

static char prevMousePos[80];

void ReportMousePos( int x, int y )
{ /* ReportMousePos begin */

int svc;
int moff;

SetViewMenuArea( );

svc = getcolor();
moff = Mouseoff();

setcolor( BLACK );
Mhidemouse();
outtextxy( mstatx, mstaty + BEVEL_ADJUST, prevMousePos );

/* Call me with x=-1, y=-1 to erase the last
 * mouse pos without updating the field
 * with new information
 */

if ( (x >= 0) && (y >= 0) )
  {
  sprintf( prevMousePos, "X:%3d Y:%3d", x, y);
  setcolor( WHITE );
  outtextxy( mstatx, mstaty + BEVEL_ADJUST, prevMousePos );
  }

if (moff == FALSE)
  Mshowmouse();

setcolor( svc );

SetViewDrawingArea( );
} /* ReportMousePos end */



/****************************************
 * Clear the status line display
 * area to make room for another
 * message
*/

void ClearStatusLine( void )

{ /* ClearStatusLine begin */
int i;
int t;
int moff;

SetViewMenuArea( );

moff = Mouseoff();

i = getcolor();

setcolor(bkColor);

Mhidemouse();

for (t=mstaty; t<= MenuInfo.Max.y; t++)
  line(0,t,mstatx-1,t);

setcolor(i);

if (moff == FALSE)
  Mshowmouse();

SetViewDrawingArea( );

} /* ClearStatusLine end */

/*\p***********************************
 Procedure:
 void StatusLine( char *msg,
                  int color );
 Display a message at the bottom of the
 screen in the selected color.

 Inputs:
 char *msg - String to display
 int color - Foreground color to use

 Output:


 Globals:

\p*************************************/

void StatusLine( char *msg, int color )
{ /* StatusLine begin */
int i;
int moff;

/*---------------------------------
Display a status line at the bottom
of the screen for prompts and
other interactive feedback
---------------------------------*/
ClearStatusLine();
SetViewMenuArea( );
/* Save user's color
*/
i = getcolor();


/* Keep mouse pointer out of the picture
*/

moff = Mouseoff();

Mhidemouse();

infobox( 0, mstaty,
         textwidth(msg)+2*BEVEL_ADJUST,
         MenuInfo.Max.y,
         infoColor,
         topBorderColor,
         botBorderColor,
         BK_OVRWRITE );

/* Set text justification and output message
*/

setcolor( color );
settextjustify( LEFT_TEXT, TOP_TEXT );
outtextxy( BEVEL_ADJUST, mstaty + BEVEL_ADJUST, msg );

if (moff == FALSE)
  Mshowmouse();

/* Restore color
*/
setcolor(i);

SetViewDrawingArea( );

} /* StatusLine end */




/****************************************
* StatusLineGet - Get input from the status
* line.  Show a prompt and allow the
* user to input and return a string.
*
*/

#define FILLMSGLEN 120

void StatusLineGet( char *msg, char *outstr, int fcolor,
                    int bcolor, int inputlen )
{ /* StatusLineGet begin */

char    fillmsg[FILLMSGLEN+1];
int     len;
int     index;
int     x;
int     y;
int     tw;
int     fc_save;
int     key;
char    keystr[2];
char    cursor[2];

/*----------
* For starters we shall assume that the gets()
* works in graphics mode.  I dunno, we just
* gotta start somewhere.
*/

strncpy( fillmsg, msg, FILLMSGLEN );
len = strlen( fillmsg );

tw = textwidth( "W" );

index = 0;
while ( ((index+len) < FILLMSGLEN ) && (index < inputlen) )
  {
  fillmsg[len+index] = ' ';
  index++;
  }

fillmsg[len+index] = 0;

StatusLine( fillmsg, fcolor );
SetViewMenuArea();
Mhidemouse();

x = len * tw;
y = mstaty + BEVEL_ADJUST;

keystr[1] = 0;
index     = 0;
cursor[1] = 0;
cursor[0] = '_';

fc_save = getcolor();
setcolor( fcolor );
moveto( x, y );
outtext( cursor );
moveto( x, y );
do
  {
  key = getch();
  if (key == 0) /* then it's a function or arrow key, ignore it */
    {
    key = getch();
    }
  else
    {
    if ( (key >= SPACE) &&
         (key <= 'z') )
      {
      setcolor( bcolor );
      moveto( x, y );
      outtext( cursor );
      setcolor( fcolor );
      moveto( x, y );
      keystr[0] = (char)key;
      moveto( x, y );
      outtext( keystr );
      outstr[index] = (char)key;
      index++;
      x+=tw;
      outtext( cursor );
      }
    else if (key == BS)
      {
      if (index > 0)
        {
        setcolor( bcolor );
        moveto(x, y);
        outtext( cursor );
        moveto(x, y);
        index--;
        x-=tw;
        moveto(x, y);
        keystr[0] = outstr[index];
        outtext( keystr );
        setcolor( fcolor );
        moveto(x, y);
        outtext( cursor );
        moveto(x, y);
        }
      else
        {
        putch( BEEP );
        }
      }
    }
  }
while( (key != CR) && (index < inputlen) );
outstr[index] = 0;

setcolor( fc_save );

Mshowmouse();
SetViewDrawingArea();

} /* StatusLineGet end */




/****************************************
 *
 * Initialize the coordinates for the
 * start of the mouse coordinate
 * display field.
 */

void inizMousePos( void )
{

SetViewMenuArea( );

mstatx = MenuInfo.Max.x -
         MOUSE_POS_FIELD_WIDTH * textwidth( "W" )
         - 2 * BEVEL_ADJUST;
mstaty = MenuInfo.Max.y - textheight( "W" ) - 2 * BEVEL_ADJUST;

strcpy(prevMousePos,"X:000 Y:000");

SetViewDrawingArea( );

}



/****************************************
 *
 *
 * Setup viewport access to drawing area only
 */

void SetViewDrawingArea( void )
{ /* SetViewDrawingArea begin */

/* Adjust drawing area to be within the bounds of a frame
 * just below the menu bar, just above the status line,
 * and out to the edges of the display
 */

/* setviewport( BEVEL_ADJUST, STATUS_LINE_HEIGHT + 3 * BEVEL_ADJUST,
             MenuInfo.Max.x - BEVEL_ADJUST, mstaty - 3*BEVEL_ADJUST,
             TRUE ); */
setviewport( 0, 0,
             MenuInfo.Max.x - BEVEL_ADJUST, mstaty - 3*BEVEL_ADJUST,
             TRUE );



} /* SetViewDrawingArea end */



/****************************************
* SetClippedViewport( int *xoffset, int *yoffset )
*
* Set a clipped viewport for drawing the
* objects.  Return to the caller an x and
* y offset to be used to adjust the
* object coordinates as they are output to
* the display.
*/

void SetClippedViewport( int *xoffset, int *yoffset )

{ /* SetClippedViewport begin */

*xoffset = BEVEL_ADJUST,
*yoffset = STATUS_LINE_HEIGHT + 2 * BEVEL_ADJUST + 2;

setviewport( *xoffset, *yoffset,
             MenuInfo.Max.x - BEVEL_ADJUST, mstaty - 3*BEVEL_ADJUST,
             TRUE );

} /* SetClippedViewport end */



/****************************************
 * Setup viewport access to drawing screen and
 * menus, enable clipping
 */

void SetViewMenuArea( void )

{ /* SetViewMenuArea end */

setviewport( 0, 0, MenuInfo.Max.x, MenuInfo.Max.y, TRUE );

} /* SetViewMenuArea end */


/****************************************
 *
 * Initialize the menu driver package.
 * Inform it of the current display resolution and
 * colors.  Allow it to initialize it's module variables
 * for controling the state of the mouse, message
 * areas and so on.
 */

void InizMenuDriver( GRAPH_DESCR_TYPE *gPtr )

{ /* InizMenuDriver begin */

MenuInfo.Max.x          = gPtr->Max.x;
MenuInfo.Max.y          = gPtr->Max.y;
MenuInfo.MaxColor       = gPtr->MaxColor;
MenuInfo.Driver         = gPtr->Driver;
MenuInfo.Mode           = gPtr->Mode;


if (MenuInfo.MaxColor == 1) /* A monochromatic mode */
  {
  topBorderColor = 1;
  botBorderColor = 1;
  infoColor      = 0;
  barColor       = 1;
  bkColor        = 0;
  promptColor    = 1;
  menuColor      = 1;
  normalMenu     = FALSE;
  }
  
inizMousePos();
SetViewMenuArea();

/* Frame the drawing work area */

normalMenu = FALSE;

infobox( 0, STATUS_LINE_HEIGHT + 2*BEVEL_ADJUST,
         MenuInfo.Max.x, mstaty - 2*BEVEL_ADJUST,
         bkColor, topBorderColor,
         botBorderColor, BK_OVRWRITE );

/* Shut off any pending sound before
 * starting our own audio feedback.
 */

nosound();

/****************************************
 * Default to normal menu
 */

normalMenu = TRUE;

WorkAreaTop.x = BEVEL_ADJUST;
WorkAreaTop.y = STATUS_LINE_HEIGHT + 2 * BEVEL_ADJUST + 2;
WorkAreaBot.x = MenuInfo.Max.x - BEVEL_ADJUST;
WorkAreaBot.y = mstaty - 3 * BEVEL_ADJUST;

} /* InizMenuDriver end */



/****************************************
 * void AudioSelect( void )
 *
 * Make a brief acknowledgement sound
 * for a button selection
 *
 */


void AudioSelect( void )
{ /* AudioSelect begin */

sound( AUDIO_SEL_TONE );
delay( AUDIO_SEL_DURATION );
nosound();

} /* AudioSelect end */



/****************************************
 * void AudioError( void )
 *
 * Make a brief error sound
 * for an error condition
 *
 */


void AudioError( void )
{ /* AudioError begin */

sound( AUDIO_ERR_TONE );
delay( AUDIO_ERR_DURATION );
nosound();
delay( AUDIO_SEL_DURATION );
sound( AUDIO_ERR_TONE/2 );
delay( AUDIO_ERR_DURATION );

nosound();
} /* AudioError end */



/****************************************
 * GetPoint( VIEW_COORD_TYPE *prevPt,
 *           VIEW_COORD_TYPE *newPt,
 *           int             *button,
 *           int              showcoords );
 *
 * Get a point via mouse input. Return the
 * button pressed, either LEFT_BUTTON or
 * RIGHT_BUTTON.  Return the coordinate
 * in the newPt.
 * If prevPt is not NULL then draw a rubberband
 * line until the user hits a button.
 * If left button is pressed, leave the
 * line on the screen.  If the right
 * button is pressed, erase the rubber
 * band line.  If showcoords is true
 * then report the mouse position while
 * drawing.
 */

void GetPoint( VIEW_COORD_TYPE  *prevPt,
               VIEW_COORD_TYPE *newPt,
               int             *button,
               int             showcoords )

{ /* GetPoint begin */

VIEW_COORD_TYPE pos;
VIEW_COORD_TYPE lastpos;
VIEW_COORD_TYPE prev;
int             rubberband;
int             delta;
int             exitnow;
int             inarea;
int             key;

/*----------
 * if previous point is not a null
 * pointer, then setup for rubber
 * band line draw.
 */

if (prevPt != NULL)
  {
  prev.x = prevPt->x;
  prev.y = prevPt->y;
  rubberband = TRUE;
  }
else
  {
  rubberband = FALSE;
  }

/*----------
 * Read initial mouse pos
 */

getmousecoords( &lastpos.x, &lastpos.y );
/*----------
 * Draw first rubber band
 * line, if enabled
 */

Lsetwritemode( XOR_PUT );
if (rubberband)
  {
  mline( prev.x, prev.y, lastpos.x, lastpos.y );
  }
/*----------
 * If coordinate display enabled
 * show it for the first time
 */

if (showcoords)
  ReportMousePos( lastpos.x, lastpos.y );

/*----------
 * Setup for the loop
 */

exitnow = FALSE;
MpurgeButtons();

do
  { /* until mouse button pressed */
  do
    { /* wait until within drawing area */
    getmousecoords( &pos.x, &pos.y );
    inarea = mouseinbox( WorkAreaTop.x,
                         WorkAreaTop.y,
                         WorkAreaBot.x,
                         WorkAreaBot.y,
                         pos.x, pos.y );
    }
  while (inarea == FALSE);

  /*----------
  * Detect mouse movement
  */
  if ( (pos.x != lastpos.x) ||
       (pos.y != lastpos.y) )
    {
    delta = TRUE;
    }
  else
    {
    delta = FALSE;
    }
  /*----------
  * Rubber band
  */
  setwritemode( XOR_PUT );
  if ( (delta == TRUE) && (rubberband == TRUE) )
    {
    mline(prev.x, prev.y, lastpos.x, lastpos.y); /* Erase */
    mline(prev.x, prev.y, pos.x, pos.y );        /* Draw  */
    }
  lastpos.x = pos.x;
  lastpos.y = pos.y;
  /*----------
  * Show coords
  */
  if ( (delta == TRUE) && (showcoords == TRUE) )
    {
    ReportMousePos( pos.x, pos.y );
    }
  exitnow = mousebuttonpressed( LEFT_BUTTON );
  if (kbhit())
    {
    key = getch();
    key = key & 0x7f;
    }
  else
    key = 0;
  if ( (exitnow == TRUE) || (key == CR) )
    {
    *button = LEFT_BUTTON;
    }
  else
    {
    exitnow = mousebuttonpressed( RIGHT_BUTTON );
    if ( (exitnow == TRUE) || (key == ESC) )
      {
      *button = RIGHT_BUTTON;
      if (rubberband == TRUE)
        { /* Done, erase line */
        mline( prev.x, prev.y, pos.x, pos.y );
        }
      }
    }
  }
while (exitnow == FALSE);

newPt->x = pos.x;
newPt->y = pos.y;

setwritemode( COPY_PUT );

} /* GetPoint end */



/****************************************
 * Set a point in the current color,
 * between hidmouse and showmouse
 * calls.
 */

void msetpoint( int mx, int my )
{ /* msetpoint begin */
int i;

i = getcolor();
Mhidemouse();
putpixel( mx, my, i );
Mshowmouse();

} /* msetpoint end */



/****************************************
* ClearWorkArea() - Use the module variables
* for the graphics work area to define a
* temporary view port, then clear it. Then
* restore the normal drawing viewport.
*/

void ClearWorkArea( void )
{ /* ClearWorkArea begin */

Mhidemouse();
setviewport( WorkAreaTop.x-1,
             WorkAreaTop.y-1,
             WorkAreaBot.x+1,
             WorkAreaBot.y+1,
             TRUE );
clearviewport();
SetViewDrawingArea( );
Mshowmouse();

} /* ClearWorkArea end */



/****************************************
*
* int ReadMouseButton( void );
*
* If left button pressed return LEFT_BUTTON,
* else if right button pressed return
* RIGHT_BUTTON, else return NEITHER_BUTTON_PRESSED.
*
*/

int ReadMouseButton( void )

{ /* ReadMouseButton begin */

int button;

button = mousebuttonpressed( LEFT_BUTTON );

if (button == TRUE)
  button = LEFT_BUTTON;
else
  {
  button = mousebuttonpressed( RIGHT_BUTTON );
  if (button == TRUE)
    button = RIGHT_BUTTON;
  else
    button = NEITHER_BUTTON_PRESSED;
  }


return( button );

} /* ReadMouseButton end */

/*\p*********************************************/
/* Procedure:                                   */
/* DrawDashedLine - Draw a dashed line from     */
/* point 1 to point 2.  Save line style and     */
/* color before drawing, and restore when done. */
/*                                              */
/* Inputs:                                      */
/* pt1Ptr - Point 1                             */
/* pt1Ptr - Point 2                             */
/* color  - Dashed line color                   */
/*                                              */
/* Output:                                      */
/* Purdy picture                                */
/*                                              */
/* Globals:                                     */
/* none                                         */
/*\p*********************************************/

void DrawDashedLine( VIEW_COORD_TYPE *pt1Ptr,
                     VIEW_COORD_TYPE *pt2Ptr,
                     int             color )

{ /* DrawDashedLine begin */

struct linesettingstype linStat;
int                     lColor;

getlinesettings( &linStat );
lColor = getcolor();

setlinestyle( USERBIT_LINE, 0x5555, NORM_WIDTH );
setcolor( color );

line (pt1Ptr->x, pt1Ptr->y, pt2Ptr->x, pt2Ptr->y );
putpixel( pt1Ptr->x, pt1Ptr->y, color );
putpixel( pt2Ptr->x, pt2Ptr->y, color );

setlinestyle( linStat.linestyle, linStat.upattern, linStat.thickness );
setcolor( lColor );

} /* DrawDashedLine end */

/****************************************
* void MessageBox( char *msgstr )
*
* Display a message box and wait
* for the user to click any mouse button
* to acknowledge.
*
* New lines in the string are used
* as markers to find the next
* line of the message box.
*/

void MessageBox ( const char *msgstr )

{ /* MessageBox begin */

/* Local constants */

#define MAX_MSG_LINES   24
#define MAX_MSG_BUFFER  80*24

/* Local Variables */

char    *lines[MAX_MSG_LINES+2];
char    bufferstr[MAX_MSG_BUFFER];
int     index;
int     line_index;
int     exitnow;


strncpy(bufferstr, msgstr, MAX_MSG_BUFFER-1 );

line_index = 0;
lines[line_index] = bufferstr;
line_index++;
index = 0;
exitnow = FALSE;
do
  {
  if ( bufferstr[index] == '\n')
    {
    bufferstr[index] = 0;
    index++;
    lines[line_index] = &bufferstr[index];
    line_index++;
    }
  else if ( bufferstr[index] == '\0')
    exitnow = TRUE;
  else
    index++;
  }
while ( (exitnow == FALSE) &&
        (line_index < MAX_MSG_LINES) );

lines[line_index] = " >> OK << ";
line_index++;
lines[line_index] = NULL;

VertMenu( DIALOG_BOX_INDEX, lines, NULL, BK_SAVE );

} /* MessageBox end */

/****************************************
* Draw a cross hair at the location specified,
* using the color requested. The size is
* the height and width, in pixels, of the
* cross hair to be drawn.
*/


void CrossHair( VIEW_COORD_TYPE *refPtr,
                int              pasColor,
                int              pasSize )

{ /* CrossHair begin */

int sc;

sc = getcolor();
setcolor( pasColor );
line (refPtr->x - pasSize, refPtr->y, refPtr->x + pasSize, refPtr->y );
line (refPtr->x, refPtr->y - pasSize, refPtr->x, refPtr->y + pasSize );
setcolor( sc );

} /* CrossHair end */



/****************************************
* DragBox ( left, top, right, bottom,
*           VIEW_COORD_TYPE *endPos,
*           int *button );
*
* Allow the user to draw a box of the size
* left, top, right, bottom anywhere on the
* display.  Draw the box around the center
* of the current mouse position.
*
* Return the endPos of the box center and
* the button pressed.
*/

void DragBox( int left,
              int top,
              int right,
              int bottom,
              VIEW_COORD_TYPE *endPos,
              int *button,
              int color )

{ /* DragBox begin */

/****************************************
* Local variables
*/

VIEW_COORD_TYPE newPos;
int             width;
int             height;
int             exitnow;
int             sc;

/*----------
* Begin
*/
sc = getcolor();
newPos.x = (left+right)/2;
newPos.y = (top+bottom)/2;

movemouse( newPos.x, newPos.y );

width = (right-left)/2;
height = (bottom-top)/2;

Lsetwritemode( XOR_PUT );
exitnow = FALSE;

do
  {
  /*----------
  * Show the box
  */

  Lsetwritemode( XOR_PUT );
  box( newPos.x-width, newPos.y-height,
       newPos.x+width, newPos.y+height,
       color,
       NOT_FILL_BOX );
  /*----------
  * Show box center
  */
  box ( newPos.x-2, newPos.y-2,
        newPos.x+2, newPos.y+2,
        color,
        NOT_FILL_BOX );
  Lsetwritemode( COPY_PUT );

  /*----------
  * Now get mouse input
  * and check for position change
  * or button press.
  */

  do
    {
    getmousecoords( &endPos->x, &endPos->y );
    *button = mousebuttonpressed( LEFT_BUTTON );
    if (*button == TRUE)
      {
      *button = LEFT_BUTTON;
      exitnow = TRUE;
      }
    else
      {
      *button = mousebuttonpressed( RIGHT_BUTTON );
      if (*button == TRUE)
        {
        *button = RIGHT_BUTTON;
        exitnow = TRUE;
        }
      else
        *button = NEITHER_BUTTON_PRESSED; /* in Radcad.h */
      }
    if ( (exitnow == TRUE) ||
         (endPos->x != newPos.x) ||
         (endPos->y != newPos.y) )
      {
      /*----------
      * Erase old boxes
      */
      Lsetwritemode( XOR_PUT );
      box( newPos.x-width, newPos.y-height,
           newPos.x+width, newPos.y+height,
           color,
           NOT_FILL_BOX );
      box ( newPos.x-2, newPos.y-2,
            newPos.x+2, newPos.y+2,
            color,
            NOT_FILL_BOX );
      Lsetwritemode( COPY_PUT );
      newPos.x = endPos->x;
      newPos.y = endPos->y;
      exitnow  = TRUE; /* Redraw new box posn */
      ReportMousePos( newPos.x, newPos.y );
      }
    }
  while (exitnow == FALSE);
  if (*button == NEITHER_BUTTON_PRESSED)
    exitnow = FALSE;
  }
while (exitnow == FALSE);

Lsetwritemode( COPY_PUT );
setcolor(sc);
} /* DragBox end */



/****************************************
* SizeBox ( int *left, int *top,
*           int *right, int *bottom,
*           int uniform_flag );
*
* Allow the user to draw a dynamically
* resizable box.  The uniform_flag, if TRUE,
* forces uniform sizing.  Readings are taken
* as delta_x and delta_y from the mouse.
* Whichever is largest becomes the new delta
* for both x and y on the box coordinates.
* All 4 coordinates are adjusted.  So if delta_x
* is +2, then +1 is added to right and -1 is added
* to left.
*
* If uniform_flag is false, then delta_x and
* delta_y are allowed to be different.
*
* The resultant sized box is returned through the
* left, top, right, bottom pointers.  It is
* up to the caller to decide how to use the
* information (such as computing a scale
* factor).
*
* Return the button pressed, either LEFT_BUTTON
* or RIGHT_BUTTON.
*
*/

int SizeBox ( int *left, int *top,
               int *right, int *bottom,
               int uniform_flag,
               int color )

{ /* SizeBox begin */

int             delta_x;
int             delta_y;
int             button;
int             validarea;
int             sc;
int             maxx;
int             maxy;

VIEW_COORD_TYPE mouse;
VIEW_COORD_TYPE oldTop;
VIEW_COORD_TYPE oldBot;
VIEW_COORD_TYPE newTop;
VIEW_COORD_TYPE newBot;

VIEW_COORD_TYPE center;

sc = getcolor();
maxx = getmaxx();
maxy = getmaxy();

/*----------
* If uniform_flag == TRUE
* then delta_x will be forced
* to be equal to delta_y.
* There is one common redraw
* algorithm, however.
*
*/

Mshowmouse();

setwritemode( XOR_PUT );

center.x = (*right+*left)/2;
center.y = (*bottom+*top)/2;

oldTop.x = *left;
oldTop.y = *top;
oldBot.x = *right;
oldBot.y = *bottom;

/* Draw box center */

box( center.x-2, center.y-2,
     center.x+2, center.y+2, color, FALSE );

/* Draw first outline size box */

box( oldTop.x, oldTop.y, oldBot.x, oldBot.y, color, FALSE );

/* Set mouse position to box lower rit corner, where sizie ref point is */

mouse.x = oldBot.x;
mouse.y = oldBot.y;


movemouse( mouse.x, mouse.y );
do
  {
  do
    {
    button = ReadMouseButton();
    getmousecoords( &delta_x, &delta_y );
    validarea = mouseinbox( center.x+2, center.y+2,
                            maxx, maxy,
                            delta_x, delta_y );
    }
  while ( (button == NEITHER_BUTTON_PRESSED) && (validarea == FALSE));
  if (button == NEITHER_BUTTON_PRESSED)
    {
    delta_y = delta_y - mouse.y;
    delta_x = delta_x - mouse.x;
    if (uniform_flag)
      {
      delta_y = delta_x;
      }

    if ( ((iabs(delta_x)) >= 2) ||
         ((iabs(delta_y)) >= 2) )
      {
      newTop.x = oldTop.x - delta_x/2;
      newTop.y = oldTop.y - delta_y/2;
      newBot.x = oldBot.x + delta_x/2;
      newBot.y = oldBot.y + delta_y/2;
      box( oldTop.x, oldTop.y,
           oldBot.x, oldBot.y, color, FALSE );
      box( newTop.x, newTop.y,
           newBot.x, newBot.y, color, FALSE );
      oldTop.x = newTop.x;
      oldTop.y = newTop.y;
      oldBot.x = newBot.x;
      oldBot.y = newBot.y;
      movemouse( newBot.x, newBot.y );
      mouse.x = newBot.x;
      mouse.y = newBot.y;
      ReportMousePos( mouse.x, mouse.y );
      }
    }
  }
while (button == NEITHER_BUTTON_PRESSED );

box( oldTop.x, oldTop.y,
     oldBot.x, oldBot.y, color, FALSE );

box( center.x-2, center.y-2,
     center.x+2, center.y+2, color, FALSE );


*left   = oldTop.x;
*top    = oldTop.y;
*right  = oldBot.x;
*bottom = oldBot.y;

setcolor( sc );
setwritemode( COPY_PUT );

return( button );

} /* SizeBox end */

/****************************************
* The height of the menu bar
*
*/

int MenuBarHeight( void )
{

return ( textheight("W")+2*BEVEL_ADJUST );

}
