
/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... MENU.H          */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 12-FEB-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Declarations    */
/*  related to menu selections.                         */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
13-MAR-90 MR Add StatusLine, StatusLineY, mline, and mlineto
             functions, which deal with the MOUSE as well as
             the graphical user interface.
09-APR-90 MR Final cleanup for LAB5.

*\r*/


/****************************************
* Set definition indicating that the
* menu header has been included
*/

#define MENU

/* Initialize the menu driver package.
 * Inform it of the current display resolution and
 * colors.  Allow it to initialize it's module variables
 * for controling the state of the mouse, message
 * areas and so on.
 */

void InizMenuDriver( GRAPH_DESCR_TYPE *gPtr );

/*

Allow the user to make a menu selection on a graphics
screen. The caller passes a pointer to an array of string
pointers.  The last entry must be a NULL pointer.  It is up
to the caller to make certain the menu will fit on the
display in the current graphics mode.  GrafMenuSelect will
clear the top text line of the display, and output the menu
selections across the top.  The user may then press the left
and right arrow keys to highlight a selection and press
ENTER to select it.  If the cancelKey is detected, then
CANCEL_MENU is returned (a -1 flag).  Else an integer from 0
to the number of menu selections-1 is returned.
Unpredictable results will occur if the caller requests a
larger menu than will fit across the top of the display.

*/

#define CANCEL_MENU     -1      /* User hit the cancelKey       */
#define MAX_MENU_ITEMS   40     /* Can only handle 40 items now */
#define ERASE_ALWAYS    -2      /* Erase item == -1? Then always*/
                                /* erase the menu on exit       */
#define ERASE_NEVER     -3      /* Don't erase menu on exit     */

int GrafMenuSelect( char *items[], int eraseitem );

/*
8 pixels hight on status line
*/

#define STATUS_LINE_HEIGHT      8

/*
 Display a status message at the bottom of the screen.
*/

void StatusLine( char *msg, int color );
/*
 Display a status message at any Y offset of the screen.
*/

void StatusLineY( char *msg, int color, int y );

void ClearStatusLine( void );

/*
Function to test for valid menu selection
*/

#define NO_SELECTION -1
#define CANCEL_VERT_MENU -2     /* Don't want anything from this menu */

int findselection( VIEW_COORD_TYPE *mouse,
                   VIEW_COORD_TYPE *topLeft,
                   VIEW_COORD_TYPE *botRight,
                   int entries );
/*

This function is for drawing a line, while using the mouse.
It simply calls the turbo C graphics line function, in between
hidemouse, showmouse calls.  This is best used when making
frequent calls to draw lines between mouse movements.  When
drawing a great number of graphics primitives, it is best to
simply hide the mouse and then make all the calls to update
the display, then show mouse.  For user interaction, mline
is best, however.

*/

void mline( int x1, int y1, int x2, int y2 );
/*

This function is for drawing a line relative to CP, while using the mouse.
It simply calls the turbo C graphics lineto function, in between
hidemouse, showmouse calls.  This is best used when making
frequent calls to draw lines between mouse movements.

*/

void mlineto( int x2, int y2 );
/*
Draw a box with bevels on the edges for depth effect,
use for information presentation of some sort (e.g. menus, dialog boxes)

*/

#define BK_SAVE         TRUE
#define BK_OVRWRITE     FALSE

void *infobox  ( int x1,
                 int y1,
                 int x2,
                 int y2,
                 int color,
                 int tl_edge,
                 int rb_edge,
                 int bk_save );


/* Draw a box
 * in the color specified
 * if fill == TRUE then fill the
 * box
 */

#define FILL_BOX        TRUE
#define NOT_FILL_BOX    FALSE

void box( int tx,
          int ty,
          int bx,
          int by,
          int color,
          int fill );

/* Draw a diamond
 * in the color specified
 * if fill == TRUE then fill the
 * diamond
 */

#define FILL_DIAMOND        TRUE
#define NOT_FILL_DIAMOND    FALSE

void diamond( int cx,
              int cy,
              int height,
              int width,
              int color,
              int fill );


/* Manage the state of the mouse
 * and allow user to test the
 * mouse state, on or off
 */

void Mhidemouse( void );
void Mshowmouse( void );
int Mouseoff( void );

/* Vertical menu driver
 * interface.
 */

#define VM_SELECT       'S'
#define VM_DISABLED     'D'
#define VM_NO_SELECT    'N'

#define DIALOG_BOX_INDEX        -1

int VertMenu( int index,
              char *items[],
              char *selects,
              int  save );
/*
 * Purge the buffered mouse button presses,
 * when we're not ready for them
 */

void MpurgeButtons( void );

/* Allow the user to define menu colors
 */

#define NO_COLOR_CHANGE -1

void SetMenuSpeed( int normal ); /* If normal == TRUE, normal menus */
                                 /* If normal == FALSE, fast menus  */

void SetMenuColors( int topedgecolor,
                    int botedgecolor,
                    int facecolor,
                    int textcolor,
                    int prmptcolor,
                    int bcolor,
                    int normal ); /* If normal == TRUE, normal menus */
                                  /* If normal == FALSE, fast menus  */

/* Report the position of the mouse
 */

#define ERASE_MOUSE_FIELD -1

void ReportMousePos( int x, int y );

/* Work on just the drawing
 */

void SetViewDrawingArea( void );

/* Work on drawing or select menus
 */

void SetViewMenuArea( void );

/****************************************
* SetClippedViewport( int *xoffset, int *yoffset )
*
* Set a clipped viewport for drawing the
* objects.  Return to the caller an x and
* y offset to be used to adjust the
* object coordinates as they are output to
* the display.
*/

void SetClippedViewport( int *xoffset, int *yoffset );


/****************************************
 * Make a brief audio acknowledgement
 * of a menu selection
 */

#define AUDIO_SEL_TONE 640
#define AUDIO_ERR_TONE 120
#define AUDIO_SEL_DURATION 5
#define AUDIO_ERR_DURATION 15

void AudioSelect( void );

void AudioError( void );

/****************************************
 * GetPoint( VIEW_COORD_TYPE *prevPt,
 *           VIEW_COORD_TYPE *newPt,
 *           int             *button,
 *           int              showcoords );
 *
 * Get a point via mouse input. Return the
 * button pressed, either LEFT_BUTTON or
 * RIGHT_BUTTON.  Return the coordinate
 * in the newPt.
 * If prevPt is not NULL then draw a rubberband
 * line until the user hits a button.
 * If left button is pressed, leave the
 * line on the screen.  If the right
 * button is pressed, erase the rubber
 * band line.  If showcoords is true
 * then report the mouse position while
 * drawing.
 */

#define SHOW_COORDS             TRUE
#define DONT_SHOW_COORDS        FALSE

void GetPoint( VIEW_COORD_TYPE  *prevPt,
               VIEW_COORD_TYPE *newPt,
               int             *button,
               int             showcoords );

/****************************************
 * Set a point in the current color,
 * between hidmouse and showmouse
 * calls.
 */

void msetpoint( int mx, int my );

/****************************************
* Clear the main drawing area for a display
* update.
*/

void ClearWorkArea( void );

/****************************************
* Output a status message and then
* accept user input:
*/


void StatusLineGet( char *msg, char *outstr, int fcolor,
                    int bcolor, int inputlen );


#define NEITHER_BUTTON_PRESSED -1


/****************************************
*
* int ReadMouseButton( void );
*
* If left button pressed return LEFT_BUTTON,
* else if right button pressed return
* RIGHT_BUTTON, else return NEITHER_BUTTON_PRESSED.
*
*/

int ReadMouseButton( void );

/****************************************
* DrawDashedLine - Draw a dashed line
* from point 1 to 2, in the color
* specified.
*/

void DrawDashedLine( VIEW_COORD_TYPE *pt1Ptr,
                     VIEW_COORD_TYPE *pt2Ptr,
                     int             color );
/****************************************
* void MessageBox (const char *msgstr )
*
* Display an information message box
* on the screen. A quick and easy way
* to put a message up. The user simply
* passes a pointer to a string, we do
* the rest. If the user wants multiple lines,
* each line is separated by a newline char.
* The end of the message must be null
* terminated. Maximum size is 24 lines
* of 80 column text.
*/

void MessageBox ( const char *msgstr );

/****************************************
* Draw a cross hair at the location specified,
* using the color requested. The size is
* the height and width, in pixels, of the
* cross hair to be drawn.
*/


void CrossHair( VIEW_COORD_TYPE *refPtr,
                int              pasColor,
                int              pasSize );


void DragBox( int left,
              int top,
              int right,
              int bottom,
              VIEW_COORD_TYPE *endPos,
              int *button,
              int color );

#define RECTANGLE_FLAG  FALSE
#define BOX_FLAG        TRUE

int SizeBox ( int *left, int *top,
               int *right, int *bottom,
               int uniform_flag,
               int color );

int MenuBarHeight( void );


