/*   mouse.c -- Routines to support a Microsoft compatible mouse.
 *   These functions also provide keyboard emulation of a mouse,
 *   if one does not exist. This package assumes that you are
 *   running under graphics mode. To move the emulated cursor use
 *   the arrow keys on the keyboard and the INS and DEL keys as
 *   the left and right mouse buttons, respectively. The gray +
 *   and - keys can be used to change the amount the emulated
 *   mouse cursor is moved by.
 */

/******************************
* This package was provided by the isntructor
* for the use of the whole programming
* class.  I don't like it. If it finds no
* mouse present at initialization, it attempts
* to emulate one, which promptly locks up your computer!
* My software detects this condition and gives you
* a safe exit option, before a crash occurs.
* I've been wanting to rewrite the mouse driver,
* making it smart enough to accept TAB keys to
* highlight the next menu item, ARROW keys to
* move the mouse pointer ALT-ARROW and CONTROL-ARROW
* keys to move the mouse pointer farther/faster...
* and Alt-Alphabet keys to select menu items automatically.
* But this is rather low on my priority list. Too much
* other work.  Below is the hierarchy chart for this
* tersley coded, mouse hack:


- mouse                 


- initmouse             
>>  resetmouse            
>>  resetmouse            
>>  movemouse             
>>  showmouse             
>>  _initcursor           
>>  showmouse             


- resetmouse            
>>  mouse                 


- movemouse             
>>  hidemouse             
>>  showmouse             
>>  mouse                 


- hidemouse             
>>  _togglecursor          
>>  mouse                 


- showmouse             
>>  _togglecursor          
>>  mouse                 


- getmousecoords          
>>  mouse                 


- getmousemovement          
>>  mouse                 


- mousebuttonreleased          
>>  _simulatedbuttonreleased          
>>  testbutton            


- mousebuttonpressed          
>>  _simulatedbuttonpressed          
>>  testbutton            


- testbutton            
>>  mouse                 
>>  mouse                 


- _simulatedbuttonpressed          
>>  _getkbinteraction          


- _simulatedbuttonreleased          
>>  _getkbinteraction          


- mouseinbox            


- getinput              
>>  _getkbinteraction          
>>  mousebuttonpressed          
>>  mousebuttonreleased          
>>  mousebuttonreleased          


- waitforinput          
>>  getinput              


- _getkbinteraction          
>>  _getkb                
>>  hidemouse             
>>  showmouse             
>>  showmouse             
>>  showmouse             
>>  showmouse             


- _getkb                


- _initcursor           
>>  _drawcursor           


- _togglecursor          


- _drawcursor           

*/


#include <alloc.h>
#include <math.h>
#include <dos.h>
#include <graphics.h>
#include <process.h>
#include <stdio.h>
#include <conio.h>
#include "mouse.h"

#define TRUE  	1
#define FALSE 	0
#define _MAXINC 32  /* Largest increment amount of emulated mouse cursor */
int mouseexists;    /* Internal variable set true if a mouse driver is */
                    /* detected during initialization. This variable is */
                    /* used to select between the mouse code (if a */
                    /* mouse exists) and the keyboard's emulated mouse. */
/*  All of the variables and functions that begin with an underscore
 *  are used to simulate a mouse when one is not present.
 */
int _cx, _cy;       /* Internal variables used to maintain */
                    /* the cursor location when the mouse is not used */
int _cinc;          /* Internal variable used for increment */
                    /* amount of the nonmouse cursor */
char *_cursor;      /* Points to image of the emulated mouse */
char *_undercursor; /* Area saved under emulated mouse */
int _cursoron;      /* TRUE if cursor currently visible */
int _cminx, _cmaxx; /* Minimum, maximum x coordinates for internal cursor */
int _cminy, _cmaxy; /* Minimum, maximum y coordinates for internally */
                    /* maintained cursor */
int _lbuttonpress;  /* True if simulated left or right */
int _rbuttonpress;  /* button is pressed */
int _numlpress;     /* Count the number of button presses */
int _numrpress;
int _numlrelease;   /* Count the number of button releases */
int _numrrelease;
int _cdeltax, _cdeltay;  /* keeps track of emulated mouse's movements */

/*  This routine provides the communication between the mouse driver and
 *  an application program. There are several predefined mouse functions
 *  supported by the Microsoft mouse -- see accompanying text. Parameters
 *  are sent back and forth to the mouse driver through the ax, bx, cx
 *  and dx registers.
 */
void mouse(int *m1, int *m2, int *m3, int *m4)
{
    union REGS inregs, outregs;

    inregs.x.ax = *m1;   inregs.x.bx = *m2;
    inregs.x.cx = *m3;   inregs.x.dx = *m4;
    int86(0x33,&inregs,&outregs);
    *m1 = outregs.x.ax;  *m2 = outregs.x.bx;
    *m3 = outregs.x.cx;  *m4 = outregs.x.dx;
}

/*  Call this routine at the beginning of your program, after the graphics
 *  adapter has been initialized. It will initialize the mouse and display
 *  the mouse cursor at the top, left of the screen, if one is present.
 *  Otherwise it will cause the emulated mouse to appear and the keyboard
 *  will be used to move the "mouse" cursor.
 */
int initmouse(void)
{
    int gmode;
    char far *memory = (char far *)0x004000049L;

    mouseexists = TRUE;
    if (resetmouse()) {                /* If mouse reset okay, assume */
        gmode = getgraphmode();        /* mouse exists. Test if Hercules */
        if (gmode == HERCMONOHI) {     /* is used. If so, patch memory */
            *memory = 0x06;            /* location 40h:49h with 6. */
            resetmouse();
        }
        movemouse(0,0);                /* Mouse exists and draw the */
        showmouse();                   /* cursor on the screen at (0,0) */
        return(1);                     /* Return a success flag */
    }
    else {                             /* No mouse found -- emulate a */
        mouseexists = FALSE;           /* mouse using the keyboard */
        _cx = _cy = 0;                 /* The functionality is the */
        _cinc = 1; _cminx = 0;         /* same as the real thing */
        _cmaxx = getmaxx()-1;          /* Therefore, set the emulated */
        _cminy = 0;                    /* cursor to the top left of the */
        _cmaxy = getmaxy()-1;          /* screen, set its bounds to the */
        _cdeltax = _cdeltay = 0;       /* full screen and initialize its */
        _lbuttonpress = FALSE;         /* mouse movement counters, and */
        _rbuttonpress = FALSE;         /* initialize all button states */
        _numlpress = 0;                /* to false */
        _numrpress = 0;                /* Set simulated mouse button */
        _numlrelease = 0;              /* press and release counters to */
        _numrrelease = 0;              /* zero */
        _initcursor();                 /* Create cursor image */
        _cursoron = FALSE;
        showmouse();                   /* Call showmouse() to display it */
        return(0);                     /* return a no-mouse found flag */
    }
}

/*  Resets the mouse cursor to: screen center, mouse hidden, using arrow
 *  cursor and with minimum and maximum ranges set to full virtual screen
 *  dimensions. If a mouse driver exists, this function returns a -1
 *  otherwise it returns a 0.
 */
int resetmouse(void)
{
    int m1, m2, m3, m4;

    m1 = RESET_MOUSE;
    mouse(&m1,&m2,&m3,&m4);
    return(m1);
}

/*  Moves the mouse to the location (x,y)
 */
void movemouse(int x, int y)
{
    int m1, m2;

    if (!mouseexists) {
        hidemouse();   	         /* Erase the current mouse cursor */
        _cx = x;  _cy = y;       /* Update the mouse cursor's location */
        showmouse();             /* Display mouse at the new location */
        _cdeltax = _cdeltay = 0; /* Reset the mouse movement variables */
        return;
    }
    m1 = SET_MOUSE_COORD;
    if (getmaxx() == 319) x *= 2; /* Adjust between virtual and actual */
    mouse(&m1,&m2,&x,&y);         /* coordinates if necessary */
}

/*  Removes the mouse cursor from the screen. Call this function before you
 *  write or draw anything to the screen. It is also a good idea to turn off
 *  the mouse at the end of a program. Use showmouse() to restore the mouse
 *  on the screen. The mouse movement will be maintained while the mouse is
 *  not visible. Due to a peculiarity of the mouse driver, make sure you
 *  don't call hidemouse() if the mouse is not already visible. See text
 *  discussion for more on this.
 */
void hidemouse(void)
{
    int m1, m2, m3, m4;

    if (!mouseexists)                   /* Mouse doesn't exist, so turn */
        _togglecursor();                /* off the emulated cursor */
    else {
        m1 = HIDE_MOUSE;                /* Invoke the hide mouse function */
        mouse(&m1,&m2,&m3,&m4);
    }
}

/*  Display the mouse cursor. Normally, you should not call this
 *  function if the mouse is already visible. The keyboard mouse
 *  is clipped to the minimum and maximum ranges in this routine.
 */

void showmouse(void)
{
    int m1, m2, m3, m4;

    if (!mouseexists)             /* If the mouse doesn't exist, */
        _togglecursor();          /* turn on the emulated cursor */
    else {
        m1 = SHOW_MOUSE;
        mouse(&m1,&m2,&m3,&m4);   /* Display the mouse cursor */
    }
}

/*  Get the current location of the mouse cursor */
void getmousecoords(int *x, int *y)
{
    int m1, m2;

    if (!mouseexists) {
        *x = _cx; *y = _cy;	 /* The position of the emulated */
        return;                  /* mouse is given by _cx and _cy */
    }
    m1 = GET_MOUSE_STATUS;
    mouse(&m1,&m2,x,y);
    if (getmaxx() == 319) (*x) /= 2; /* Adjust for virtual coordinates */
}                                    /* of the mouse */

/*  Find out how far the mouse has moved since the last call to this
 *  function
 */
void getmousemovement(int *x, int *y)
{
    int m1, m2;

    if (!mouseexists) {                 /* If the mouse doesn't exist, */
        *x = _cdeltax;  *y = _cdeltay;  /* use emulated mouse locators */
        _cdeltax = _cdeltay = 0;        /* Reset counters for next call */
        return;                         /* to getmousemovement */
    }
    m1 = GET_MOUSE_MOVEMENT;
    mouse(&m1,&m2,x,y);
    if (getmaxx() == 319) (*x) /= 2;    /* Adjust for virtual coordinates */
}

/*  Test if a button has been released since the last call to this
 *  function. If so, return 1, otherwise return 0. In the keyboard mode,
 *  button release is simulated by striking on the button key again.
 */
int mousebuttonreleased(int whichbutton)
{
    if (!mouseexists) return(_simulatedbuttonreleased(whichbutton));
    return(testbutton(CHECK_BUTTON_RELEASE,whichbutton));
}

/*  Return a 1 if the mouse button specified has been pressed since the
 *  last check with this function. If the button has not been pressed,
 *  return a 0.
 */
int mousebuttonpressed(int whichbutton)
{
    if (!mouseexists) return(_simulatedbuttonpressed(whichbutton));
    return(testbutton(CHECK_BUTTON_PRESS,whichbutton));
}

/*  Called by mousebuttonpressed() and mousebuttonreleased() to explicitly
 *  test the mouse button states. The function returns TRUE if the
 *  specified mouse button (in whichbutton) performed the specified
 *  action (as indicated by testtype). Otherwise the function returns
 *  FALSE which means that the action tested for did not occur.
 */
int testbutton(int testtype, int whichbutton)
{
    int m1, m2, m3, m4;

    m1 = testtype;
    if (whichbutton == LEFT_BUTTON || whichbutton == EITHER_BUTTON) {
        m2 = LEFT_BUTTON;
        mouse(&m1,&m2,&m3,&m4);
	if (m2) return(TRUE);     /* Return TRUE if the action occurred */
    }
    if (whichbutton == RIGHT_BUTTON || whichbutton == EITHER_BUTTON) {
        m1 = testtype;
        m2 = RIGHT_BUTTON;
        mouse(&m1, &m2, &m3, &m4);
        if (m2) return(TRUE);     /* Return TRUE if the action occurred */
    }
    return(FALSE);		  /* Return FALSE as a catch all */
}

/* Returns TRUE if the specified simulated button has been pressed
 */
int _simulatedbuttonpressed(int whichbutton)
{
    if ((whichbutton == LEFT_BUTTON || whichbutton == EITHER_BUTTON) &&
				                 _numlpress) {
        _numlpress--;
        return(TRUE);
    }
    else if ((whichbutton == RIGHT_BUTTON || whichbutton == EITHER_BUTTON)
				     && _numrpress) {
        _numrpress--;
        return(TRUE);
    }
/*  If there isn't already a button pressed, check and see if
 *  the user just pressed one, and if so repeat the tests above.
 */
    else if (_getkbinteraction(whichbutton) < 0) {
        if (whichbutton == LEFT_BUTTON || whichbutton == EITHER_BUTTON)
            return(TRUE);
        else if (whichbutton == RIGHT_BUTTON || whichbutton == EITHER_BUTTON)
            return(TRUE);
    }
    return(FALSE);     /* Return a value that the button was not pressed */
}

/* Returns TRUE if the specified simulated button has been released
 */
int _simulatedbuttonreleased(int whichbutton)
{
    if ((whichbutton == LEFT_BUTTON || whichbutton == EITHER_BUTTON) &&
				                 _numlrelease) {
        _numlrelease--;
        return(TRUE);
    }
    else if ((whichbutton == RIGHT_BUTTON || whichbutton == EITHER_BUTTON)
				     && _numrrelease) {
        _numrrelease--;
        return(TRUE);
    }
/*  If there isn't already a button released, check and see if
 *  the user just released one, and if so repeat the tests above.
 */
    else if (_getkbinteraction(whichbutton) < 0) {
        if (whichbutton == LEFT_BUTTON || whichbutton == EITHER_BUTTON)
            return(TRUE);
        else if (whichbutton == RIGHT_BUTTON || whichbutton == EITHER_BUTTON)
            return(TRUE);
    }
    return(FALSE);     /* Return a value that the button was not pressed */
}

/*  Test if the mouse cursor is within the box specified. Returns TRUE
 *  if the mouse is in the box; otherwise the function returns FALSE.
 */
int mouseinbox(int left, int top, int right, int bottom, int x, int y)
{
    return(((x >= left) && (x <= right) && (y >= top) && (y <= bottom)) ? 1 : 0);
}


/*  Returns a character if a key has been pressed, or -1 if a mouse
 *  button has been pressed, or a zero if none of the above. If a
 *  mouse exists, this routine favors any keyboard action.
 */
int getinput(int whichbutton)
{
    int c;

    if (!mouseexists) {
        c = _getkbinteraction(whichbutton);
        return(c);
    }
    if (kbhit())            /* Check if a key has been pressed */
        return(getch());    /* return the character */
    else {
        if (mousebuttonpressed(whichbutton)) {
            while (!mousebuttonreleased(whichbutton)) ;
            return(-1);
        }
        else if (mousebuttonreleased(whichbutton))
            return(-1);
    return(0);
    }
}

/*  Continue to call getinput() until a button or key has been pressed */
int waitforinput(int whichbutton)
{
    int c;

    while ((c=getinput(whichbutton)) == 0) ;
    return(c);
}

/*  This routine is only used if the mouse does not exist. It updates
 *  the location of the cursor whenever an arrow key is pressed or
 *  will set the button pressed flags if a button keys is pressed. In
 *  addition, it will change the cursor increment amount if the plus or
 *  minus keys are pressed. If the "button" specified is pressed or was
 *  already pressed, the function returns -1. If an arrow key is pressed
 *  then a 0 is returned, which means that the caller does not have
 *  to perform any action. Finally, if some other key is pressed, then
 *  its value is returned.
 */
int _getkbinteraction(int whichbutton)
{
    int c;

    if (kbhit()) {
        c = _getkb();
        hidemouse();         /* Emulated mouse may be moved */
        switch(c) {
          case 0x5200 :      /* INS key -- Emulate left mouse button press */
            _lbuttonpress = (_lbuttonpress) ? FALSE : TRUE;
            showmouse();
            if (whichbutton == LEFT_BUTTON || whichbutton == EITHER_BUTTON)
                return(-1);  /* Return mouse button click signal */
            if (_lbuttonpress) _numlpress++;
                else _numlrelease++;
            return(0);       /* Wrong mouse "button" pressed */
          case 0x5300 :      /* DEL key emulates right mouse button */
            showmouse();
            _rbuttonpress = (_rbuttonpress) ? FALSE : TRUE;
            if (whichbutton == RIGHT_BUTTON || whichbutton == EITHER_BUTTON)
                return(-1);  /* Return mouse button signal */
            if (_rbuttonpress) _numrpress++;
               else _numrrelease++;
            return(0);      /* Wrong mouse "button" pressed */
          case 0x002B :     /* '+' key: increase the mouse movement amount */
            _cinc = (_cinc < _MAXINC) ? _cinc + 6 : _MAXINC;
            break;
          case 0x002D :     /* '-' key: decrease the mouse movement amount */
            _cinc = (_cinc > 1 + 6) ? _cinc - 6 : 1;
            break;
          case 0x4800 :           /* UP key moves the cursor up by the */
            if ((_cy -= _cinc) < _cminy) _cy = _cminy;
            _cdeltay -= _cinc;    /* increment amount and clip; also */
            break;                /* decrement movement amount */
          case 0x5000 :           /* DOWN key */
            if ((_cy += _cinc) > _cmaxy) _cy = _cmaxy;
            _cdeltay += _cinc;
            break;
          case 0x4B00 :           /* LEFT key */
            if ((_cx -= _cinc) < _cminx) _cx = _cminx;
            _cdeltax -= _cinc;
            break;
          case 0x4D00 :           /* RIGHT key */
            if ((_cx += _cinc) > _cmaxx) _cx = _cmaxx;
            _cdeltax += _cinc;
            break;
          case 0x4700 :           /* HOME key */
            if ((_cy -= _cinc) < _cminy) _cy = _cminy;
            if ((_cx -= _cinc) < _cminx) _cx = _cminx;
            _cdeltax -= _cinc;   _cdeltay -= _cinc;
            break;
          case 0x4900 :           /* PGUP key */
            if ((_cy -= _cinc) < _cminy) _cy = _cminy;
            if ((_cx += _cinc) > _cmaxx) _cx = _cmaxx;
            _cdeltax += _cinc;  _cdeltay -= _cinc;
            break;
          case 0x4F00 :           /* END key */
            if ((_cy += _cinc) > _cmaxy) _cy = _cmaxy;
            if ((_cx -= _cinc) < _cminx) _cx = _cminx;
            _cdeltax -= _cinc;  _cdeltay += _cinc;
            break;
          case 0x5100 :           /* PGDN key */
            if ((_cy += _cinc) > _cmaxy) _cy = _cmaxy;
            if ((_cx += _cinc) > _cmaxx) _cx = _cmaxx;
            _cdeltax += _cinc;  _cdeltay += _cinc;
            break;
          default : showmouse(); return(c);
        }
        showmouse();    /* Restore mouse to possibly new position */
    }
    return(0);          /* Tell caller not to take any action */
}

/*  Low-level keyboard input that retrieves the some of the extended
 *  codes produced by the arrow keys and grey keys. This routine does
 *  not support all of the extended key codes.
 */
int _getkb(void)
{
    int ch1, ch2;

    ch1 = getch();
    if (ch1 == 0) {        /* Arrow keys have two character sequences */
        ch2 = getch();     /* where the first character is a zero */
        ch2 = ch2 << 8;    /* Combine these two characters into one */
        ch2 |= ch1;        /* and return the value */
        return(ch2);
    }
    else
        return(ch1);
}

/*  Creates the image of the simulated mouse. It calls drawcursor()
 *  which is a routine that you must supply to actually draw the cursor.
 *  Note: A drawcursor() routine is provided below which draws an arrow.
 */
void _initcursor(void)
{
/*  Allocate space for the emulated mouse cursor and for the space
 *  below it. If not enough memory, quit the program.
 */
    _cursor = malloc(imagesize(0,0,CURSWIDTH,CURSHEIGHT));
    _undercursor = malloc(imagesize(0,0,CURSWIDTH,CURSHEIGHT));
    if (_cursor == NULL || _undercursor == NULL) {
        closegraph();
        printf("Not enough memory for program.\n");
        exit(1);
    }
/*  Save the image of the screen where the cursor image will be created.
 *  Clear this space and call drawcursor() to draw the cursor.
 */
    getimage(_cx,_cy,_cx+CURSWIDTH,_cy+CURSHEIGHT,_undercursor);
    setlinestyle(SOLID_LINE,0,0);
    setfillstyle(SOLID_FILL,BLACK);
    bar(_cx,_cy,_cx+CURSWIDTH,_cy+CURSHEIGHT);
    _drawcursor(_cx,_cy);
/*  Save the image of the cursor and overwrite the screen area where the
 *  cursor is with the original screen image.
 */
    getimage(_cx,_cy,_cx+CURSWIDTH,_cy+CURSHEIGHT,_cursor);
    putimage(_cx,_cy,_undercursor,COPY_PUT);
}

/*  Used by the simulated mouse to turn the mouse cursor on and off.
 *  The viewport settings may have changed so temporarily reset them
 *  to the full screen while the cursor image is displayed or erased.
 */
void _togglecursor(void)
{
    struct viewporttype vp;
    int oldx, oldy;

    getviewsettings(&vp);                       /* Save view settings */
    oldx = getx();   oldy = gety();             /* and current position */
    setviewport(0,0,getmaxx(),getmaxy(),1);
    if (_cursoron) {                            /* To erase the cursor */
        putimage(_cx,_cy,_undercursor,COPY_PUT);/* overwrite it with */
        _cursoron = FALSE;                      /* the saved image of */
    }                                           /* the screen */
    else {                                      /* To draw the cursor */
        getimage(_cx,_cy,_cx+CURSWIDTH,_cy+CURSHEIGHT,_undercursor);
        putimage(_cx,_cy,_cursor,COPY_PUT);     /* first save the area */
        _cursoron = TRUE;                       /* where the cursor */
    }                                           /* will be and then */
                                                /* display the cursor */
/*  Reset the viewport settings and its current position */
    setviewport(vp.left,vp.top,vp.right,vp.bottom,1);
    moveto(oldx,oldy);
}

/*  This routine draws a cursor if a mouse does not exist. It draws
 *  a small arrow. If you want a different cursor, just change this
 *  routine. CURSWIDTH and CURSHEIGHT define the dimensions of cursor.
 */
void _drawcursor(int x, int y)
{
    setcolor(getmaxcolor());
    line(x+1,y+1,x+CURSWIDTH-1,y+CURSHEIGHT-1);
    line(x+2,y+1,x+CURSWIDTH-1,y+CURSHEIGHT-2);
    line(x+1,y+2,x+CURSWIDTH-2,y+CURSHEIGHT-1);
    line(x+2,y+1,x+2,y+5);
    line(x+1,y+1,x+1,y+5);
    line(x+1,y+1,x+5,y+1);
    line(x+1,y+2,x+5,y+2);
}

