/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... OBJECTS.C       */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 30-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Object manip-   */
/*  ulation handler                                     */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
30-MAR-90 MR Transplanted from LAB5.C for modularity (and my
             SANITY).
31-MAR-90 MR Bust huge switch/case into separate routines
             for modularity.
02-APR-90 MR Add object rotation support.
02-APR-90 MR Add object 3-d support.
05-APR-90 MR Spin an object that's already of SPIN3D_OP type by
             FIRST deleting all but the first edge from
             the structure.  This will restore the original
             edge, at the current screen position.
07-APR-90 MR Move COPY function here.  Make place for ANIMATION
             function too.  Not sure how to animate yet.
09-APR-90 MR Final cleanup for LAB5


*\r*/

/* ---------- System include files -------------------- */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <graphics.h>
#include <alloc.h>
#include <dos.h>

/* ---------- User include files ---------------------- */

#include "lab.h"
#include "mouse.h"
#include "matrix3d.h"
#include "menu.h"
#include "spclmenu.h"
#include "objdefs.h"
#include "objops.h"
#include "RADCAD.H"               /* Access to sysinfo structure */


/* ---------- Module globals -------------------------- */

#include "objectsv.h"

/* ---------- Module procedure prototpyes ------------- */

void objectsMove( SYSINFO_TYPE *si );
void objectsScale(SYSINFO_TYPE *si );
void objectsUndo ( SYSINFO_TYPE *si );
void objectsUnerase( SYSINFO_TYPE *si );
void objectsCleanup( SYSINFO_TYPE *si );
void objectsRotate( SYSINFO_TYPE *si );
void objects3D( SYSINFO_TYPE *si );
void objectsCopy( SYSINFO_TYPE *si );
void objectsAnimate( SYSINFO_TYPE *si );



/****************************************
 * ObjectsHandler - This menu is used to
 * manipulate objects already created.
 *
 * OBJECT_ROTATE - Select an object and specify
 * rotation about 1 or more axes
 * OBJECT_MOVE - Move the object on the screen
 * OBJECT_SCALE - Scale the object, new size
 * OBJECT_3_D - Make axis and rotate object
 * OBJECT_UNDO - Undo object operation
 * OBJECT_UNDELETE - Review deleted objects and undelete
 * OBJECT_CLEANUP - Clean up buffer memory, where Undo and
 * Delete buffers are.
 * OBJECT_HELP - Help with object menu
 * OBJECT_DONE - Done with objects
 *
 */

int        ObjectsHandler( SYSINFO_TYPE *si )

{ /* ObjectsHandler begin */

int exitnow;
int status;

exitnow = FALSE;

do
  {
  status = VertMenu( MAIN_OBJECTS,
                     objectsMenu,
                     objectsMenuSel, BK_SAVE );
  switch( status )
    {
    case( OBJECT_ROTATE ):
      if (si->TotalActiveObjects != 0)
        objectsRotate( si );
      else
        noObjects( "to Rotate", si->ErrorColor);
      break;

    case( OBJECT_MOVE ):
      if (si->TotalActiveObjects != 0)
        objectsMove( si );
      else
        noObjects( "to Move", si->ErrorColor );
      break;

    case( OBJECT_SCALE ):
      if (si->TotalActiveObjects != 0)
        objectsScale( si );
      else
        noObjects("to Scale", si->ErrorColor );
      break;

    case( OBJECT_COPY):
      if (si->TotalActiveObjects != 0)
        objectsCopy( si );
      else
        noObjects("to Copy", si->ErrorColor );
      break;

    case( OBJECT_ANIMATE):
      if (si->TotalActiveObjects != 0)
        objectsAnimate( si );
      else
        noObjects("to Animate", si->ErrorColor );
      break;

    case( OBJECT_3_D ):
      if (si->TotalActiveObjects != 0)
        objects3D( si );
      else
        noObjects("to Spin-3D", si->ErrorColor );
      break;

    case( OBJECT_UNDO ):
      if (si->TotalActiveObjects != 0)
        objectsUndo( si );
      else
        noObjects("to Undo", si->ErrorColor );
      break;

    case( OBJECT_UNERASE ):
      if (si->TotalActiveObjects != si->TotalObjects)
        objectsUnerase( si );
      else
        noObjects("to Unerase", si->ErrorColor );
      break;

    case( OBJECT_CLEANUP ):
      if (si->TotalObjects != 0)
        objectsCleanup( si );
      else
        noObjects("to Cleanup", si->ErrorColor );
      break;

    case( OBJECT_REDRAW ):
      RedrawDisplay( si->ObjectColor );
      break;

    case( OBJECT_HELP ):
      VertMenu( MAIN_OBJECTS, objectHelp, objectHelpSel, BK_OVRWRITE );
      break;

    default:
      exitnow = TRUE;
      break;
    }
  }
while( exitnow == FALSE );

ClearStatusLine();

return( status );
} /* ObjectsHandler end */



/****************************************
* OBJECT_MOVE handler.
* Allow the user to interactively
* select objects and TRANSLATE those
* dudes elsewhere
*
*/

void objectsMove( SYSINFO_TYPE *si )
{ /* objectsMove begin */

int             exitnow;
int             index;
int             button;
VIEW_COORD_TYPE pos;
float           tx;
float           ty;
float           tz;
OBJECT_TYPE     *obPtr;

exitnow = FALSE;
MpurgeButtons();
do
  {
  index = SelectObject("to MOVE");
  if (index != OBJECT_NOT_SELECTED)
    {
    obPtr = si->ObjectList[index];
    FrameObject( obPtr, TRUE );
    StatusLine("LEFT-New object center, RIGHT-Cancel",
                si->MenuColor);
    DragBox( obPtr->sMin.x, obPtr->sMin.y,
             obPtr->sMax.x, obPtr->sMax.y,
             &pos, &button, si->ObjectColor );
    FrameObject( obPtr, TRUE );
    Mshowmouse();
    if (button == LEFT_BUTTON)
      {
      tz = 0.0;
      ty = (float)pos.y - obPtr->vCenter.y;
      tx = (float)pos.x - obPtr->vCenter.x;
      StatusLine("Computing translation", si->MenuColor);
      TranslateObject( &si->ObjectList[index],
                       tx, ty, tz );
      }
    else
      exitnow = TRUE;
    }
  else
    exitnow = TRUE;

  }
while( exitnow == FALSE );

ClearStatusLine();
Mshowmouse();

} /* objectsMove end */



/****************************************
* OBJECT_SCALE handler.
*
* Allow the user to specify UNIFORM or
* DIFFERENTIAL scaling.  He then moves
* the mouse pointer to set the new box
* size.  It is always drawn around the
* box's center.  No need to scale about
* an arbitrary point - more work for the
* user, and I can see no real practicality
* to it.
*/


void objectsScale(SYSINFO_TYPE *si )

{ /* objectsScale begin */

int             index;
OBJECT_TYPE     *obPtr;
int             button;
int             uniform_scale;
VIEW_COORD_TYPE top;
VIEW_COORD_TYPE bot;
float           xscale;
float           yscale;


MpurgeButtons();
uniform_scale = VertMenu( MAIN_OBJECTS,
                          objectScaleMenu,
                          objectScaleMenuSel,
                          BK_SAVE );


if (uniform_scale != CANCEL_VERT_MENU)
  {
  if (uniform_scale == UNIFORM_SCALE_REQUEST)
    uniform_scale = TRUE;
  else
    uniform_scale = FALSE;
  do
    {
      index = SelectObject("to SCALE");
      if (index != OBJECT_NOT_SELECTED)
        {
        obPtr = si->ObjectList[index];
        if (obPtr->NumPts == 1)
          {
          button = RIGHT_BUTTON;
          StatusLine( "Can't scale a single POINT!", si->ErrorColor );
          AudioError();
          }
        else
          {
          top.x = obPtr->sMin.x;
          top.y = obPtr->sMin.y;
          bot.x = obPtr->sMax.x;
          bot.y = obPtr->sMax.y;
          button = SizeBox( &top.x,  &top.y,
                            &bot.x,  &bot.y, uniform_scale,
                            si->ObjectColor );
          if (button != RIGHT_BUTTON)
            {

            xscale = (float)(bot.x-top.x)/
                     (float)(obPtr->sMax.x - obPtr->sMin.x);
            yscale = (float)(bot.y-top.y)/
                     (float)(obPtr->sMax.y - obPtr->sMin.y);
            StatusLine( "Computing new Object Size", si->MenuColor );
            ScaleObject( &si->ObjectList[index],
                         xscale,
                         yscale,
                         (float)1.0); /* ZSCALE */
            }
          }
        }
      else
        button = RIGHT_BUTTON;
    }
  while (button != RIGHT_BUTTON);
  } /* end if not cancel */

} /* objectsScale end */




/****************************************
* UNDO - Select an object. Check to see
* if its UNDO pointer is not null. If not
* then use the OBJOPS.C package to undo
* it.
*/


void objectsUndo ( SYSINFO_TYPE *si )

{ /* objectsUndo begin */

int exitnow;
int index;


exitnow = FALSE;
MpurgeButtons();
do
  {
  index = SelectObject("to UNDO");
  if (index == OBJECT_NOT_SELECTED)
    exitnow = TRUE;
  else
    {
    FrameObject( si->ObjectList[index], TRUE );
    si->ObjectList[index] = UndoObject( si->ObjectList[index] );
    if (si->ObjectList[index]->Undo != NULL)
      FrameObject( si->ObjectList[index]->Undo, TRUE );
    else
      FrameObject( si->ObjectList[index], TRUE );
    DrawObject( si->ObjectList[index], si->ObjectColor );
    }
  }
while (exitnow == FALSE);

} /* objectsUndo end */


/****************************************
* OBJECT_UNERASE handler. Show the user
* the objects that have been previously
* erased.  Allow him to selectively unerase
* them and restore to the drawing.
*/

void objectsUnerase( SYSINFO_TYPE *si )

{ /* objectsUnerase begin */

int             index;
int             erased;
VIEW_COORD_TYPE pos;
int             button;
OBJECT_TYPE     *obPtr;

MpurgeButtons();
erased = 0;
for (index=0; index<si->TotalObjects; index++)
  if (si->ObjectList[index]->Operation == DELETED_OP)
    erased++;
if ((si->TotalObjects == 0) || (erased == 0) )
  {
  StatusLine("No objects in erase queue", si->MenuColor );
  }
else
  { /* else unerase */
  StatusLine("LEFT-Unerase, RIGHT-Skip", si->MenuColor);
  for (index=0; index<si->TotalObjects; index++)
    { /* unerase loop */
    obPtr = si->ObjectList[index];
    if (obPtr != NULL)
      { /* if object exists */
      if (obPtr->Operation == DELETED_OP)
        { /* if object deleted (erased) */
        obPtr->Operation = CREATE_OP;
        DrawObject( obPtr, si->ObjectColor );
        FrameObject( obPtr, FALSE );
        GetPoint( NULL, &pos, &button, si->ShowMousePos );
        FrameObject( obPtr, FALSE );
        if (button == RIGHT_BUTTON)
          { /* if not unerase */
          obPtr->Operation = DELETED_OP;
          DrawObject( obPtr, si->BkColor );
          } /* end if not unerase */
        else
          { /* else we just activated another object */
          si->TotalActiveObjects++;
          }
        } /* end if object deleted */
      } /* end if object exists */
    } /* end unerase loop */
  } /* end else unerase */

} /* objectsUnerase end */



/****************************************
* OBJECT_CLEANUP handler.  Cleanup left
* over UNDO and DELETE ID structures, which
* are kept only to allow the user to restore
* any boo-boos.  If he decides to clean
* them up, they're gone for good.  It frees
* memory which can then be used for more
* object definitions.
*
*/

void objectsCleanup( SYSINFO_TYPE *si )

{ /* objectsCleanup begin */

int             status;
int             index;
int             erased;
char            msg[40];

erased = 0;
MpurgeButtons();
if (si->TotalObjects != 0)
  { /* if objects to examine */
  status = VertMenu( MAIN_OBJECTS,
                     objectCleanup,
                     objectCleanupSel, BK_SAVE );
  if ( (status == CLEANUP_UNDO) ||
       (status == CLEANUP_ALL) )
    {
    StatusLine("Cleanup UNDO storage", si->MenuColor );
    for(index = 0; index < si->TotalObjects; index++)
      {
      if (si->ObjectList[index] != NULL)
        {
        if (si->ObjectList[index]->Undo != NULL)
          {
          DestroyObject( &si->ObjectList[index]->Undo );
          erased++;
          }
        }
      }
    }
  if ( (status == CLEANUP_DELETE) ||
       (status == CLEANUP_ALL) )
    {
    StatusLine("Cleanup DELETE storage", si->MenuColor );
    for (index = 0; index < si->TotalObjects; index++)
      {
      if (si->ObjectList[index] != NULL)
        {
        if (si->ObjectList[index]->Operation == DELETED_OP)
          {
          DestroyObject( &si->ObjectList[index] );
          erased++;
          }
        }
      }
    }
  } /* end if objects to examine */
if (erased == 0)
  {
  if (status != CLEANUP_CANCEL)
    StatusLine("No Objects to clean up", si->MenuColor);
  }
else
  {
  sprintf(msg, "%d Objects Freed", erased);
  StatusLine( msg, si->MenuColor );
  }

} /* objectsCleanup end */



/****************************************
* objectsRotate - select object,
* specify axis of rotation,
* input angle of rotation,
* then rotate the object.
*
*/

#define CHARS_ALLOWED 15        /* Characters allowed in input line */

void objectsRotate( SYSINFO_TYPE *si )

{ /* objectsRotate begin */

char            rotangle[DEFAULT_STR_SIZE+1];
int             index;
int             status;
int             exitnow;
int             button;
int             operation;
float           degrees;
VIEW_COORD_TYPE center;
VIEW_COORD_TYPE start;
OBJECT_TYPE     *obPtr;

MpurgeButtons();
index = SelectObject("to ROTATE");

if (index != OBJECT_NOT_SELECTED)
  {
  obPtr = si->ObjectList[index];
  ClearStatusLine();
  FrameObject( obPtr, SHOW_CENTER );
  status = VertMenu( MAIN_OBJECTS,
                     objectRotateMenu,
                     objectRotateSel,
                     BK_SAVE );

  exitnow = FALSE;
  switch( status )
    {
    case( X_ROTATE ):
      operation = ROTATEX_OP;
      break;
    case( Y_ROTATE ):
      operation = ROTATEY_OP;
      break;
    case( Z_ROTATE ):
      operation = ROTATEZ_OP;
      break;
    default:
      exitnow = TRUE;
      break;
    }
  if (exitnow == FALSE)
    {
    StatusLine("LEFT-Center of rotation, RIGHT-Cancel", si->MenuColor);
    Mshowmouse();
    center.x = (int)obPtr->vCenter.x;
    center.y = (int)obPtr->vCenter.y;
    movemouse( center.x, center.y );
    start.x = center.x;
    start.y = center.y;
    setcolor( si->ObjectColor );
    GetPoint( &start, &center, &button, si->ShowMousePos );
    if (button != RIGHT_BUTTON)
      {

      Mhidemouse();
      box( center.x-2, center.y-2, center.x+2, center.y+2,
           si->ObjectColor, NOT_FILL_BOX );
      line( start.x, start.y, center.x, center.y );
      Mshowmouse();


      /*----------
      * We now have an object selected,
      * an axis of rotation specified,
      * and a center of rotation.  Next
      * we need the user's requested angle
      * of rotation
      */
      StatusLineGet( "Angle of rotation: ", rotangle,
                     si->MenuColor, si->MenuBkColor,
                     CHARS_ALLOWED );
      degrees = atof( rotangle );
      if (degrees == 0.0)
        {
        StatusLine("Angle specified = 0, nothing to do", si->MenuColor );
        }
      else
        {
        StatusLine("Computing rotation", si->MenuColor );
        /*----------
        * Rotate the puppy NOW
        */
        RotateObject( &si->ObjectList[index],
                      operation,
                      (float)center.x,
                      (float)center.y,
                      (float)0.0,
                      degrees );

        } /* end else rotate it */
      } /* end if !RIGHT_BUTTON, cancel */
    } /* end if !CANCEL_VERT_MENU */
  } /* end if !OBJECT_NOT_SELECTED */

RedrawDisplay( si->ObjectColor );

} /* objectsRotate end */



/****************************************
* objects3D - Select an object,
* draw an axis for it,
* allow the user to specify the step angle
* of rotation from a vertical menu selection,
* the entries of which are an integral divisor
* for 360.  The values will be:
*
* 4 to 90 degrees, in various increments,
* resulting in 90 to 4 sides on the final
* object.
*
* After object is "spun" allow LEFT click
* to accept or RIGHT click to "re-spin"
* with a new step angle specification.
* If RIGHT button is pressed during step
* angle specification, the rotation is
* aborted.
*/

void objects3D( SYSINFO_TYPE *si )

{ /* objects3D begin */

int             index;
int             degsel;
int             button;
float           degrees;
VIEW_COORD_TYPE p1;
VIEW_COORD_TYPE p2;
OBJECT_TYPE     *obPtr;
int             savepts;
int             saveid;

MpurgeButtons();
index = SelectObject("to SPIN 3D");
if (index != OBJECT_NOT_SELECTED)
  {
  obPtr = si->ObjectList[index];
  saveid = obPtr->Ident;
  FrameObject( obPtr, SHOW_CENTER );
  if (obPtr->Ident == SPIN3D_ID)
    {
    DrawObject( obPtr, si->BkColor ); /* erase */
    savepts = obPtr->NumPts;
    obPtr->NumPts = obPtr->PtsPerSide;
    obPtr->Ident  = POLYGON_ID;
    DrawObject( obPtr, si->ObjectColor );
    }
  AudioSelect();
  StatusLine("Draw AXIS of ROTATION, LEFT-end pts", si->MenuColor );
  AudioSelect();
  setcolor( si->ObjectColor );
  GetPoint( NULL, &p1, &button, si->ShowMousePos );
  if (button != RIGHT_BUTTON)
    {
    setcolor( si->ObjectColor );
    GetPoint( &p1, &p2, &button, si->ShowMousePos );
    if (button != RIGHT_BUTTON)
      {
      /****************************************
      * We now have an axis of rotation and
      * a confirmation the user wishes to
      * continue. We must now prompt
      * for rotation angle step.
      * If not CANCEL_VERT_MENU, then decipher
      * the step angle to use, initialize the
      * parameters, and call Spin3dObject in
      * the objops.c package to do the rest of
      * the work.
      */
      StatusLine("LEFT-Select angle step of rotation, RIGHT-Cancel 3D",
                 si->MenuColor );
      degsel = VertMenu( MAIN_OBJECTS,
                        objectSpinMenu,
                        objectSpinMenuSel,
                        BK_SAVE );
      switch( degsel )
        {
        case(STEP_10):
          degrees = 10.0;
          break;
        case(STEP_18):
          degrees = 18.0;
          break;
        case(STEP_20):
          degrees = 20.0;
          break;
        case(STEP_24):
          degrees = 24.0;
          break;
        case(STEP_30):
          degrees = 30.0;
          break;
        case(STEP_36):
          degrees = 36.0;
          break;
        case(STEP_72):
          degrees = 72.0;
          break;
        case(STEP_90):
          degrees = 90.0;
          break;
        default:
          button = RIGHT_BUTTON;
          break;
        } /* end switch index */
      if (button != RIGHT_BUTTON) /* then didn't cancel on step request */
        {
        StatusLine("Creating 3D Object", si->MenuColor );
        /* Erase center line */
        setcolor( si->BkColor );
        mline( p1.x, p1.y, p2.x, p2.y );
        setcolor( si->ObjectColor );
        /* Erase SELECT frame */
        FrameObject( si->ObjectList[index], SHOW_CENTER );
        /*----------
        * Now generate the 3D
        * image.
        */

        Spin3dObject( &si->ObjectList[index], degrees, &p1, &p2 );
        obPtr = si->ObjectList[index];
        if ( (obPtr != NULL) && (saveid == SPIN3D_ID) )
          { /* Then Undo pointer fixup by restoring id & point count */
          obPtr = obPtr->Undo;
          if (obPtr != NULL)
            {
            obPtr->Ident = SPIN3D_ID;
            obPtr->NumPts = savepts;
            }
          }
        }
      else
        {
        RedrawDisplay( si->ObjectColor );
        }
      } /* end if not cancel on second end point */
    else
      {
      FrameObject( si->ObjectList[index], SHOW_CENTER );
      /* Restore object if SPIN3D_ID type */
      if (saveid == SPIN3D_ID)
        {
        obPtr->Ident = SPIN3D_ID;
        obPtr->NumPts = savepts;
        RedrawDisplay( si->ObjectColor );
        }
      }
    } /* end if not cancel on first end point */
  else
    {
    FrameObject( si->ObjectList[index], SHOW_CENTER );
    /* Restore object if SPIN3D_ID type */
    if (saveid == SPIN3D_ID)
      {
      obPtr->Ident = SPIN3D_ID;
      obPtr->NumPts = savepts;
      RedrawDisplay( si->ObjectColor );
      }
    }
  } /* end if object not selected */

ClearStatusLine();

} /* objects3D end */



/****************************************
* Allow the user to select an object,
* make a duplicate copy, and then move
* a 'dragbox' to the duplicate's new location.
* Then perform the necessary transformation
* to put the copy in a new position.
*
*
*/

void objectsCopy( SYSINFO_TYPE *si )

{ /* objectsCopy begin */

int             index;
int             exitnow;
int             button;
VIEW_COORD_TYPE pos;
OBJECT_TYPE     *obPtr;
OBJECT_TYPE     *svPtr;
float           tx;
float           ty;
float           tz;

exitnow = FALSE;
MpurgeButtons();

do
  {
  index = SelectObject( "to COPY" );
  if (index != OBJECT_NOT_SELECTED)
    {
    obPtr = si->ObjectList[index];
    FrameObject( obPtr, TRUE );
    StatusLine("LEFT-Object copy new loc, RIGHT-Cancel",
               si->MenuColor);
    DragBox( obPtr->sMin.x, obPtr->sMin.y,
             obPtr->sMax.x, obPtr->sMax.y,
             &pos, &button, si->MenuColor );
    FrameObject( obPtr, TRUE );
    Mshowmouse();
    if (button == LEFT_BUTTON)
      {
      tz = 0.0;
      ty = (float)pos.y - obPtr->vCenter.y;
      tx = (float)pos.x - obPtr->vCenter.x;
      index = GetObjectIndex(); /* Get index for new object */
      setcolor( si->ObjectColor );
      if (index != GET_OBJECT_FAILED)
        {
        svPtr = obPtr;
        CopyObject( obPtr, &si->ObjectList[index],
                    tx, ty, tz );
        DrawObject( svPtr, si->ObjectColor );
        }
      }
    else
      exitnow = TRUE;
    }
  else
    exitnow = TRUE;
  }
while (exitnow == FALSE);

} /* objectsCopy end */



/****************************************
* objectsAnimate - Animate all the
* objects on the display that have
* active Undo pointers.  This is a very
* simple algorithm that simply draws each
* object with its undo pointer, then
* with its current pointer, and back again
* until the user clicks a button to stop
* the animation.
*
* This isn't as sophisticated as what the
* instructor requested, but it's easy to
* implement.  I'm more concerned with
* getting all the other mechanics and
* functionality completed in this program.
* Later animation can become much more complex
* by allowing the user to selectively "record"
* elements of an transformations on any
* object, with some editing options.
*
* If time permits, I'll also write a "canned"
* animation sequence that moves, rotates, and
* scales an object for a nice 3D effect.
*
* MJWR 7-APR-90
*/

void objectsAnimate( SYSINFO_TYPE *si )

{ /* objectsAnimate begin */

int             index;
int             button;
int             found_undos;
OBJECT_TYPE     *obPtr;

found_undos = FALSE;
index       = 0;

MpurgeButtons();
do
  {
  if (si->ObjectList[index] != NULL)
    {
    obPtr = si->ObjectList[index];
    if (obPtr->Undo != NULL)
      found_undos = TRUE;
    }
  index++;
  }
while ( (index < si->TotalObjects) && (found_undos == FALSE) );

if (found_undos == FALSE)
  {
  StatusLine("No UNDO history found, can't animate", si->ErrorColor);
  AudioError();
  }
else
  {
  StatusLine("Click button to stop animation", si->MenuColor);
  AudioSelect();
  setcolor( si->ObjectColor );
  do
    {
    index = 0;
    do
      {
      /*----------
      * Draw Undo objects first
      */
      if (si->ObjectList[index] != NULL)
        {
        obPtr = si->ObjectList[index];
        if ( (obPtr->Undo != NULL) &&
             (obPtr->Operation != DELETED_OP) )
          {
          RedrawObject( obPtr, obPtr->Undo );
          }
        }
      index++;
      button = ReadMouseButton();
      }
    while ( (button == NEITHER_BUTTON_PRESSED) &&
            (index < si->TotalObjects) );
    index = 0;
    delay(50);
    do
      {
      /*----------
      * Draw Current objects next
      */
      if (si->ObjectList[index] != NULL)
        {
        obPtr = si->ObjectList[index];
        if ( (obPtr->Undo != NULL) &&
             (obPtr->Operation != DELETED_OP) )

          {
          RedrawObject( obPtr->Undo, obPtr );
          }
        }
      index++;
      button = ReadMouseButton();
      }
    while ( (button == NEITHER_BUTTON_PRESSED) &&
            (index < si->TotalObjects) );
    delay(50);
    }
  while (button == NEITHER_BUTTON_PRESSED);
  RedrawDisplay( si->ObjectColor);
  }

MpurgeButtons();

} /* objectsAnimate end */

