/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... OBJOPS.C        */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 26-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Object operation*/
/*  package. Create, delete, translate, etc.            */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
26-MAR-90 MR Begin construction of Object operations for LAB4.C
             (No, I did NOT procrastinate! I've been busting my
             hump on MATRIX3D and MENU programs, neither a piece
             of cake!)
06-APR-90 MR Add ScaleObject
07-APR-90 MR Add CopyObject
09-APR-90 MR Final cleanup for LAB5


*\r*/

/* ---------- System include files -------------------- */

#include <stdio.h>
#include <alloc.h>              /* Memeory allocation INTENSIVE package */
#include <graphics.h>
#include <math.h>
#include <mem.h>

/* ---------- User include files ---------------------- */
#include "LAB.H"                /* VIEW_COORD_TYPE definition           */
#include "MATRIX3D.H"           /* VECT and MATRIX definitions          */
#include "MENU.H"               /* Display out of memory menu, if       */
                                /* necessary.                           */
#include "OBJDEFS.H"            /* Object structures.                   */
#include "OBJOPS.H"             /* Procedure prototypes                 */
#include "SPCLMENU.H"           /* Special menu declarations    */
#include "EGRAPH.H"
#include "EGRAPHP.H"




/****************************************
* Create an object of the type
* specified.  If numpts is not zero, then
* go ahead and allocate the point storage
* as well.  If the object is of type
* SPIN3D, then go ahead and allocate
* the Axis points as well.  If we get
* an out of memory error, inform the
* user that he/she should select CLEANUP
* from the OBJECTS menu.  CLEANUP will
* destroy all deleted and undo objects,
* freeing their associated memory.
* Saving a file and reloading will have
* an equivalent effect.
*
* Revision: 7-APR-90  Pass in the current
* object, curOb.  If it is a NEW object,
* then simply pass in a NULL pointer.
*
* If not NULL, then CreateObject will
* conditionally allocate memory. For example:
*
* if (curOb->Undo != NULL)
*   {
*   tempOb = curOb->Undo;
*   if (tempOb->NumPts == numpts)
*     {
*     newOb = tempOb;  No allocation/reallocation required
*     newOb->Undo = curOb;
*     curOb->Undo = NULL;
*     }
*   }
*
* If tempOb->NumPts != numpts then we have
* to handle the realloc() of the points.
*
* This new techique will:
*  minimize allocation
*  eliminate calls to DestroyObject from this module
*  prevent callers from ever changing an OBJECT_TYPE pointer.
*/

int  CreateObject( OBJECT_TYPE  **curOb,
                                  int objectname,
                                  int numpts,
                                  int ptsperside,
                                  int transform )

{ /* CreateObject begin */


OBJECT_TYPE *obPtr;
OBJECT_TYPE *curPtr;
int         allocation_error;
int         index;

curPtr = *curOb;


allocation_error = FALSE;

if (curPtr == NULL) /* Then time to make a new object */
  {
  obPtr = (OBJECT_TYPE *)malloc(sizeof(OBJECT_TYPE));
  if ( (numpts != 0) &&
       (obPtr != NULL) )
    {
    obPtr->Undo = NULL;
    obPtr->sPts = (VIEW_COORD_TYPE *)malloc(numpts*sizeof(VIEW_COORD_TYPE));
    obPtr->vPts = (VECTOR3D_TYPE *)malloc(numpts*sizeof(VECTOR3D_TYPE));
    if ( (obPtr->sPts == NULL) ||
         (obPtr->vPts == NULL) )
      {
      DestroyObject( &obPtr );
      allocation_error = TRUE;
      }
    }
  else if (obPtr == NULL)
    allocation_error = TRUE;
  }
else
  {
  if (curPtr->Undo != NULL)
    {
    obPtr = curPtr->Undo;       /* Swap pointers around if      */
    obPtr->Undo = curPtr;       /* curPtr->Undo is available    */
    curPtr->Undo = NULL;
    if ( (obPtr->NumPts != numpts) &&
         (numpts != 0) )
      {
      obPtr->sPts = (VIEW_COORD_TYPE *)malloc(numpts*sizeof(VIEW_COORD_TYPE));
      obPtr->vPts = (VECTOR3D_TYPE *)malloc(numpts*sizeof(VECTOR3D_TYPE));
      if ( (obPtr->sPts == NULL) ||
           (obPtr->vPts == NULL) )
        {
        DestroyObject( &obPtr );
        allocation_error = TRUE;
        }
      else
        {
        obPtr->NumPts = numpts;
        }
      }
    }
  else
    { /* curPtr->Undo == NULL, must allocate new obj */
    obPtr = (OBJECT_TYPE *)malloc(sizeof(OBJECT_TYPE));
    if ( (numpts != 0) &&
         (obPtr != NULL) )
      {
      obPtr->Undo = curPtr;
      curPtr->Undo = NULL;
      obPtr->sPts = (VIEW_COORD_TYPE *)malloc(numpts*sizeof(VIEW_COORD_TYPE));
      obPtr->vPts = (VECTOR3D_TYPE *)malloc(numpts*sizeof(VECTOR3D_TYPE));
      if ( (obPtr->sPts == NULL) ||
           (obPtr->vPts == NULL) )
        {
        DestroyObject( &obPtr );
        allocation_error = TRUE;
        }
      }
    else if (obPtr == NULL)
      allocation_error = TRUE;
    }
  }

/*----------
* At this point obPtr points to a new object or it is NULL, if any
* allocation errors occurred.  If not null, its Undo pointer has
* been initialized, based on what was found in curPtr.
*
* allocation_error has been set based on allocation success or
* failure.  Initialize the remaining elements of obPtr, if the
* creation was successful.
*/

if (allocation_error == FALSE)
  {
  obPtr->Ident                  = objectname;
  obPtr->NumPts                 = numpts;
  obPtr->PtsPerSide             = ptsperside;
  obPtr->Operation              = transform;
  obPtr->sAxis[0].x             = 0;
  obPtr->sAxis[0].y             = 0;
  obPtr->sAxis[1].x             = 0;
  obPtr->sAxis[1].y             = 0;
  obPtr->vAxis[0].x             = 0.0;
  obPtr->vAxis[0].y             = 0.0;
  obPtr->vAxis[0].z             = 0.0;
  obPtr->vAxis[0].h             = 0.0;
  obPtr->vAxis[1].x             = 0.0;
  obPtr->vAxis[1].y             = 0.0;
  obPtr->vAxis[1].z             = 0.0;
  obPtr->vAxis[1].h             = 1.0;
  for (index=0; index < numpts; index++)
    {
    obPtr->vPts[index].x = 0.0;
    obPtr->vPts[index].y = 0.0;
    obPtr->vPts[index].z = 0.0;
    obPtr->vPts[index].h = 1.0;
    obPtr->sPts[index].x = 0;
    obPtr->sPts[index].y = 0;
    }
  *curOb = obPtr;
  index = NEW_OBJECT_READY;
  }
else
  {
  ReportNoMemory();
  index = NO_MEMORY_FOR_NEW_OBJECT;
  }

return(index);

} /* CreateObject end */



/****************************************
* DestroyObject - Completely destroy
* an object by freeing all memory
* associated with it.  First test the
* undo pointer.  If it is not null, then
* call (recursive) DestroyObject with
* that pointer.  Pointers which are
* allocated that must be tested and
* freed, respectively, are as follows:
* DUAL_COORD_TYPE *Pts - Object's data
* points.
* DUAL_COORD_TYPE *Axis - Object's
* axis of rotation, if any.
* OBJECT_TYPE *obPtr - Finally free
* the memory associated with the object
* pointer itself.
*/

void DestroyObject( OBJECT_TYPE **oldPtr )

{ /* DestroyObject begin */

OBJECT_TYPE *obPtr;

/*----------
* If you are here, then you REALLY
* want to delete an object.
*/

if (*oldPtr != NULL)
  {
  obPtr = *oldPtr;
  if (obPtr->Undo != NULL)
    {
    DestroyObject( &obPtr->Undo );
    }
  else
    {
    if (obPtr->sPts != NULL)
      {
      free(obPtr->sPts);
      }
    if (obPtr->vPts != NULL)
      {
      free(obPtr->vPts);
      }
    free( obPtr );
    *oldPtr = NULL;
    }
  }

} /* DestroyObject end */



/****************************************
* DeleteObject - perform a "soft delete"
* on an object.  In this instance we
* simply set an object's operation
* type to DELETED, then redraw the object
* in the background color.
*
*/

void DeleteObject( OBJECT_TYPE *obPtr )

{ /* DeleteObject begin */

if (obPtr != NULL)
  {
  obPtr->Operation = DELETED_OP;
  DrawObject( obPtr, BLACK );
  }

} /* DeleteObject end */



/****************************************
* UndoObject - perform an UNDO on the
* selected object.  If the Undo pointer
* of that object is not null, then make
* it the current object.  Take the
* object passed in and make it the UNDO
* object (we can toggle between two object
* states by selectiong undo repetitively).
* If the Undo pointer is NULL then indicate
* it on the display.
*
* If the object cannot be undone, then
* return the pointer passed in.  Do not
* return a NULL unless that was what was
* passed to this module.
*
*/

OBJECT_TYPE *UndoObject( OBJECT_TYPE *obPtr )

{ /* UndoObject begin */

OBJECT_TYPE *newPtr;

/* Simply ignore the request if
*  the object pointer passed in
*  null.
*/

newPtr = obPtr; /* Default */

if (obPtr != NULL)
  {
  if (obPtr->Undo == NULL)
    ReportNoUndo();
  else
    {
    newPtr       = obPtr->Undo;
    newPtr->Undo = obPtr;
    obPtr->Undo  = NULL;
    }
  RedrawObject( obPtr, newPtr );
  }

return( newPtr );

} /* UndoObject end */



/****************************************
* Our first REAL operation for an object.
* Translate() - The algorithm parallels
* the classical method.  First we generate
* a MATRIX3D_TYPE, which is a 4x4 matrix
* with the transformation elements set.
* We then make multiple calls to
* VectMatMult3d to transform each point
* from the previous object to the new one.
* If the object being transformed has a
* valid Undo pointer, that object is first
* destroyed.  Then a new object is created.
* Then the current object is transformed
* into the new object.  The new object's
* undo pointer then points to the original
* object's address.  This technique will
* be used for all of the following
* transformations.
*/


int TranslateObject( OBJECT_TYPE **obPtr,
                     float tx,
                     float ty,
                     float tz )

{ /* TranslateObject begin */

MATRIX3D_TYPE  xlate;
OBJECT_TYPE   *newPos;
int            create_ok;

newPos = *obPtr;
create_ok = CreateObject( obPtr,
                          newPos->Ident,
                          newPos->NumPts,
                          newPos->PtsPerSide,
                          TRANSLATE_OP );

if (create_ok)
  {
  newPos = *obPtr;
  /*----------
  * Generate transformation matrix
  */
  TranslateMat3d( &xlate, tx, ty, tz );
  ApplyTransform( &xlate, newPos->Undo, newPos );
  }

return( create_ok );

} /* TranslateObject end */



/****************************************
* CopyObject( OBJECT_TYPE *oldPtr,
*             OBJECT_TYPE **newPtr,
*             float tx,
*             float ty,
*             float tz );
*
* Make a duplicate copy of the object at
* oldPtr.  Then perform a standard translation
* on it to move it to the desired new location
* at delta tx, ty, tz.
*
* Note that after translation the newPtr object
* as an active Undo pointer.  It will point back
* to a duplicate of oldPtr.  If the user were to
* select UNDO for this newPtr object, it would
* end up back on top of the oldPtr object.  Two
* identical objects at exactly the same place.
* That's illogical.  So, after taking care of
* duplication and translation, call DestroyObject()
* for the newPtr->Undo object.
*
*/

int CopyObject( OBJECT_TYPE *oldPtr,
                OBJECT_TYPE **newPtr,
                float tx,
                float ty,
                float tz )

{ /* CopyObject begin */

OBJECT_TYPE *obPtr;
int         create_ok;


create_ok = CreateObject( newPtr,
                          oldPtr->Ident,
                          oldPtr->NumPts,
                          oldPtr->PtsPerSide,
                          CREATE_OP );
                          
if (create_ok)
  {
  /*----------
  * Duplicate the object
  */
  obPtr = *newPtr;
  memcpy( (void *)obPtr->sPts, (void *)oldPtr->sPts,
          oldPtr->NumPts*sizeof(VIEW_COORD_TYPE));
  memcpy( (void *)obPtr->vPts, (void *)oldPtr->vPts,
          oldPtr->NumPts*sizeof(VECTOR3D_TYPE));
  create_ok = TranslateObject( newPtr, tx, ty, tz );
  if (create_ok)
    {
    obPtr = *newPtr;
    if (obPtr->Undo != NULL)
      {
      DestroyObject( &obPtr->Undo ); /* Can't undo a newly created copy */
      obPtr->Operation = CREATE_OP;
      }
    }
  }

return( create_ok );

} /* CopyObject end */



/****************************************
* ScaleObject - Scale an object about
* its own center:
*
* Translate the center to the origin,
* scale the object,
* translate the center back to its
* original location.
*
*/

int ScaleObject( OBJECT_TYPE **obPtr,
                 float        xscale,
                 float        yscale,
                 float        zscale )

{ /* ScaleObject begin */

OBJECT_TYPE     *newPtr;
OBJECT_TYPE     *svPtr;
MATRIX3D_TYPE   translate;
MATRIX3D_TYPE   scale;
int             create_ok;

newPtr = *obPtr;
create_ok = CreateObject( obPtr,
                          newPtr->Ident,
                          newPtr->NumPts,
                          newPtr->PtsPerSide,
                          SCALE_OP );
if (create_ok)
  {
  newPtr = *obPtr;
  svPtr  = newPtr->Undo;
  TranslateMat3d( &translate,
                  -svPtr->vCenter.x,
                  -svPtr->vCenter.y,
                  -svPtr->vCenter.z );
  ScaleDifferentialMat3d( &scale,
                          xscale,
                          yscale,
                          zscale );
  Mult3d( &translate, &scale, &scale );
  TranslateMat3d( &translate,
                  svPtr->vCenter.x,
                  svPtr->vCenter.y,
                  svPtr->vCenter.z );
  Mult3d( &scale, &translate, &scale );
  ApplyTransform( &scale, svPtr, newPtr );
  }

return( create_ok );

} /* ScaleObject end */




/****************************************
* RotateObject - Create a new object,
* if successful, generate a translation
* matrix to get the specified center
* to the origin.  Generate the inverse
* translation matrix. Then generate
* the rotation matrix for the axis of
* rotation specified.  Then build a
* composite matrix and call ApplyTransform
* to get the job done.
*
*/

int RotateObject( OBJECT_TYPE **obPtr,
                  int axis_op,
                  float centerx,
                  float centery,
                  float centerz,
                  float degrees )

{ /* RotateObject begin */

MATRIX3D_TYPE   xlate_center;
MATRIX3D_TYPE   xlate_restore;
MATRIX3D_TYPE   rotate_axis;
OBJECT_TYPE     *newPos;
float           rad;
int             create_ok;

/*----------
* First create a new object
* to put the results of the
* rotation in
*/

newPos = *obPtr;
create_ok = CreateObject( obPtr,
                          newPos->Ident,
                          newPos->NumPts,
                          newPos->PtsPerSide,
                          axis_op );

if (create_ok)
  {
  newPos = *obPtr;

  rad = Radians( degrees );
  /*----------
  * Step 1, translate object center
  * to center of rotation
  */
  TranslateMat3d( &xlate_center,
                  -centerx,
                  -centery,
                  -centerz );

  /*----------
  * Step 2, generate inverse
  * translation matrix to restore
  * relative position of resultant
  * rotated object
  */
  TranslateMat3d( &xlate_restore,
                  centerx,
                  centery,
                  centerz );

  /*----------
  * Setup rotation matrix
  * based on axis_op specified
  */
  switch( axis_op )
    {
    case( ROTATEX_OP ):
      RotMatRad3dX( &rotate_axis, -rad );
      break;
    case( ROTATEY_OP ):
      RotMatRad3dY( &rotate_axis, -rad );
      break;
    case( ROTATEZ_OP ):
      RotMatRad3dZ( &rotate_axis, -rad );
      break;
    default:
      Identity3d( &rotate_axis );
      break;
    }
  /*----------
  * Composite = xlate_center * rotate_axis * xlate_restore
  */
  Mult3d( &xlate_center, &rotate_axis, &rotate_axis );
  Mult3d( &rotate_axis, &xlate_restore, &rotate_axis );
  /*----------
  * Composite transform now in rotate_axis
  *
  * Apply the transformation:
  */
  ApplyTransform( &rotate_axis, newPos->Undo, newPos );
  }

return( create_ok );

} /* RotateObject end */



/****************************************
* Spin3dObject - Generate a 3D object
* around the axis specified by the view
* coordinates p1 & p2.  Use the step
* angle degrees to determine the distance
* of rotation before generating each facet
* of the object.
*/

int Spin3dObject( OBJECT_TYPE     **obPtr,
                  float           degree_step,
                  VIEW_COORD_TYPE *axis1,
                  VIEW_COORD_TYPE *axis2 )

{ /* Spin3dObject begin */

int             sides;
int             index1;
int             index2;
int             i;
int             sideofs;
int             rotate_adjust;
float           totalPts;
float           totalSize;
float           angle;
float           dx;
float           dy;
int             numPts;
OBJECT_TYPE     *newOb;
OBJECT_TYPE     *svPtr;
MATRIX3D_TYPE   translate;
MATRIX3D_TYPE   retn;
MATRIX3D_TYPE   spin;
MATRIX3D_TYPE   rotate;
MATRIX3D_TYPE   composite;
int             create_ok;

if (degree_step != 0.0)
  sides = (int)(360.0 / degree_step );
else
  {
  sides = 10;
  degree_step = 36.0;
  }

newOb = *obPtr;
totalPts = (float)sides * (float)newOb->NumPts;
totalSize = totalPts * (float)sizeof(VECTOR3D_TYPE);
if (totalSize > 32767.0)
  {
  ObjectTooLarge();
  }
else
  {
  numPts = (int)totalPts;
  create_ok = CreateObject( obPtr,
                            SPIN3D_ID,
                            numPts,
                            newOb->NumPts, /* pts per side */
                            SPIN3D_OP );
  if (create_ok)
    {
    newOb = *obPtr;
    svPtr = newOb->Undo;
    newOb->PtsPerSide = svPtr->NumPts;
    newOb->sAxis[0].x = axis1->x;
    newOb->sAxis[0].y = axis1->y;
    newOb->sAxis[1].x = axis2->x;
    newOb->sAxis[1].y = axis2->y;
    newOb->vAxis[0].x = (float)axis1->x;
    newOb->vAxis[0].y = (float)axis1->y;
    newOb->vAxis[1].x = (float)axis2->x;
    newOb->vAxis[1].y = (float)axis2->y;
    newOb->vAxis[0].z = 0.0;
    newOb->vAxis[1].z = 0.0;
    /*----------
    * Generate translation matrix
    * to position axis point 1 at
    * the origin
    */
    TranslateMat3d( &translate,
                    -newOb->vAxis[0].x,
                    -newOb->vAxis[0].y,
                    -newOb->vAxis[0].z );
    /*----------
    * And the inverse:
    */
    TranslateMat3d( &retn,
                    newOb->vAxis[0].x,
                    newOb->vAxis[0].y,
                    newOb->vAxis[0].z );

    if (axis1->x == axis2->x)
      { /* then spin about y axis */
      RotMatDeg3dY( &spin, degree_step );
      rotate_adjust = FALSE;
      }
    else if (axis1->y == axis2->y)
      { /* then spin about x axis */
      RotMatDeg3dX( &spin, degree_step );
      rotate_adjust = FALSE;
      }
    else
      {
      rotate_adjust = TRUE;

      dx = (float)axis1->x - (float)axis2->x;
      dy = (float)axis1->y - (float)axis2->y;
      if (dy > dx)
        {
        angle = atan( dx/dy );
        RotMatRad3dZ( &rotate, angle);
        RotMatDeg3dY( &spin, degree_step );
        }
      else
        {
        angle = atan( dy/dx );
        RotMatRad3dZ( &rotate, -angle );
        RotMatDeg3dX( &spin, degree_step );
        }
      }
    /*----------
    * Now have translate, retn, spin, and rotate matrices
    *
    * Composite is:
    *
    * composite = translate * rotate * spin * rotate^-1 * retn
    */
    if (rotate_adjust == TRUE)
      {
      Mult3d( &translate, &rotate, &composite );
      Mult3d( &composite, &spin, &composite );
      Transpose3d( &rotate );
      Mult3d( &composite, &rotate, &composite );
      Mult3d( &composite, &retn, &composite );
      }
    else
      {
      Mult3d( &translate, &spin, &composite );
      Mult3d( &composite, &retn, &composite );
      }

    /****************************************
    * We now have a composite transformation
    * matrix that will spin any point
    * in the original object about the arbitrary
    * specified line.  We know total points
    * and points per side.  Now setup the
    * loop to transform all the points
    *
    * The first edge in the new object will
    * be a copy of the edge in the original
    * object.
    */
    svPtr = newOb->Undo;
    for (index1=0; index1<newOb->PtsPerSide; index1++)
      {
      newOb->vPts[index1].x = svPtr->vPts[index1].x;
      newOb->vPts[index1].y = svPtr->vPts[index1].y;
      newOb->vPts[index1].z = svPtr->vPts[index1].z;
      newOb->vPts[index1].h = svPtr->vPts[index1].h;
      MapToScreen( &newOb->vPts[index1], &newOb->sPts[index1] );
      }
    for (index1 = newOb->PtsPerSide;
         index1 < newOb->NumPts;
         index1+= newOb->PtsPerSide)
      {
      sideofs = newOb->PtsPerSide;
      for (index2=0; index2 < newOb->PtsPerSide; index2++)
        {
        i = index1 + index2;
        VectMatMult3d( &newOb->vPts[index1 - sideofs + index2],
                       &composite,
                       &newOb->vPts[i]);
        MapToScreen( &newOb->vPts[i], &newOb->sPts[i] );
        }
      }
    CenterMinMaxObject( newOb ); /* re-Generate Center, Min, Max values */
    DrawObject( newOb, WHITE );
    }
  }

return( create_ok );

} /* Spin3dObject end */




/****************************************
* MapToScreen( VECTOR3D_TYPE *vec,
*              VIEW_COORD_TYPE *scr );
*
* Currently this is a simple orthographic
* parallel projection. Therefore the
* x and y elements of the view coordinate
* are simply the integer equivalent of the
* x and y components of the vector.  By
* using this function for all screen
* mapping, it is easier to modify the software
* to support other projection methods.
*
*/

void MapToScreen( VECTOR3D_TYPE *vec, VIEW_COORD_TYPE *scr )

{ /* MapToScreen begin */


scr->x = (int)vec->x;
scr->y = (int)vec->y;

} /* MapToScreen end */



/****************************************
* Erase the old object at ob by drawing
* it in the background color.  Then
* draw the new object at the newPos in
* the forground color.  The act of erasing
* the old object may erase parts of other
* objects.  I don't care. The user has the
* option of redrawing all objects as one
* of the menu options.
*/


void  RedrawObject( OBJECT_TYPE *obPtr, OBJECT_TYPE *newPtr )
{ /* RedrawObject begin */
int i;

i = getbkcolor();

DrawObject( obPtr, i );
DrawObject( newPtr, WHITE );

} /* RedrawObject end */



/****************************************
* DrawObject( OBJECT_TYPE *ob, int color );
*
* Draw the specified object in the selected
* color.  Look at the object type Ident
* and Operation to determine the proper
* drawing method.
*/

void DrawObject( OBJECT_TYPE *obPtr, int color )

{ /* DrawObject begin */

int index;
int ptcnt;
int xadj, yadj;
Mhidemouse();

SetClippedViewport( &xadj, &yadj );

if (obPtr != NULL)
  {
  setcolor ( color );
  switch( obPtr->Ident )
    {
    case( POINT_ID ):
      for (index=0; index < obPtr->NumPts; index++)
        putpixel( obPtr->sPts[index].x - xadj,
                  obPtr->sPts[index].y - yadj, color );
      break;
    case( LINE_ID ):
       line( obPtr->sPts[0].x - xadj, obPtr->sPts[0].y - yadj,
             obPtr->sPts[1].x - xadj, obPtr->sPts[1].y - yadj);
      break;
    case( POLYGON_ID ):
      moveto( obPtr->sPts[0].x - xadj,
              obPtr->sPts[0].y - yadj );
      for (index=0; index < obPtr->NumPts; index++)
        lineto( obPtr->sPts[index].x - xadj,
                obPtr->sPts[index].y - yadj );
      break;
    case( SPIN3D_ID ):
      /*----------
      * Draw polygons first
      */
      ptcnt = 0;
      for (index = 0; index < obPtr->NumPts; index++)
        {
        if (ptcnt == 0)
          moveto( obPtr->sPts[index].x - xadj,
                  obPtr->sPts[index].y - yadj );
        else
          lineto( obPtr->sPts[index].x - xadj,
                  obPtr->sPts[index].y - yadj );
        ptcnt++;
        if (ptcnt == obPtr->PtsPerSide)
          ptcnt = 0;
        }
      /*----------
      * Now connect vertices of each polygon
      */
      for (index = 0; index < obPtr->PtsPerSide; index++)
        {
        moveto( obPtr->sPts[index].x - xadj,
                obPtr->sPts[index].y - yadj );
        for (ptcnt=index; ptcnt < obPtr->NumPts; ptcnt+= obPtr->PtsPerSide)
          {
          lineto( obPtr->sPts[ptcnt].x - xadj,
                  obPtr->sPts[ptcnt].y - yadj );
          }
        /*----------
        * Close the wire frame
        */
        lineto( obPtr->sPts[index].x - xadj,
                obPtr->sPts[index].y - yadj );
        }
      break;
    }
  }

Mshowmouse();
SetViewMenuArea();

} /* DrawObject end */



/****************************************
 * CenterMinMaxObject( OBJECT_TYPE *ob );
 *
 * Find the Center, Min and Max x and y coords
 * of all the points in the object.  This
 * data is used to keep track of the select
 * area of the object.  Use this information
 * to set the center of the object.  It is
 * used to translate the objecto the origin,
 * rotate, and translate back once again.
 */

void CenterMinMaxObject( OBJECT_TYPE *obPtr )

{ /* CenterMinMaxObject begin */

int             index;
VECTOR3D_TYPE testmin;
VECTOR3D_TYPE testmax;

if (obPtr != NULL)
  { /* Ignore null pointers */
  if ( obPtr->NumPts > 1)
    {
    /*----------
     * Set defalt initial values
     * of our test min and max
     * coordinates.  We use the
     * floating point vector 3d
     * coords for accuracy, to minimize
     * round off errors.
     */

    testmin.x = obPtr->vPts[0].x;
    testmin.y = obPtr->vPts[0].y;
    testmin.z = obPtr->vPts[0].z;
    testmax.x = testmin.x;
    testmax.y = testmin.y;
    testmax.z = testmin.z;

    /*----------
     * Loop through all points in
     * the object, testing all screen
     * points.
     */

    for (index = 1; index < obPtr->NumPts; index++)
      {
      if (obPtr->vPts[index].x < testmin.x)
        testmin.x = obPtr->vPts[index].x;
      if (obPtr->vPts[index].x > testmax.x)
        testmax.x = obPtr->vPts[index].x;
      if (obPtr->vPts[index].y < testmin.y)
        testmin.y = obPtr->vPts[index].y;
      if (obPtr->vPts[index].y > testmax.y)
        testmax.y = obPtr->vPts[index].y;
      if (obPtr->vPts[index].z < testmin.z)
        testmin.z = obPtr->vPts[index].z;
      if (obPtr->vPts[index].z > testmax.z)
        testmax.z = obPtr->vPts[index].z;
      }
    if ((testmax.x - testmin.x) < 2*POINT_EFFECT_SIZE)
      {
      testmax.x += POINT_EFFECT_SIZE;
      testmin.x -= POINT_EFFECT_SIZE;
      }
    if ((testmax.y - testmin.y) < 2*POINT_EFFECT_SIZE)
      {
      testmax.y += POINT_EFFECT_SIZE;
      testmin.y -= POINT_EFFECT_SIZE;
      }
    if ((testmax.z - testmin.z) < 2*POINT_EFFECT_SIZE)
      {
      testmax.z += POINT_EFFECT_SIZE;
      testmin.z -= POINT_EFFECT_SIZE;
      }
    obPtr->vCenter.x = testmin.x + (testmax.x - testmin.x)/2.0;
    obPtr->vCenter.y = testmin.y + (testmax.y - testmin.y)/2.0;
    obPtr->vCenter.z = testmin.z + (testmax.z - testmin.z)/2.0;
    obPtr->sMax.x    = (int)testmax.x;
    obPtr->sMin.x    = (int)testmin.x;
    obPtr->sMax.y    = (int)testmax.y;
    obPtr->sMin.y    = (int)testmin.y;
    }
  else
    { /* For a point Min, Max, and Center differ a bit */
    obPtr->vCenter.x = obPtr->vPts[0].x;
    obPtr->vCenter.y = obPtr->vPts[0].y;
    obPtr->sMin.x    = obPtr->sPts[0].x - POINT_EFFECT_SIZE;
    obPtr->sMin.y    = obPtr->sPts[0].y - POINT_EFFECT_SIZE;
    obPtr->sMax.x    = obPtr->sPts[0].x + POINT_EFFECT_SIZE;
    obPtr->sMax.y    = obPtr->sPts[0].y + POINT_EFFECT_SIZE;
    }
  }

} /* CenterMinMaxObject end */



/****************************************
* ApplyTransform - Apply a MATRIX3D_TYPE
* composite transformation to an object
* mapping into another object. It is up to
* the caller to have allocated all the
* proper storage for the new destination
* object.
*/

void ApplyTransform( MATRIX3D_TYPE *xfrm,
                     OBJECT_TYPE   *obPtr,
                     OBJECT_TYPE   *newPos )

{ /* ApplyTransform begin */

int index;

/*----------
* Now transform each point in the object
*/
for (index = 0; index < obPtr->NumPts; index++)
  {
  VectMatMult3d( &obPtr->vPts[index],
                 xfrm,
                 &newPos->vPts[index] );
  /*----------
  * Now map homogeneous coordinate
  * to screen coordinate
  */

  MapToScreen( &newPos->vPts[index], &newPos->sPts[index] );
  }

/*----------
* Put on screen asap
*/

RedrawObject( obPtr, newPos );

/*----------
* Handle axis update for 3d objects
*/

if (obPtr->Ident == SPIN3D_ID) /* then translate axis too */
  {
  VectMatMult3d( &obPtr->vAxis[0], xfrm, &newPos->vAxis[0] );
  VectMatMult3d( &obPtr->vAxis[1], xfrm, &newPos->vAxis[1] );
  newPos->sAxis[0].x = (int)newPos->vAxis[0].x;
  newPos->sAxis[0].y = (int)newPos->vAxis[0].y;
  newPos->sAxis[0].x = (int)newPos->vAxis[1].x;
  newPos->sAxis[0].y = (int)newPos->vAxis[1].y;
  }

/*----------
* Setup max/min information
* for selection and center data
*/

CenterMinMaxObject( newPos ); /* re-Generate Center, Min, Max values */

} /* ApplyTransform end */


/****************************************
* PlotObject( OBJECT_TYPE *ob, int color );
*
* Plot the specified object in the selected
* color.  Look at the object type Ident
* and Operation to determine the proper
* drawing method.
*/

/****************************************
* Quick and dirty scaling macros to fit
* objects on epson plotter.
*/

#define EscalX(a) exmax*(long)a/xmax
#define EscalY(a) eymax*(long)a/ymax

void PlotObject( OBJECT_TYPE *obPtr, int color )

{ /* PlotObject begin */

int index;
int ptcnt;

long xmax;
long ymax;
long exmax;
long eymax;


xmax = (long)getmaxx();
ymax = (long)getmaxy();

exmax = (long)EgMaxx();
eymax = (long)EgMaxy();

if (obPtr != NULL)
  {
  EgColor ( color );
  switch( obPtr->Ident )
    {
    case( POINT_ID ):
      for (index=0; index < obPtr->NumPts; index++)
        EgPlot(   EscalX(obPtr->sPts[index].x),
                  EscalY(obPtr->sPts[index].y) );
      break;

    case( LINE_ID ):
       EgDraw( EscalX(obPtr->sPts[0].x), EscalY(obPtr->sPts[0].y),
               EscalX(obPtr->sPts[1].x), EscalY(obPtr->sPts[1].y));
      break;

    case( POLYGON_ID ):
      EgPosition( EscalX(obPtr->sPts[0].x),
                  EscalY(obPtr->sPts[0].y) );
      for (index=0; index < obPtr->NumPts; index++)
        EgDrawto( EscalX(obPtr->sPts[index].x),
                  EscalY(obPtr->sPts[index].y) );
      break;
    case( SPIN3D_ID ):
      /*----------
      * Draw polygons first
      */
      ptcnt = 0;
      for (index = 0; index < obPtr->NumPts; index++)
        {
        if (ptcnt == 0)
          EgPosition( EscalX(obPtr->sPts[index].x),
                      EscalY(obPtr->sPts[index].y) );
        else
          EgDrawto( EscalX(obPtr->sPts[index].x),
                    EscalY(obPtr->sPts[index].y) );
        ptcnt++;
        if (ptcnt == obPtr->PtsPerSide)
          ptcnt = 0;
        }
      /*----------
      * Now connect vertices of each polygon
      */
      for (index = 0; index < obPtr->PtsPerSide; index++)
        {
        EgPosition( EscalX(obPtr->sPts[index].x),
                    EscalY(obPtr->sPts[index].y) );
        for (ptcnt=index; ptcnt < obPtr->NumPts; ptcnt+= obPtr->PtsPerSide)
          {
          EgDrawto( EscalX(obPtr->sPts[ptcnt].x),
                    EscalY(obPtr->sPts[ptcnt].y) );
          }
        /*----------
        * Close the wire frame
        */
        EgDrawto( EscalX(obPtr->sPts[index].x),
                  EscalY(obPtr->sPts[index].y) );
        }
      break;
    }
  }

Mshowmouse();
SetViewMenuArea();

} /* PlotObject end */


