/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... OBJOPS.H        */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 26-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Procedure       */
/*  declarations for object operations.                 */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
06-APR-90 MR Add ScaleObject
07-APR-90 MR Add CopyObject
09-APR-90 MR Final cleanup for LAB5.


*\r*/


/****************************************
* Create an object of the type
* specified.  If numpts is not zero, then
* go ahead and allocate the point storage
* as well.  If the object is of type
* SPIN3D, then go ahead and allocate
* the Axis points as well.  If we get
* an out of memory error, inform the
* user that he/she should select CLEANUP
* from the OBJECTS menu.  CLEANUP will
* destroy all deleted and undo objects,
* freeing their associated memory.
* Saving a file and reloading will have
* an equivalent effect.  If the object
* is SPIN3D type, the caller must also
* specify points per side for proper
* initialization of the structure elements.
* Upon return, the user need only copy
* the DUAL_COORD_TYPE elements into the
* newly created structure. This will
* usally be done through a transformation
* algorithm, however.  The caller
* also specifies the transformation that
* brings this object into existence, wheter
* it's CREATE or ROTATEX, etc.
*/


#define NEW_OBJECT_READY                 1
#define NO_MEMORY_FOR_NEW_OBJECT         0

int  CreateObject( OBJECT_TYPE **curOb,
                   int objectname,
                   int numpts,
                   int ptsperside,
                   int transform );


/****************************************
* DestroyObject - Completely destroy
* an object by freeing all memory
* associated with it.  First test the
* undo pointer.  If it is not null, then
* call (recursive) DestroyObject with
* that pointer.  Pointers which are
* allocated that must be tested and
* freed, respectively, are as follows:
* DUAL_COORD_TYPE *Pts - Object's data
* points.
* DUAL_COORD_TYPE *Axis - Object's
* axis of rotation, if any.
* OBJECT_TYPE *obPtr - Finally free
* the memory associated with the object
* pointer itself.
*/

void DestroyObject( OBJECT_TYPE **oldPtr );


/****************************************
* DeleteObject - perform a "soft delete"
* on an object.  In this instance we
* simply set an object's operation
* type to DELETED, then redraw the object
* in the background color.
*
*/

void DeleteObject( OBJECT_TYPE *obPtr );

/****************************************
* UndoObject - perform an UNDO on the
* selected object.  If the Undo pointer
* of that object is not null, then make
* it the current object.  Take the
* object passed in and make it the UNDO
* object (we can toggle between two object
* states by selectiong undo repetitively).
* If the Undo pointer is NULL then indicate
* it on the display.
*
*/

OBJECT_TYPE *UndoObject( OBJECT_TYPE *obPtr );

/****************************************
* Our first REAL operation for an object.
* Translate() - The algorithm parallels
* the classical method.  First we generate
* a MATRIX3D_TYPE, which is a 4x4 matrix
* with the transformation elements set.
* We then make multiple calls to
* VectMatMult3d to transform each point
* from the previous object to the new one.
* If the object being transformed has a
* valid Undo pointer, that object is first
* destroyed.  Then a new object is created.
* Then the current object is transformed
* into the new object.  The new object's
* undo pointer then points to the original
* object's address.  This technique will
* be used for all of the following
* transformations.
*/

int TranslateObject( OBJECT_TYPE **obPtr,
                     float tx,
                     float ty,
                     float tz );

/****************************************
* CopyObject( OBJECT_TYPE *oldPtr,
*             OBJECT_TYPE **newPtr,
*             float tx,
*             float ty,
*             float tz );
*
* Make a duplicate copy of the object at
* oldPtr.  Then perform a standard translation
* on it to move it to the desired new location
* at delta tx, ty, tz.
*
*
*/

int CopyObject( OBJECT_TYPE *oldPtr,
                OBJECT_TYPE **newPtr,
                float tx,
                float ty,
                float tz );

/****************************************
* ScaleObject - Scale an object about
* its own center:
*/

int ScaleObject( OBJECT_TYPE **obPtr,
                 float        xscale,
                 float        yscale,
                 float        zscale );

/****************************************
* Spin3dObject - Generate a 3D object
* around the axis specified by the view
* coordinates p1 & p2.  Use the step
* angle degrees to determine the distance
* of rotation before generating each facet
* of the object.
*/

int Spin3dObject( OBJECT_TYPE     **obPtr,
                  float           degree_step,
                  VIEW_COORD_TYPE *axis1,
                  VIEW_COORD_TYPE *axis2 );

/****************************************
 * RotateObject - Rotate an object
 * about the specified axis
 *
 */

int RotateObject( OBJECT_TYPE **obPtr,
                  int axis_op,
                  float centerx,
                  float centery,
                  float centerz,
                  float degrees );

/****************************************
* MapToScreen( VECTOR3D_TYPE *vec,
*              VIEW_COORD_TYPE *scr );
*
* Currently this is a simple orthographic
* parallel projection. Therefore the
* x and y elements of the view coordinate
* are simply the integer equivalent of the
* x and y components of the vector.  By
* using this function for all screen
* mapping, it is easier to modify the software
* to support other projection methods.
*
*/

void MapToScreen( VECTOR3D_TYPE *vec, VIEW_COORD_TYPE *scr );

/****************************************
* Erase the old object at ob by drawing
* it in the background color.  Then
* draw the new object at the newPos in
* the forground color.  The act of erasing
* the old object may erase parts of other
* objects.  I don't care. The user has the
* option of redrawing all objects as one
* of the menu options.
*/


void  RedrawObject( OBJECT_TYPE *obPtr, OBJECT_TYPE *newPtr );

/****************************************
* DrawObject( OBJECT_TYPE *ob, int color );
*
* Draw the specified object in the selected
* color.  Look at the object type Ident
* and Operation to determine the proper
* drawing method.
*/

void DrawObject( OBJECT_TYPE *obPtr, int color );

void PlotObject( OBJECT_TYPE *obPtr, int color );

/****************************************
 * CenterMinMaxObject( OBJECT_TYPE *obPtr );
 *
 * Find the Center, Min and Max x and y coords
 * of all the points in the object.  This
 * data is used to keep track of the select
 * area of the object.  Use this information
 * to set the center of the object.  It is
 * used to translate the objecto the origin,
 * rotate, and translate back once again.
 */

void CenterMinMaxObject( OBJECT_TYPE *obPtr );

/****************************************
* ApplyTransform - Apply a MATRIX3D_TYPE
* composite transformation to an object
* mapping into another object. It is up to
* the caller to have allocated all the
* proper storage for the new destination
* object.
*/

void ApplyTransform( MATRIX3D_TYPE *xfrm,
                     OBJECT_TYPE   *obPtr,
                     OBJECT_TYPE   *newPtr );

