/*-------------------------------------------------------------------*

 Name		PrintImage

 Description	Prints an image to an Epson compatible printer, ala
		GetImage and PutImage.

 Syntax         Eprintimage(char *title, int x1, int y1, int x2, int y2);
 Syntax         Hprintimage(char *title, int x1, int y1, int x2, int y2);

 *-------------------------------------------------------------------*/
#include <graphics.h>
#include <stdio.h>
#include <io.h>
#include <bios.h>

#define		ESC		'\x1B'
#define		LPT1		0
#define		LPT2		1
#define		prn_putc(x)	biosprint(0,(x),LPT1)

/*
    Sets Epson printer to bit image mode.
    N is the number of bytes to print.
*/
static void bitImage(int N)
{
register int        n1, n2;

n2 = N >> 8;
n1 = N - (n2 << 8);

prn_putc(ESC);
prn_putc('*');

/****************************************
* If width > 600 use mode 6, else use
* mode 4.
*/

if (N > 600)
  prn_putc(6);
else
  prn_putc(4);

                /*****************************************************/
                /* Epson ESC, '*' - Bit Image Mode Selection:        */
                /*                                                   */
                /* ESC,'*',m,n1,n2,d1....dn1n2                       */
                /*                                                   */
                /* m    Mode            DotsPerInch     Print Speed  */
                /* 0    Normal                  60              16   */
                /* 1    Dual Density           120               8   */
                /* 2    DblSpeed Dual Densith  120              16   */
                /* 3    Quad-Density           240               8   */
                /* 4    CRT Graphics            80               8   */
                /* 5    Plotter graphics        72              12   */
                /* 6    CRT Graphics II         90               8   */
                /*****************************************************/

prn_putc(n1);
prn_putc(n2);
}

/*
    Get pixels from the screen and convert them to
    the printer's pin order.
*/
static unsigned char getScrBits(int x, int y)
{
unsigned char firePins;

firePins  = (getpixel(x, y++)==0)? 0: 0x80;
firePins |= (getpixel(x, y++)==0)? 0: 0x40;
firePins |= (getpixel(x, y++)==0)? 0: 0x20;
firePins |= (getpixel(x, y++)==0)? 0: 0x10;
firePins |= (getpixel(x, y++)==0)? 0: 0x08;
firePins |= (getpixel(x, y++)==0)? 0: 0x04;
firePins |= (getpixel(x, y++)==0)? 0: 0x02;
firePins |= (getpixel(x, y  )==0)? 0: 0x01;

return(firePins);
}


static unsigned char getHpScrBits(int x, int y)
{
unsigned char firePins;

firePins  = (getpixel(x++, y)==0)? 0: 0x80;
firePins |= (getpixel(x++, y)==0)? 0: 0x40;
firePins |= (getpixel(x++, y)==0)? 0: 0x20;
firePins |= (getpixel(x++, y)==0)? 0: 0x10;
firePins |= (getpixel(x++, y)==0)? 0: 0x08;
firePins |= (getpixel(x++, y)==0)? 0: 0x04;
firePins |= (getpixel(x++, y)==0)? 0: 0x02;
firePins |= (getpixel(x,   y)==0)? 0: 0x01;

return(firePins);
}


int Eprintimage( char *title,
                 int x1, int y1, int x2, int y2)
{

int     x;
int     y;
int     width;

width  = x2-x1+1;

fprintf(stdprn,"     %s\n\n", title );

for (y=y1; y<=y2; y+=8)
  {
  prn_putc(ESC); prn_putc('1'); /* Initialize line spacing to 7/72" */
  bitImage(width);
  for (x=x1; x<=x2; x++)
    {
    prn_putc(getScrBits(x,y));
    }
  prn_putc('\n');
  }

fprintf(stdprn,"%c%c@", 12, ESC );

return(0);

}

/****************************************
* Simply blow out the bit image
* graphics for an HP deskjet or laserjet
* printer.
* ESC;"*t75R" - Set hp resolution to 75 dpi
* ESC;"*b70W" - 70 bytes of raster data follows
* ESC;"*rB"   - End of raster graphics.
*
*/
int Hprintimage( char *title,
                 int x1, int y1, int x2, int y2)
{

int     x;
int     y;
int     width;
char    lineCtl[10];
char   *cPtr;

width  = x2-x1+1;

width = (width + 7)/8;

fprintf(stdprn,"     %s\r\n\r\n", title );

/* Init printer graphics mode */

prn_putc(ESC);
prn_putc('*');
prn_putc('t');

if ((x2-x1) < 600)
  {
  /* for 75 dpi */
  prn_putc('7');
  prn_putc('5');
  }
else
  {
  /* for 100 dpi */
  prn_putc('1');
  prn_putc('0');
  prn_putc('0');
  }

prn_putc('R');

sprintf(lineCtl,"%d", width);

for (y=y1; y<=y2; y++)
  {
  prn_putc(ESC);
  prn_putc('*');
  prn_putc('b');
  /****************************************
  * Ascii number string for # of graphics
  * bytes to follow
  */
  cPtr = lineCtl;
  while (*cPtr != '\0')
    {
    prn_putc( *cPtr );
    cPtr++;
    }
  prn_putc('W');

  for (x=x1; x< x1+8*width; x+=8)
    {
    prn_putc(getHpScrBits(x,y));
    }
  }

/****************************************
* End graphics mode
*/

prn_putc(ESC);
prn_putc('*');
prn_putc('r');
prn_putc('B');

fprintf(stdprn,"%c", 12 );

return(0);
}

