/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... RADCAD.C        */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 13-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Create objects  */
/*  using the mouse.                                    */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
09-APR-90 MR Final cleanup of RADCAD


*\r*/

/******************************

The main() presents the program title on a simple
text display, then uses functions in LAB.C (and
VIDEOID.ASM) to detect all available graphics
modes in your computer. It presents a list of
those modes, no more and no less (so you cannot
attempt to run this program in a graphics mode
the machine isn't capable of), for the user
to run. Main handles the horizontal menu bar
across the top of the display, and dispatches
control to a separate handler for each menu
that drops down below the entry at the top.
*/


/* ---------- System include files -------------------- */

#include <stdlib.h>
#include <stdio.h>      /* Text & file I/O functions and*/
                        /* NULL are defined here        */
#include <conio.h>      /* I like to use getch()        */
#include <graphics.h>   /* For "interactive graphics"   */
                        /* we will always include this  */
#include <math.h>       /* Need trig functions for      */
                        /* polygon generator            */
#include <string.h>     /* Input numbers                */
#include <dos.h>


/* ---------- User include files ---------------------- */

#include "lab.h"        /* Definitions common to all    */
                        /* lab projects by MJWR         */
#include "mouse.h"

#include "matrix3d.h"   /* Our 3d vector package and    */
                        /* data types                   */
#include "menu.h"       /* Prototype for my budding     */
                        /* menu driver package          */
#include "spclmenu.h"   /* Special menu declarations    */

#include "objdefs.h"    /* Object structures            */

#include "objops.h"     /* Object operations            */

#include "radcad.h"       /* Radcad structures              */
#include "circle.h"     /* My circle draw procedure     */



/* ---------- Module globals -------------------------- */

/* Big stack please */

extern unsigned _stklen = 0xA000;

#define SHAREWARE 0

char    programName[]   = {"RADCAD.C"};

char   *description[]   = {
"Ŀ",
"Rad CAD 2.0 by Matthew J. W. Ratcliff, Update: 01-DEC-90   ",
"͵",
"Rad CAD is a growing 3D graphics package.  It is a drawing ",
"program that allows you to create and manipulate objects   ",
"such as lines, points, circles, triangles, squares,        ",
"rectangles, and so forth.  The objects may then be moved,  ",
"scaled (larger or smaller), rotated, or SPUN around an     ",
"arbitrary axis to become a 3D object.                      ",
"Pictures can be saved to a file, printed to an EPSON       ",
"compatible printer, and loaded once again. Animation is    ",
"currently supported.  Load the sample file DOG.CAD and     ",
"select ANIMATE from the OBJECTS menu for an example.       ",
"Ĵ",
"Future enhancements include hidden line removal and text   ",
"objects.                                                   ",
#if SHAREWARE
" * SHAREWARE * ͵",
"Send $25 Registration fee : Matthew Ratcliff               ",
"Ŀ  Ratware Softworks              ",
"Fee is to register & use   PO Box 295                     ",
"SOURCE and/or EXECUTABLE   Villa Ridge, MO  63089         ",
"",
#else
"͵",
" Written In Borland C by  Matthew Ratcliff               ",
"Ŀ  Ratware Softworks              ",
"TeamB-CIS ID 76711,1443  PO Box 295                     ",
"ٳ  Villa Ridge, MO 63135          ",
"",
#endif
NULL };

/*----------
* We will use an object array which
* is dynamically reallocated as necessary
* during the creation, manipulation and
* destruction of objects.  Any object that
* is Destroyed will have its pointer set to
* NULL in this list.  A procedure GetObjectPtr
* will first search this global list for a
* NULL pointer.  If one is available, that index
* is returned.  If none found, then the object
* list is increased by the size of one object
* pointer.  GetObjectPtr manages the mess.
*/


SYSINFO_TYPE Si;        /* System info structure */

/*
OBJECT_TYPE **ObjectList;
int TotalObjects        = 0;
int TotalActiveObjects  = 0;

int ObjectColor = WHITE;
int MenuColor   = WHITE;
int BkColor     = BLACK;
int MenuBkColor = LIGHTGRAY;
int ErrorColor  = LIGHT_RED;
int ShowMousePos = TRUE;
*/
/*-----------------------------------
Below are the menus for this program
They are used by the module
GrafMenuSelect, in file MENU.C
prototype in MENU.H
-------------------------------------*/

/* Get module global
 * menu variables
 */

#include "menusv.h"

/* Global flag to enable/disable display of mouse coordinates */

GRAPH_DESCR_TYPE        graf;



/* ---------- Module procedure prototypes ------------- */


/****************************************
* See RADCAD.H
*/

/* The main control center of this program:
 */
void main( void );


/*\p*********************
 Procedure:
 main

 Inputs:
 none

 Output:
 Generate an object with
 mouse input control

 Globals:
 all module globals
\p**********************/



void main( void )
{ /* main begin */

int                     status;
int                     exitnow;
/* Iniz variables:
*/

Si.TotalObjects         = 0;
Si.TotalActiveObjects   = 0;
Si.ObjectColor          = WHITE;
Si.MenuColor            = WHITE;
Si.BkColor              = BLACK;
Si.MenuBkColor          = LIGHTGRAY;
Si.ErrorColor           = LIGHTRED;
Si.ShowMousePos         = TRUE;

if (graf.Driver != CGA)
  Si.ObjectSelectColor    = LIGHTCYAN;
else
  Si.ObjectSelectColor    = WHITE;

/*
 Show program title, author, etc.
*/

PresentTitle ( programName, description );

/*
 Prompt for desired graphics mode
*/

status = SelectGraphicsMode( &graf, "Rad-CAD by Ratcliff" );

/*
 Main control loop, until DONE.
*/

exitnow = FALSE;
if (status == SUCCEEDED)
  { /* if SUCCEEDED then perform */
  if (graf.Driver == CGA)
    {
    SetNormalMenu( FALSE );
    }
  SetStdMenuColors( );
  InizMenuDriver( &graf );
  status = initmouse();

  /****************************************
   * If mouse driver not found, inform
   * the user and allow him/her to exit
   * program using keyboard emulated
   * mouse.  After first hidemouse() the
   * computer ALWAYS locks up. This is
   * a bug in the MOUSE.C program. I didn't
   * write it and do NOT have time to debug
   * it!  It works long enough to allow
   * user to make one menu selection, which
   * is to EXIT my program without
   * crashing. Then he/she can load mouse
   * driver and try again!
   */

  if (status == 0)
    {
    Mshowmouse();
    VertMenu( DIALOG_BOX_INDEX,
              keyMouseMsg,
              keyMouseMsgSel,
              BK_OVRWRITE );
    closegraph();
    printf("End of %s by Matthew J. W. Ratcliff\n\n", programName);
    exit(0);
    }
  else
    {
    Mhidemouse();
    movemouse( graf.Max.x/2, graf.Max.y/2 );
    Mshowmouse();
    VertMenu( DIALOG_BOX_INDEX,
              keyMouseMsg1,
              keyMouseMsgSel,
              BK_OVRWRITE );
    }
  do
    {

    /* Mouse driven graphical menu selection
    */
    status = GrafMenuSelect( mainMenu, ERASE_NEVER );

    /* Decode mouse selection and continue
    */
    switch( status )
      {
      case( MAIN_CREATE ):
        CreateHandler( &Si );
        break;
      case( MAIN_ERASE ):
        EraseHandler( &Si );
        break;
      case( MAIN_OBJECTS ):
        ObjectsHandler( &Si );
        break;
      case( MAIN_FILE ):
        FileHandler( &Si );
        break;
      case( MAIN_MISC ):
        MiscHandler( &Si );
        break;
      case( MAIN_DONE ):
        exitnow = DoneHandler( );
        break;
      default:
        break;
      }
    }
  while (exitnow == FALSE);
  Mhidemouse();
  closegraph();
  }

printf("End of %s by Matthew J. W. Ratcliff\n\n", programName);
exit(0);
} /* main end */



/****************************************
 * DoneHandler - Present the DONE menu.
 * If the user is certain he wishes to
 * exit the program, return TRUE. Else
 * return FALSE.
 */

int DoneHandler( void )

{ /* DoneHandler begin */
int status;
int exitnow;

exitnow = FALSE;

SetErrorColors( );
status = VertMenu( MAIN_DONE, doneMenu, doneMenuSel, BK_SAVE );
SetStdMenuColors( );

if (status == DONE_NOW)
  {
  DestroyAllObjects();
  cleardevice();
  exitnow = TRUE;
  }

return( exitnow );

} /* DoneHandler end */


/****************************************
* Setup the menu speed flag and various
* color configurations for RADCAD.
* Some are set in MENU.C some here
* in RADCAD.C.  It is a bit awkward and
* should be rewritten, but, frankly, I'm
* sick of working on it!
*
*/


static int normalMenuSpeedFlag = TRUE;

/****************************************
 * void SetErrorColors( void );
 *
 * Set colors for error dialog boxes
 *
 */

void SetErrorColors( void )

{ /* SetErrorColors begin */

/* CGA test */

if (graf.Driver == CGA)
  {
  SetMenuColors( 1,
                 1,
                 0,
                 1,
                 1,
                 1,
                 normalMenuSpeedFlag );
  Si.ObjectColor = 1;
  Si.MenuColor   = 1;
  Si.ErrorColor  = 1;
  Si.BkColor     = 0;
  }

/* EGA test */
else if ( (graf.Driver == EGA) ||
          (graf.Driver == VGA) )
  {
  if (normalMenuSpeedFlag)
    SetMenuColors( LIGHTRED,
                   RED,
                   YELLOW,
                   BLACK,
                   RED,
                   BLUE,
                   normalMenuSpeedFlag );
  else
    SetMenuColors( LIGHTRED,
                   RED,
                   YELLOW,
                   BLACK,
                   RED,
                   LIGHTBLUE,
                   normalMenuSpeedFlag );
  }

} /* SetErrorColors end */

/****************************************
 * void SetStdMenuColors( );
 *
 * Set colors for normal menus
 *
 */

void SetStdMenuColors( void )

{ /* SetStdMenuColors begin */

/* CGA test */

/* Call format:

void SetMenuColors( int topedgecolor,
                    int botedgecolor,
                    int facecolor,
                    int textcolor,
                    int prmptcolor,
                    int bcolor,
                    int normalspeed );
*/

if   (graf.Driver == CGA)
  {
  SetMenuColors( 1,
                 1,
                 0,
                 1,
                 1,
                 1,
                 normalMenuSpeedFlag );
  Si.ObjectColor = 1;
  Si.MenuColor   = 1;
  Si.ErrorColor  = 1;
  Si.BkColor     = 0;
  }

/* EGA/VGA test */
else if ( (graf.Driver == EGA) ||
          (graf.Driver == VGA) )
  {
  if (normalMenuSpeedFlag)
    {
    SetMenuColors( WHITE,
                   DARKGRAY,
                   LIGHTGRAY,
                   BLACK,
                   BLUE,
                   LIGHTCYAN,
                   normalMenuSpeedFlag );
    Si.ObjectColor = WHITE;
    Si.MenuColor   = BLUE;
    Si.ErrorColor  = RED;
    }
  else
    {
    SetMenuColors( LIGHTRED,
                   RED,
                   YELLOW,
                   BLACK,
                   RED,
                   LIGHTBLUE,
                   normalMenuSpeedFlag );
    Si.ObjectColor = WHITE;
    Si.MenuColor   = LIGHTBLUE;
    Si.ErrorColor  = RED;
    }
  }

} /* SetStdMenuColors end */

/****************************************
 * Set normal menu flag to true or false.
 * If true, pretty menus are employed.
 * If false, then simple outline menus
 * are used.
 */

void SetNormalMenu( int tf_flag )


{ /* SetNormalMenu begin */

normalMenuSpeedFlag = tf_flag;

if ( tf_flag == FALSE )
  {
  if (graf.Driver != CGA)
    {
    Si.MenuBkColor = BLACK;
    Si.MenuColor   = LIGHTBLUE;
    }
  }
else
  {
  if (graf.Driver != CGA)
    {
    Si.MenuBkColor = LIGHTGRAY;
    Si.MenuColor   = BLUE;
    }
  }
} /* SetNormalMenu end */



/****************************************
 * Get an object index.  Search the list
 * for a NULL.  If found, return its index.
 * If none is found, then realloc the object
 * list with an increased size of one.  If
 * the result is a NULL, then out of memory.
 * Return a GET_OBJECT_FAILED status if we
 * couldn't create a new object.
 */


int GetObjectIndex( void )

{ /* GetObjectIndex begin */

int index;
int exitnow;

OBJECT_TYPE **newObject;

index = GET_NEW_OBJECT;

if (Si.TotalObjects > 0)
  {
  index = 0;
  exitnow = FALSE;
  do
    {
    if (Si.ObjectList[index] == NULL)
      exitnow = TRUE;
    else
      index++;
    if (index == Si.TotalObjects)
      {
      index = GET_NEW_OBJECT;
      exitnow = TRUE;
      }
    }
  while( exitnow == FALSE);
  }

if (index == GET_NEW_OBJECT)
  {
  newObject = (OBJECT_TYPE **)realloc( Si.ObjectList,
              (Si.TotalObjects+1)*sizeof(OBJECT_TYPE *) );
  if (newObject != NULL)
    {

    /* Now we have Si.TotalObjects+1 place holders */
    /* for OBJECT_TYPE pointers, the caller may    */
    /* now CREATE a new object and tack it on	   */

    Si.ObjectList = newObject;
    index = Si.TotalObjects;
    Si.ObjectList[index] = NULL;
    Si.TotalObjects = Si.TotalObjects + 1;

    }
  else
    {
    ReportNoMemory();
    index = GET_OBJECT_FAILED;
    }
  }

if (index != GET_OBJECT_FAILED)
  Si.TotalActiveObjects = Si.TotalActiveObjects + 1;

return( index );
} /* GetObjectIndex end */


/****************************************
 * Special menus and dialog boxes
 */



/****************************************
 * Can't perform requested memory allocation
 */

void ReportNoMemory( void )

{ /* ReportNoMemory begin */


VertMenu( DIALOG_BOX_INDEX, noMemoryError, NULL, BK_OVRWRITE );
RedrawDisplay( Si.ObjectColor );

} /* ReportNoMemory end */



/****************************************
* Can't perform requested UNDO
*/

void ReportNoUndo( void )
{ /* ReportNoUndo begin */

StatusLine("Cannot UNDO this object", Si.ErrorColor );
AudioError();
delay(400);
MpurgeButtons();
} /* ReportNoUndo end */



/****************************************
 * Under construction dialog
 */

void UnderConstruction( int index )
{ /* UnderConstruction begin */

VertMenu( index, constDialog, constDialogSel, BK_SAVE );

} /* UnderConstruction end */


/****************************************
* void ObjectTooLarge( void );
*/

void ObjectTooLarge( void )
{ /* ObjectTooLarge begin */

VertMenu( DIALOG_BOX_INDEX, tooBig, tooBigSel, BK_SAVE );

} /* ObjectTooLarge end */



/****************************************
 * Destroy all objects allocated. Step
 * through the whole list of current
 * objects and make multiple calls to
 * the DestroyObject procedure to free
 * their memory.  Then free the Si.ObjectList.
 */

void DestroyAllObjects( void )

{ /* DestroyAllObjects begin */

int index;

if (Si.TotalObjects > 0)
  {
  for(index = 0; index < Si.TotalObjects; index++)
    {
    if (Si.ObjectList[index] != NULL)
      {
      DestroyObject( &Si.ObjectList[index] );
      }
    }
  Si.TotalObjects = 0;
  Si.TotalActiveObjects = 0;
  }

RedrawDisplay( Si.ObjectColor );

} /* DestroyAllObjects end */



/****************************************
 * int SelectObject() - Allow the user
 * to select one of the objects on
 * the display.  Return an index into the
 * Si.ObjectList[] for the item chosen.
 *
 * I use the max/min method.  Find the pointer
 * within the rectangle of an object.
 * Continue the search until the end of the
 * object list, keeping index to the smallest
 * rectangle within which the mouse pointer
 * was found.
 */

#define SELECT_MSG_SIZE 30

int SelectObject( const char *msg )

{ /* SelectObject begin */


int             index;
int             button;
int             tempindex;
int             inside;
float           area1;
float           area2;
int             firstfound;
int             exitnow;
VIEW_COORD_TYPE pos;
VIEW_COORD_TYPE size;
VIEW_COORD_TYPE minsize;
OBJECT_TYPE     *obPtr;
char            str[DEFAULT_STR_SIZE];

strcpy(str,"Pick object ");
strncat(str, msg, DEFAULT_STR_SIZE-SELECT_MSG_SIZE );
strcat(str, ", LFT-OK RT-CANCEL");
StatusLine( str, Si.MenuColor );
exitnow = FALSE;

do
  { /* do until RIGHT_BUTTON cancel or LEFT_BUTTON & selected */
  GetPoint( NULL, &pos, &button, Si.ShowMousePos );
  if (button == RIGHT_BUTTON)
    { /* if cancel button */
    index = OBJECT_NOT_SELECTED;
    StatusLine("Cancel Object Select", WHITE);
    exitnow = TRUE;
    } /* end if cancel button */
  else
    { /* else LEFT_BUTTON, find object */
    StatusLine("Searching Object List", WHITE);
    firstfound = FALSE;
    tempindex = OBJECT_NOT_SELECTED;
    for (index=0; index<Si.TotalObjects; index++)
      { /* search loop */
      obPtr = Si.ObjectList[index];
      if (obPtr->Operation != DELETED_OP) /* no sel if not here! */
        { /* if not deleted */
        inside = mouseinbox( obPtr->sMin.x,
                             obPtr->sMin.y,
                             obPtr->sMax.x,
                             obPtr->sMax.y,
                             pos.x, pos.y );
        if (inside == TRUE)
          { /* if inside */
          size.x = obPtr->sMax.x - obPtr->sMin.x;
          size.y = obPtr->sMax.y - obPtr->sMin.y;
          if (firstfound == FALSE)
            { /* end if first match */
            firstfound = TRUE;
            minsize.x = size.x;
            minsize.y = size.y;
            tempindex = index;
            } /* end if first match */
          else
            { /* if check new minimum */
            area1 = (float)size.x * (float)size.y;
            area2 = (float)minsize.x * (float)minsize.y;
            if (area1 < area2)
              { /* if smaller area */
              minsize.x = size.x;
              minsize.y = size.y;
              tempindex = index;
              } /* end if smaller area */
            } /* end else check new minimum */
          } /* end if pointer inside min/max area */
        } /* end if not deleted */
      } /* end search loop */
    index = tempindex;
    if (index != OBJECT_NOT_SELECTED)
      { /* if selected */
      exitnow = TRUE;
      } /* end if selected */
    else
      {
      StatusLine("Object not found. RIGHT=CANCEL", Si.MenuColor);
      }
    } /* end else LEFT_BUTTON, find object */
  }
while (exitnow == FALSE);

return( index );

} /* SelectObject end */



/****************************************
 * RedrawDisplay - Redraw all objects
 * on the display.  The caller specifies
 * the color.
 */

void RedrawDisplay( int color )

{ /* RedrawDisplay begin */

int index;

ClearWorkArea();
if (Si.TotalActiveObjects != 0)
  {
  for (index=0; index<Si.TotalObjects; index++)
    {
    if (Si.ObjectList[index] != NULL)
      if (Si.ObjectList[index]->Operation != DELETED_OP)
        DrawObject( Si.ObjectList[index], color );
    }
  }

} /* RedrawDisplay end */


/****************************************
* We have all the data for an object structure
* in a VIEW_COORD_TYPE array.  Copy it over to
* the associated OBJECT_TYPE, and setup the
* the homogeneous coordinates and other elements.
*/



void InizObject( OBJECT_TYPE *obPtr, int totalPts,
                 VIEW_COORD_TYPE *sPtr )

{ /* InizObject begin */

int index;
VIEW_COORD_TYPE *wsPtr;

wsPtr = sPtr;

for (index=0; index<totalPts; index++)
  {
  obPtr->vPts[index].x = (float)wsPtr->x;
  obPtr->vPts[index].y = (float)wsPtr->y;
  obPtr->sPts[index].x = wsPtr->x;
  obPtr->sPts[index].y = wsPtr->y;
  wsPtr++;
  }

CenterMinMaxObject( obPtr );
DrawObject( obPtr, Si.ObjectColor );

} /* InizObject end */



/****************************************
* FrameObject( obPtr ); - Frame the
* object pionted to.  Do this by
* setting the write mode to XOR, and
* drawing a box between sMin, sMax
* components of the object's structure.
* Call this a second time with the same
* obPtr to erase the outline, and
* restore the original image.
*/

void FrameObject( OBJECT_TYPE *obPtr, int showcenter )

{ /* FrameObject begin */

VIEW_COORD_TYPE center;

setcolor( Si.ObjectColor );
Lsetwritemode( XOR_PUT );
Mhidemouse();

box( obPtr->sMin.x-1, obPtr->sMin.y-1,
     obPtr->sMax.x+1, obPtr->sMax.y+1,
     Si.ObjectColor, FALSE );

if (showcenter == TRUE)
  {
  center.x = obPtr->vCenter.x;
  center.y = obPtr->vCenter.y;
  line(center.x-2, center.y-2, center.x+2, center.y+2 );
  line(center.x-2, center.y+2, center.x+2, center.y-2 );
  line(obPtr->sMin.x, center.y, obPtr->sMin.x+2, center.y );
  line(obPtr->sMax.x, center.y, obPtr->sMax.x-2, center.y );
  line(center.x, obPtr->sMin.y, center.x, obPtr->sMin.y+2 );
  line(center.x, obPtr->sMax.y, center.x, obPtr->sMax.y-2 );
  }

Mshowmouse();
Lsetwritemode( COPY_PUT );
} /* FrameObject end */


/****************************************
* DragCircle - Allow the caller to drag
* a circle anywhere on the display and
* exit on either button click, returning
* the final center of the circle selected
*
*/


int DragCircle( VIEW_COORD_TYPE *center,
                int radius )

{ /* DragCircle begin */

VIEW_COORD_TYPE pos;
int             button;

setwritemode( XOR_PUT );

Mhidemouse();
DrawCircle( center->x, center->y, radius, Si.ObjectColor);
Mshowmouse();
movemouse( center->x, center->y );

MpurgeButtons();

setwritemode( XOR_PUT );
do
  {
  getmousecoords( &pos.x, &pos.y );
  if ( (pos.x != center->x) ||
       (pos.y != center->y) )
    {
    Mhidemouse();
    DrawCircle( center->x, center->y, radius,
                Si.ObjectColor );
    DrawCircle( pos.x, pos.y, radius,
                Si.ObjectColor ); /* Draw */
    Mshowmouse();
    center->x = pos.x;
    center->y = pos.y;
    if (Si.ShowMousePos)
      ReportMousePos( pos.x, pos.y );
    }
  button = ReadMouseButton();
  }
while (button == NEITHER_BUTTON_PRESSED);

Mhidemouse();
DrawCircle( center->x, center->y, radius,
            Si.ObjectColor ); /* Draw */
Mshowmouse();
setwritemode( COPY_PUT );

return(button);

} /* DragCircle end */



/****************************************
* int SizeCircle( VIEW_COORD_TYPE *center,
*                 int             *radius )
*
* Allow the user to change the size of a
* circle, adjusting the radius.
*
* Return the button pressed upon completion.
*/

int SizeCircle( VIEW_COORD_TYPE *center,
                int             *radius )

{ /* SizeCircle begin */

VIEW_COORD_TYPE pos;
int             button;
int             cx;
int             validarea;

setwritemode( XOR_PUT );

Mhidemouse();
DrawCircle( center->x, center->y, *radius, Si.ObjectColor );
Mshowmouse();
cx = center->x + *radius;

movemouse( center->x+*radius, center->y );


do
  {
  do
    {
    getmousecoords( &pos.x, &pos.y );
    button = ReadMouseButton();
    validarea = mouseinbox( center->x+1, center->y+1,
                            graf.Max.x, graf.Max.y,
                            pos.x, pos.y );
    }
  while ((validarea == FALSE) && (button == NEITHER_BUTTON_PRESSED) );

  if ( (pos.x != cx) && (button == NEITHER_BUTTON_PRESSED) )
    {
    Mhidemouse();
    DrawCircle( center->x, center->y, *radius,
                Si.ObjectColor); /* erase */
    *radius = pos.x - center->x;
    cx      = center->x + *radius;
    DrawCircle( center->x, center->y, *radius,
                Si.ObjectColor ); /* draw */
    Mshowmouse();
/*    movemouse( cx, center->y );*/
    }
  if (button == NEITHER_BUTTON_PRESSED)
    button = ReadMouseButton();
  }
while (button == NEITHER_BUTTON_PRESSED);

Mhidemouse();
DrawCircle( center->x, center->y, *radius,
            Si.ObjectColor ); /* ERASE */
Mshowmouse();

setwritemode( COPY_PUT );
return( button );

} /* SizeCircle end */



/****************************************
*
* No objects active. Tell the user
* followed by the message the
* caller passed in, if any.
*/

void noObjects( const char *msg, int color )

{ /* noObjects begin */

char str[DEFAULT_STR_SIZE+1];
int i;
strcpy( str, "NO OBJECTS FOUND " );

if (msg != NULL)
  {
  i = DEFAULT_STR_SIZE - strlen(str);
  strncat( str, msg, i );
  }

StatusLine( str, color );
AudioError();

} /* noObjects end */

