/********************************************************/
/*  Author   .......................... Matthew Ratcliff*/
/*  Program name ...................... RADCAD.H        */
/*  Course ............................ CSc 343, W90    */
/*  Instructor ........................ Sabharwal       */
/*  Office ............................ Rm.33, Blue Bld.*/
/*  Phone ............................. 553-5430        */
/*  Date created  ..................... 13-MAR-90       */
/*  System   .......................... IBM PC/AT Compat*/
/*  Language .......................... Turbo C 2.01    */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Function .......................... Create objects  */
/*  using the mouse.                                    */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
09-APR-90 MR Final cleanup of RADCAD


*\r*/



/****************************************
* Enumeration for the MAIN menu entries
* Kept here so that other HANDLER files
* can use these a constant reference
* values for proper positioning of their
* menus off the main menu bar.
*/

enum MAIN_MENU_SEL { MAIN_CREATE,
                     MAIN_ERASE,
                     MAIN_OBJECTS,
                     MAIN_FILE,
                     MAIN_MISC,
                     MAIN_DONE };

/****************************************
*
* Data structures and interface support
* routines for the min menu interfaces
* for LAB5.
*
*/


typedef struct SYSINFO_TYPE
  {
  OBJECT_TYPE **ObjectList;
  int TotalObjects;
  int TotalActiveObjects;
  int ObjectColor;
  int MenuColor;
  int BkColor;
  int ErrorColor;
  int MenuBkColor;
  int ObjectSelectColor;
  int ShowMousePos;
  } SYSINFO_TYPE;



/****************************************
* Result codes of GetObjectIndex function
*/

#define GET_OBJECT_FAILED -1
#define GET_NEW_OBJECT    -2

#define OBJECT_NOT_SELECTED -1



/* Procedure to set colors for menus and dialog boxes
 * related to error and warning conditions
 */

void SetErrorColors( void );

/* Procedure to set colors for menus and dialog
 * boxes for the standard display format
 */

void SetStdMenuColors( void );

/* Set normal menu TRUE or FALSE. If true, standard
 * "pretty" back filled menus are employed.
 * If false, then simple menu outlines are used, faster
 * but not as pretty
 */

void SetNormalMenu( int tf_flag );

/****************************************
 * Submenu handlers executed from the
 * main menu control:
 */

/****************************************
* Separate handlers with their own, sometimes
* complex sub-menus and user interaction
*
*/

int        CreateHandler ( SYSINFO_TYPE *si );
int        EraseHandler  ( SYSINFO_TYPE *si );
int        ObjectsHandler( SYSINFO_TYPE *si );
int        FileHandler   ( SYSINFO_TYPE *si );
int        MiscHandler   ( SYSINFO_TYPE *si );

/****************************************
* Done is relatively trivial and is
* part of RADCAD.C, immediately following
* the MAIN() function.
*/

int        DoneHandler   ( void );



/****************************************
* Support routines in RADCAD.C to make
* user interaction consistent and simpler
* to implement.
*/

void       DestroyAllObjects( void );
int        SelectObject( const char *msg );
void       RedrawDisplay( int color );
void       InizObject( OBJECT_TYPE *obPtr,
                       int totalPts,
                       VIEW_COORD_TYPE *sPtr );

#define SHOW_CENTER TRUE

void       FrameObject( OBJECT_TYPE *obPtr, int showcenter );
int        GetObjectIndex( void );

/****************************************
* int SizeCircle( VIEW_COORD_TYPE *center,
*                 int             *radius )
*
* Allow the user to change the size of a
* circle, adjusting the radius.
*
* Return the button pressed upon completion.
*/

int SizeCircle( VIEW_COORD_TYPE *center,
                int             *radius );


/****************************************
* DragCircle - Allow the caller to drag
* a circle anywhere on the display and
* exit on either button click, returning
* the final center of the circle selected
*
*/

int DragCircle( VIEW_COORD_TYPE *center,
                int radius );

