
%OUT C declaration for Videoid:  extern unsigned long far VideoId( void );

;
; see page 513, Appendix C of
; "Programmer's Guide to PC & PS/2 Video Systems"
; Adapted for Turbo C, small memory model, by Richard S. Sadowsky

;
; Name:         VideoID
;
; Function:     Detects the presence of various video subsystems
;               and associated monitors.
;
;               Returns 4 byte structure as an unsigned long.
;               By declaring as external, far, and working as
;               a function (rather than a procedure which would
;               require memory model dependent pointers), this
;               program works with ANY memory modem SMALL through HUGE.
;               (For SMALL and other "NEAR CALL" type models, the
;               "external long far" declaration is imperitive! Use
;               #include "VIDEOID.H".)
;               -- Revisions by Matthew J. W. Ratcliff
;               -- 19-JAN-90
;
; Caller:       Turbo C or Microsoft C
;
;               extern unsigned long far VideoId ( void );
;
;               struct typedef VID_TYPE
;                 {
;                 unsigned char VideoSubsystem;
;                 unsigned char Display;
;                 } VID_TYPE;
;
;               VID_TYPE *sysVid;
;               unsigned long vidwords;
;               vidwords = VideoID();
;               sysVid   = (VID_TYPE *)&vidwords;
;
;               sysVid[0].VideoSubsystem - Active video adapter
;               sysVid[0].Display        - Active display type
;               sysVid[1].VideoSubsystem - Inactive secondary video adapter
;               sysVid[1].Display        - Inactive secondary display type
;
;       Subsystem ID values:
;                               0 = (none)
;                               1 = MDA (Monochrome Display Adapter,
;                                   text only)
;                               2 = CGA
;                               3 = EGA
;                               4 = MCGA, PS/2's
;                               5 = VGA
;                              80h= HGC, Hercules Graphics Controller
;                              81h= HGC+, Hercules Graphics+
;                              82h= Hercules InColor
;
;       Display types:
;                               0 = (none)
;                               1 = MDA compatible monochrome
;                               2 = CGA compatible color
;                               3 = EGA compatible color
;                               4 = PS/2 compatible monochrome
;                               5 = PS/2 compatible color
;
;       The values returned in VIDstruct[0].VideoSubsystem and
;       VIDstruct[0].Display indicate the currently active subsystem.
;
;       The values in VIDstruct[1] indicate the alternate display
;       adapter in the system, if available. (Both entries will be
;       zero if there is only one display adapter in the system.)
;
;

;
; Pointer definitions for access to our local data structures:
;
ARGpVID         EQU     word ptr VIDstruct

Device0          EQU   WORD PTR Video0Type[DI]
Device1          EQU   WORD PTR Video1Type[DI]

; Subsystem type constants:

MDA              EQU   1        ; Subsystem types
CGA              EQU   2
EGA              EQU   3
MCGA             EQU   4
VGA              EQU   5
HGC              EQU   80h
HGCPlus          EQU   81h
InColor          EQU   82h

; Display type constants:

MDADisplay       EQU   1        ; Display types
CGADisplay       EQU   2
EGAColorDisplay  EQU   3
PS2MonoDisplay   EQU   4
PS2ColorDisplay  EQU   5

; Control flags:

TRUE             EQU   1
FALSE            EQU   0



_TEXT           SEGMENT byte public 'CODE'
                ASSUME CS:_TEXT,ds:_TEXT

                 PUBLIC _VideoID

_VideoID        PROC    FAR

       PUSH      SI             ; Push caller registers
       PUSH      DI
       PUSH      DS             ; Works with ANY memory model
       PUSH      CS             ; Our data in our code segment
       POP       DS

; Initialize the data structure that will contain the results

       MOV       DI,ARGpVID     ; DS:DI -> start of data structure

       MOV       Device0,0      ; zero these variables
       MOV       Device1,0

;
; Look for the various subsystems using the subroutines whose
; addresses are tabulated in TestSequence; each subroutine sets flags
; in TestSequence to indicate whether the subsequent subroutines need to be
; called
;
       MOV       BYTE PTR TestSequence,TRUE
       MOV       BYTE PTR EGAflag,TRUE
       MOV       BYTE PTR CGAflag,TRUE
       MOV       BYTE PTR Monoflag,TRUE

       MOV       CX,NumberOfTests
       MOV       SI,OFFSET TestSequence

L01:
       LODSB                    ; AL := flag
       TEST      AL,AL
       LODSW                    ; AX := subroutine address
       JZ        L02            ; skip subroutine if flag is false

       PUSH      SI
       PUSH      CX             ; Call subroutine to detect subsystem
       CALL      AX
       POP       CX
       POP       SI

L02:   LOOP      L01

; determine which subsystem is active

       CALL      near ptr FindActive

       MOV       AX,Device0
       MOV       DX,Device1
       POP       DS             ; Restore caller's data segment
       POP       DI             ; Restore caller registers and return
       POP       SI
       RET

_VideoID         ENDP

;
; FindPS2
;
;       This subroutine ues INT 10h function 1Ah to determine the video
;       BIOS display combination code (DCC) for each video subsystem
;       present.
;

FindPS2          PROC NEAR

       MOV       AX,1A00h
       INT       10h            ; Call video BIOS for info

       CMP       AL,1Ah         ; Exit if not supported (i.e.,
       JNE       L13            ; no MCGA or VGA in system)

; convert BIOS DCCs into specific subsystems & displays

       MOV       CX,BX
       XOR       BH,BH          ; BX := DCC for active subsystem
       OR        CH,CH
       JZ        L11            ; jump if only one subsystem present

       MOV       BL,CH          ; BX := inactive DCC
       ADD       BX,BX
       MOV       AX,[BX+OFFSET DCCtable]

       MOV       Device1,AX

       MOV       BL,CL
       XOR       BH,BH          ; BX := active DCC

L11:   ADD       BX,BX
       MOV       AX,[BX+OFFSET DCCtable]

        MOV       Device0,AX

; Reset flags for subsystems that have been ruled out

       MOV       BYTE PTR CGAflag,FALSE
       MOV       BYTE PTR EGAflag,FALSE
       MOV       BYTE PTR Monoflag,FALSE

       LEA       BX,Video0Type[DI]      ; if the BIOS reported an MDA...
       CMP       BYTE PTR [BX],MDA
       JE        L12

       LEA       BX,Video1Type[DI]
       CMP       BYTE PTR [BX],MDA
       JNE       L13

L12:   MOV       WORD PTR [BX],0        ; ... hercules can't be ruled out
       MOV       BYTE PTR Monoflag,TRUE

L13:   RET

FindPS2          ENDP

;
; FindEGA
;
; Look for an EGA.  This is done by making a call to an EGA BIOS function
; which doesn't exist in the default (MDA, CGA) BIOS.
;

FindEGA          PROC NEAR      ; Caller:       AH = flags
       MOV       BL,10h         ; Returns:      AH = flags
       MOV       AH,12h         ;               Video0Type and
       INT       10h            ;                Display0Type updated
                                ; BL := 10h (return EGA info)
                                ; AH := INT 10h function number
                                ;  call EGA BIOS for info
                                ;  if EGA BIOS is present,
                                ;  BL <> 10H
                                ;  CL = switch setting

       CMP       BL,10h
       JE        L22            ; jump if EGA BIOS not present

       MOV       AL,CL
       SHR       AL,1           ; AL := switches/2

       MOV       BX,OFFSET EGADisplays
       XLAT                     ; determine display type from switches
       MOV       AH,AL          ; AH := display type
       MOV       AL,EGA         ; AL := subsystem type
       CALL      near ptr FoundDevice

       CMP       AH,MDADisplay
       JE        L21            ; jump if EGA has a monochrome display

       MOV       CGAflag,FALSE  ; no CGA if EGA has color display
       JMP       SHORT L22

L21:   MOV       Monoflag,FALSE ; EGA has a mono display, so MDA and
                                ; Hercules are ruled out
L22:   RET

FindEGA          ENDP

;
; FindCGA
;
;       This is done by looking for the CGA's 6845 CRTC at I/O port 3D4h
;

FindCGA          PROC NEAR      ; Returns:      VIDstruct updated

       MOV       DX,3D4h        ; DX := CRTC address port
       CALL      near ptr Find6845
       JC        L31            ; jump if not present

       MOV       AL,CGA
       MOV       AH,CGADisplay
       CALL      near ptr FoundDevice

L31:   RET

FindCGA          ENDP


;
; FindMono
;
;       This is done by looking for the MDA's 6845 CRTC at I/O port 3B4h.
;       If the 6845 is found, the subroutine distinguishes between an
;       MDA and a Hercules adapter by monitoring bit 7 of the CRT
;       status byte. This bit changes on Hercules adapters but does not
;       change on an MDA. The various Hercules adapters are identified
;       by bits 4 through 6 of the CRT status value:
;
;       000b = HGC
;       001b = HGC+
;       101b = InColor card
;

FindMono         PROC NEAR      ; Returns:      VIDstruct updated

       MOV       DX,3B4h        ; DX := CRTC address port

       CALL      near ptr Find6845
       JC        L44            ; jump if not present

       MOV       DL,0BAh        ; DX := 3BAh (status port)
       IN        AL,DX
       AND       AL,80h
       MOV       AH,AL          ; AH := bit 7 (virtal sync on HGC)

       MOV       CX,8000h       ; do this 32768 times
L41:   IN        AL,DX
       AND       AL,80h         ; isolate bit 7
       CMP       AH,AL
       LOOPE     L41            ; wait for bit 7 to change

       JNE       L42            ; if bit 7 changed, it's a Hercules

       MOV       AL,MDA         ; if bit 7 didn't change, it's an MDA
       MOV       AH,MDADisplay
       CALL      near ptr FoundDevice
       JMP       SHORT L44

L42:   IN        AL,DX
       MOV       DL,AL          ; DL := value from status port

       MOV       AH,MDADisplay  ; assume it's a monochrome display

       MOV       AL,HGC         ; look for an HGC
       AND       DL,01110000b   ; mask off bits 4 through 6
       JZ        L43

       MOV      AL,HGCPlus      ; look for an HGC+
       CMP      DL,00010000b
       JE       L43             ; jump if it's an HGC+

       MOV       AL,InColor     ; it's an InColor card
       MOV       AH,EGAColorDisplay

L43:   CALL      near ptr FoundDevice

L44:   RET

FindMono         ENDP


;
; Find6845
;
;       This routine detects the presence of the CRTC on an MDA, CGA,
;       or HGC. The technique is to write and read register 0Fh of
;       the chip (Cursor Location Low).  If the same value is read
;       as written, assume the chip is present at the specified
;       port address.
;

Find6845         PROC NEAR      ; Caller:       DX = port addr
                                ; Returns:      cf set if not present
       MOV       AL,0Fh
       OUT       DX,AL          ; select 6845 reg 0Fh (cursor low)

       INC       DX

       IN        AL,DX          ; AL := current Cursor Low value
       MOV       AH,AL          ; preserve in AH
       MOV       AL,66h         ; AL := arbitrary value
       OUT       DX,AL          ; try to write to 6845

       MOV       CX,100h
L51:   LOOP      L51            ; wait for 6845 to respond

       IN        AL,DX
       XCHG      AH,AL          ; AH := returned value
                                ; AL := original value
       OUT       DX,AL          ; restore original value

       CMP       AH,66h         ; test whether 6845 responded
       JE        L52            ; jump if it did (cf is reset)

       STC                      ; set carry flag if no 6845 present

L52:   RET

Find6845         ENDP

;
; FindActive
;
;       This subroutine stores the currently active device as Device0.
;       The current video mode determines which subsystem is active.
;


FindActive       PROC NEAR

       CMP       WORD PTR Device1,0
       JE        L63            ; Exit if only one subsystem

       CMP       Video0Type[DI],4
       JGE       L63                    ; Exit if MCGA or VGA present
       CMP       Video1Type[DI],4       ;  (INT 10h function 1Ah
       JGE       L63                    ;  already did the work)

       MOV       AH,0Fh
       INT       10h                    ; AL := current BIOS video mode

       AND       AL,7
       CMP       AL,7                   ; jump if monochrome
       JE        L61                    ;  (mode 7 or 0Fh)

       CMP       Display0Type[DI],MDADisplay
       JNE       L63                    ; exit if Display0 is color
       JMP       SHORT L62

L61:   CMP       Display0Type[DI],MDADisplay
       JE        L63                    ; exit if Display0 is monochrome

L62:   MOV       AX,Device0             ; make Device0 currently active
       XCHG      AX,Device1
       MOV       Device0,AX

L63:   RET

FindActive       ENDP


;
; FoundDevice
;
;               This routine updates the list of subsystems.
;

FoundDevice      PROC NEAR              ; Caller:       AH = display #
                                        ;               AL = subsystem #
                                        ; Destroys:     BX
       LEA       BX,Video0Type[DI]
       CMP       BYTE PTR [BX],0
       JE        L71                    ; Jump if 1st subsystem

       LEA       BX,Video1Type[DI]      ; Must be 2nd subsystem

L71:   MOV       [BX],AX                ; Update list entry
       RET

FoundDevice      ENDP


;
; Data structures, stashed in code segment.
;
VIDstruct        STRUC          ; Corresponds to C data structure

Video0Type       DB    ?        ; first subsystem type
Display0Type     DB    ?        ; display attached to first subsystem
Video1Type       DB    ?        ; second subsystem type
Display1Type     DB    ?        ; display attached to second subsystem

VIDstruct        ENDS

EGADisplays      DB    CGADisplay       ; 0000b, 0001b (EGA switch values)
                 DB    EGAColorDisplay  ; 0010b, 0011b
                 DB    MDADisplay       ; 0100b, 0101b
                 DB    CGADisplay       ; 0110b, 0111b
                 DB    EGAColorDisplay  ; 1000b, 1001b
                 DB    MDADisplay       ; 1010b, 1011b

DCCtable         DB    0,0              ; Translate table for INT 10h
                 DB    MDA,MDADisplay   ; func 1Ah
                 DB    CGA,CGADisplay
                 DB    0,0
                 DB    EGA,EGAColorDisplay
                 DB    EGA,MDADisplay
                 DB    0,0
                 DB    VGA,PS2MonoDisplay
                 DB    VGA,PS2ColorDisplay
                 DB    0,0
                 DB    MCGA,EGAColorDisplay
                 DB    MCGA,PS2MonoDisplay
                 DB    MCGA,PS2ColorDisplay

TestSequence     DB    ?                ; This list of flags and addresses
                 DW    FindPS2          ; determines the order in which
                                        ; this program looks for the
EGAflag          DB    ?                ; various subsystems
                 DW    FindEGA          ; FindEGA - FORGET EGA TEST!

CGAflag          DB    ?
                 DW    FindCGA          ; FindCGA

Monoflag         DB    ?
                 DW    FindMono

NumberOfTests    EQU   ($-TestSequence)/3

_TEXT   ENDS
                 END
